<?php


class wps_addon_adaptive {


  public function __construct() {

    // Options
    $options = get_option(WPS_IC_SETTINGS);

    $this->register_new_sizes();
    add_action('init', array(&$this, 'register_new_sizes'), 1);

    if ( ! empty($options['adaptive']) && $options['adaptive'] == '1') {
      if (is_admin()) {
        return;
      }
      add_filter('wp_get_attachment_image_attributes', array(&$this, 'srcset_modified'), 25, 3);
    }

    // Edit Htaccess
    add_action('admin_init', array(&$this, 'write_htaccess'));
  }


  public function dd_imagesizes($sizes) {
    /*
     * add_image_size('img-2880', 2880, 9999, false);
    add_image_size('img-2280', 2280, 9999, false);
    add_image_size('img-1440', 1440, 9999, false);
    add_image_size('img-1140', 1140, 9999, false);
    add_image_size('img-720', 720, 9999, false);
    add_image_size('img-652', 652, 9999, false);
    add_image_size('img-570', 570, 9999, false);
    add_image_size('img-360', 360, 9999, false);
    add_image_size('img-180', 180, 9999, false);
     */

    $sizes['img-652'] = 'AI 652px Width';

    return $sizes;
  }


  public function write_htaccess() {
    #delete_option('ic_htaccess_written_v3');
    $htaccess_written  = get_option('ic_htaccess_written_v4');
    $htaccess_writable = $this->is_htaccess_writeable();

    if ( ! $htaccess_written || $htaccess_written == '') {
      if ($htaccess_writable) {
        $htaccess              = ABSPATH . '.htaccess';
        $htaccess_old_contents = file_get_contents($htaccess);

        #var_dump(strpos($htaccess_old_contents, 'WP ImageCompress Start'));
        if (strpos($htaccess_old_contents, 'WP ImageCompress Start') !== false) {
          update_option('ic_htaccess_written_v4', 'true');
          return;
        }

        // Content cache
        $htaccess_block = "#WP ImageCompress Start";
        $htaccess_block .= "\n";
        $htaccess_block .= "<IfModule mod_headers.c>" . "\n";
        $htaccess_block .= "<FilesMatch \"\.(ico|pdf|jpg|jpeg|png|gif|html|htm|xml|txt|xsl|css)$\">" . "\n";
        $htaccess_block .= "Header set Cache-Control \"max-age=172800, public, must-revalidate\"" . "\n";
        $htaccess_block .= "</FilesMatch>" . "\n";
        $htaccess_block .= "</IfModule>" . "\n";
        $htaccess_block .= "\n";
        $htaccess_block .= "<ifModule mod_headers.c>" . "\n";
        $htaccess_block .= "Header set Connection keep-alive" . "\n";
        $htaccess_block .= "</ifModule>" . "\n";
        $htaccess_block .= "\n";

        $htaccess_block .= "\n";
        $htaccess_block .= "<IfModule mod_expires.c>" . "\n";
        $htaccess_block .= "ExpiresActive On" . "\n";
        $htaccess_block .= "ExpiresByType text/css \"access plus 1 month\"" . "\n";
        $htaccess_block .= "ExpiresByType text/javascript \"access plus 1 month\"" . "\n";
        $htaccess_block .= "ExpiresByType application/javascript \"access plus 1 month\"" . "\n";
        $htaccess_block .= "</ifModule>" . "\n";
        $htaccess_block .= "\n";

        // Gzip Block
        $htaccess_block .= "<IfModule mod_deflate.c>" . "\n";
        $htaccess_block .= "<FilesMatch \"\.(js|css|html|php|svg)$\">" . "\n";
        $htaccess_block .= "SetOutputFilter DEFLATE" . "\n";
        $htaccess_block .= "</FilesMatch>" . "\n";
        $htaccess_block .= "</IfModule>" . "\n";
        $htaccess_block .= "\n";
        $htaccess_block .= "#WP ImageCompress End";

        // Copy OldHtaccess
        $copy = copy($htaccess, $htaccess . '_backup');
        if ($copy) {
          $htaccess_new_contents = $htaccess_old_contents . $htaccess_block;
          $write                 = file_put_contents($htaccess, $htaccess_new_contents);
          if ($write) {
            update_option('ic_htaccess_written_v4', 'true');
          }
        }

        flush_rewrite_rules();
      }
    }
  }


  public static function is_htaccess_writeable() {
    $htaccess = ABSPATH . '.htaccess';

    return
      ( ! file_exists($htaccess) && @fopen($htaccess, 'w')) ||
      (file_exists($htaccess) && is_writable($htaccess));
  }


  public function srcset_modified($attr, $attachment, $size) {
    return $attr;
  }


  public function generate_attachment_metadata($attachment_id, $file, $thumbnails = null) {
    if ( ! function_exists('file_is_displayable_image')) {
      require_once(ABSPATH . "wp-admin" . '/includes/image.php');
      require_once(ABSPATH . "wp-admin" . '/includes/file.php');
      require_once(ABSPATH . "wp-admin" . '/includes/media.php');
    }

    $attachment = get_post($attachment_id);
    $meta_datas = get_post_meta($attachment_id, '_wp_attachment_metadata', true);

    $metadata = [];
    if (preg_match('!^image/!', get_post_mime_type($attachment)) && file_is_displayable_image($file)) {
      $imagesize          = getimagesize($file);
      $metadata['width']  = $imagesize[0];
      $metadata['height'] = $imagesize[1];
      list($uwidth, $uheight) = wp_constrain_dimensions($metadata['width'], $metadata['height'], 128, 96);
      $metadata['hwstring_small'] = "height='$uheight' width='$uwidth'";

      // Make the file path relative to the upload dir.
      $metadata['file'] = _wp_relative_upload_path($file);

      // make thumbnails and other intermediate sizes.
      global $_wp_additional_image_sizes;

      foreach (get_intermediate_image_sizes() as $s) {
        $sizes[ $s ] = [
          'width'  => '',
          'height' => '',
          'crop'   => false,
        ];
        if (isset($_wp_additional_image_sizes[ $s ]['width'])) {
          $sizes[ $s ]['width'] = intval($_wp_additional_image_sizes[ $s ]['width']);
        } // For theme-added sizes
        else {
          $sizes[ $s ]['width'] = get_option("{$s}_size_w");
        } // For default sizes set in options
        if (isset($_wp_additional_image_sizes[ $s ]['height'])) {
          $sizes[ $s ]['height'] = intval($_wp_additional_image_sizes[ $s ]['height']);
        } // For theme-added sizes
        else {
          $sizes[ $s ]['height'] = get_option("{$s}_size_h");
        } // For default sizes set in options
        if (isset($_wp_additional_image_sizes[ $s ]['crop'])) {
          $sizes[ $s ]['crop'] = intval($_wp_additional_image_sizes[ $s ]['crop']);
        } // For theme-added sizes
        else {
          $sizes[ $s ]['crop'] = get_option("{$s}_crop");
        } // For default sizes set in options
      }

      $sizes = apply_filters('intermediate_image_sizes_advanced', $sizes);

      // Only if not all sizes.
      if (isset($thumbnails) && is_array($thumbnails) && isset($meta_datas['sizes']) && ! empty($meta_datas['sizes'])) {
        // Fill the array with the other sizes not have to be done.
        foreach ($meta_datas['sizes'] as $name => $fsize) {
          $metadata['sizes'][ $name ] = $fsize;
        }
      }

      foreach ($sizes as $size => $size_data) {
        if (isset($thumbnails)) {
          if ( ! in_array($size, $thumbnails)) {
            continue;
          }
        }

        $resized = image_make_intermediate_size($file, $size_data['width'], $size_data['height'], $size_data['crop']);

        if (isset($meta_datas['size'][ $size ])) {
          // Remove the size from the orignal sizes for after work.
          unset($meta_datas['size'][ $size ]);
        }

        if ($resized) {
          $metadata['sizes'][ $size ] = $resized;
        }
      }

      // fetch additional metadata from exif/iptc.
      $image_meta = wp_read_image_metadata($file);
      if ($image_meta) {
        $metadata['image_meta'] = $image_meta;
      }
    }

    return apply_filters('wp_generate_attachment_metadata', $metadata, $attachment_id);
  }


  public function thumbnail_rebuild($attachment_ID) {

    if ( ! defined('ABSPATH')) {
      /** Set up WordPress environment */
      require_once(dirname(__FILE__) . '/wp-load.php');
    }

    if ( ! function_exists('wp_generate_attachment_metadata')) {
      require_once ABSPATH . 'wp-admin/includes/image.php';
    }

    if ( ! function_exists('download_url')) {
      require_once(ABSPATH . "wp-admin" . '/includes/image.php');
      require_once(ABSPATH . "wp-admin" . '/includes/file.php');
      require_once(ABSPATH . "wp-admin" . '/includes/media.php');
    }

    if ( ! function_exists('update_option')) {
      require_once(ABSPATH . "wp-includes" . '/option.php');
    }

    // Count offset
    $count = get_option('wps_ic_rebuild_offset');
    update_option('wps_ic_rebuild_offset', $count + 1);
    // Get the path.
    $fullsizepath = get_attached_file($attachment_ID);

    // Regen the attachment.
    if (false !== $fullsizepath && file_exists($fullsizepath)) {
      wp_update_attachment_metadata((int)$attachment_ID, wp_generate_attachment_metadata((int)$attachment_ID, $fullsizepath));
      return ['regenrated' => true];
    } else {
      return [
        'src'   => wp_get_attachment_thumb_url($attachment_ID),
        'error' => sprintf(
          __('This file does not exists and have not been regenerated :<br/><a target="_blank" href="%1$s" >%2$s</a>', 'simple-image-sizes'),
          get_edit_post_link($attachment_ID),
          get_the_title($attachment_ID)
        ),
      ];

    }

    return [
      'src'   => wp_get_attachment_thumb_url($attachment_ID),
      'title' => get_the_title($attachment_ID),
    ];
  }


  public function count_attachments() {
    $attachments = get_children(
      [
        'post_type'      => 'attachment',
        'post_mime_type' => 'image',
        'numberposts'    => - 1,
        'post_status'    => null,
        'post_parent'    => null, // any parent
        'output'         => 'ids',
      ]
    );
    // Return the Id's and Title of medias
    wp_send_json(array('total' => count($attachments)));
  }


  public function register_new_sizes() {
    add_image_size('img-2880', 2880, 9999, false);
    add_image_size('img-2280', 2280, 9999, false);
    add_image_size('img-1440', 1440, 9999, false);
    add_image_size('img-1140', 1140, 9999, false);
    add_image_size('img-720', 720, 9999, false);
    add_image_size('img-652', 652, 9999, false);
    add_image_size('img-570', 570, 9999, false);
    add_image_size('img-360', 360, 9999, false);
    add_image_size('img-180', 180, 9999, false);
  }


}