<?php


class wps_addon_cdn {

  public static $uploads_dir;
  public static $domain;
  public static $protocol;


  public function __construct() {

    // Options
    $options = get_option(WPS_IC_SETTINGS);

    if (is_admin()) {
      return;
    }

    add_filter('get_image_tag_class', array(&$this, 'lazy_class_add'));
    add_filter('wp_get_attachment_image_attributes', array(&$this, 'lazy_class_add_att'));
    add_action('the_content', array(&$this, 'add_lazy_class_content'), 5);

    // Get uploads dir
    $uploads_dir        = wp_upload_dir();
    $uploads_dir        = $uploads_dir['baseurl'];
    $uploads_dir        = str_replace(array('https://', 'http://'), '', $uploads_dir);
    $uploads_dir        = str_replace('/', '\/', $uploads_dir);
    $uploads_dir        = rtrim($uploads_dir, '/');
    $this::$uploads_dir = $uploads_dir;

    // Get Sanitized Domain
    $domain = site_url();
    $domain = str_replace(array('https://', 'http://'), '', $domain);
    $domain = sanitize_title($domain);
    $domain = str_replace(array('-'), '', $domain);
    if (strlen($domain) >= 20) {
      $domain = substr($domain, 0, 20);
    }
    $this::$domain = $domain;

    // Get protocol
    $site = site_url();

    if (strpos($site, 'https://') !== false) {
      $this::$protocol = 'https://';
    } else {
      $this::$protocol = 'https://';
    }

    // Run Prior to "wp_calculate_image_srcset"!!!!
    if ( ! empty($options['adaptive_images']) && $options['adaptive_images'] == '1' && !$this->is_speed_test()) {
      add_action('the_content', array(&$this, 'rewrite_the_content_ai'), 10);
    }

    if ( ! empty($options['generate_webp']) && $options['generate_webp'] == '1' && !$this->is_speed_test()) {
      add_action('the_content', array(&$this, 'rewrite_the_content_webp'), 11);
    }

    // Do CDN If active
    if ( ! empty($options['cdn']) && $options['cdn'] == '1') {
      add_action('the_content', array(&$this, 'rewrite_the_content_cdn'), 12);
    }

    // Do Lazy If active
    if ( ! empty($options['lazy']) && $options['lazy'] == '1') {
      add_action('the_content', array(&$this, 'rewrite_the_content_lazy'), 20);
    }

    // Do adaptive images, if active
    if ( ! empty($options['adaptive_images']) && $options['adaptive_images'] == '1') {
      add_filter('wp_calculate_image_srcset', array(&$this, 'replace_image_srcset'), 10, 5);
    }

    add_action('save_post', array(&$this, 'save_post'));
  }


  public function add_lazy_class_content($content) {
    global $post;
    if ( ! empty($post->ID)) {
      delete_transient('ic_images_' . $post->ID);
    }

    $options = get_option(WPS_IC_SETTINGS);
    if ( ! empty($options['lazy']) && $options['lazy'] == '1') {
      $content = preg_replace_callback("/<img([^>]*)class=[\"\']([^\"\'\\/][^\"\']*)[\"\']/", array('self', 'add_lazy_class'), $content);
    }

    return $content;
  }


  public static function add_lazy_class($match) {
    global $post;
    if ( ! empty($post->ID)) {
      delete_transient('ic_images_' . $post->ID);
    }

    $options = get_option(WPS_IC_SETTINGS);
    if ( ! empty($options['lazy']) && $options['lazy'] == '1') {
      $class = $match[2];
      $class = $class . ' lazy';
      $img   = str_replace($match[2], $class, $match[0]);
    } else {
      $img = $match[2];
    }

    return $img;
  }


  public function lazy_class_add_att($attr) {
    global $post;
    if ( ! empty($post->ID)) {
      delete_transient('ic_images_' . $post->ID);
    }

    $options = get_option(WPS_IC_SETTINGS);
    if ( ! empty($options['lazy']) && $options['lazy'] == '1') {
      $attr['class'] = $attr['class'] . ' lazy';
    }

    return $attr;
  }


  public function lazy_class_add($class) {
    global $post;
    if ( ! empty($post->ID)) {
      delete_transient('ic_images_' . $post->ID);
    }

    $options = get_option(WPS_IC_SETTINGS);
    if ( ! empty($options['lazy']) && $options['lazy'] == '1') {
      $class .= ' lazy';
    }

    return $class;
  }


  public function save_post($post_id) {
    delete_post_meta($post_id, '_ic_sources');
    delete_post_meta($post_id, '_ic_cdn_content');
  }


  public function replace_image_srcset($sources, $size_array, $image_src, $image_meta, $attachment_id) {
    global $post, $current_screen;

    $options = get_option(WPS_IC_SETTINGS);

    // TODO: Maybe we need this?
    /*
    if (count($sources) <= 1) {
      return $sources;
    }
    */

    $image_path = get_attached_file($attachment_id);
    $image = wp_get_attachment_image_src($attachment_id, 'full');
    $image = $image[0];

    $full_image_dim = getimagesize($image_path);

    if ($full_image_dim[0] >= 3000 || $full_image_dim[1] >= 3000) {
			$sources['3000']['url']        = $image;
			$sources['3000']['descriptor'] = 'w';
			$sources['3000']['value']      = 3000;
		}

    if ( ! empty($current_screen) && $current_screen->base == 'upload') {
      return $sources;
    }

    $attachment_sources_cdn = get_post_meta($attachment_id, '_ic_sources', true);
    $compress               = get_post_meta($attachment_id, 'wps_ic_data', true);

    if ( ! $compress || $compress == 'not_able') {
      return $sources;
    }

    if ( ! $attachment_sources_cdn) {
      foreach ($sources as $key => $data) {
        preg_match('/(http(s?):)\/\/' . $this::$uploads_dir . '\/\d{4}\/\d{2}\/([a-zA-Z0-9\-\_\.]+)/', $data['url'], $matches);

        if ( ! empty($options['cdn']) && $options['cdn'] == '1') {
          $sources[ $key ]['url'] = str_replace($data['url'], $this::$protocol . $this::$domain . '.wpcompress.xyz/' . $matches[3], $data['url']);
        } else {
          $sources[ $key ]['url'] = $data['url'];
        }

      }
      #update_post_meta($attachment_id, '_ic_sources', $sources);

      return $sources;
    } else {
      return $attachment_sources_cdn;
    }
  }


  public static function webp_generator($input) {
    $webp = str_replace(array('.jpg', '.jpeg', '.png'), '.webp', $input);

    if (!file_exists($webp)) {
      return false;
    }

    return $webp;
  }


  public static function webp_exists($file) {
    $attachmentID = self::attachment_url_to_id($file);
    $options      = get_option(WPS_IC_SETTINGS);

    if ( ! $attachmentID) {
      return false;
    }

    $filedata = get_attached_file($attachmentID);
    $webp     = str_replace(array('jpg', 'jpeg', 'png'), 'webp', $filedata);

    if (file_exists($webp)) {
      $file  = wp_get_attachment_image_src($attachmentID, 'full');
      $file  = $file[0];
      $image = basename($file);

      if ( ! empty($options['cdn']) && $options['cdn'] == '1') {
        $file = str_replace($file, self::$protocol . self::$domain . '.wpcompress.xyz/' . $image, $file);
        # var_dump($file);
        #var_dump(str_replace(array('jpg', 'jpeg', 'png'), 'webp', $file));
      }

      return str_replace(array('jpg', 'jpeg', 'png'), 'webp', $file);
    } else {
      return false;
    }
  }


  public function rewrite_the_content_cdn($html) {
    return self::rewrite_cdn($html);
  }


  public function rewrite_the_content_lazy($html) {
    return self::rewrite_lazy($html);
  }


  public function rewrite_the_content_webp($html) {
    return self::rewrite_webp($html);
  }


  public function rewrite_the_content_ai($html) {
    return self::rewrite_ai($html);
  }


  public static function get_attributes($match) {
    if (function_exists("mb_convert_encoding")) {
      $image_node = mb_convert_encoding($match, 'HTML-ENTITIES', 'UTF-8');
    }

    $dom = new DOMDocument();
    @$dom->loadHTML($image_node);
    $image      = $dom->getElementsByTagName('img')->item(0);
    $attributes = array();
    foreach ($image->attributes as $attr) {
      $attributes[ $attr->nodeName ] = $attr->nodeValue;
    }

    return $attributes;
  }


  public static function setup_lazy($img, $type) {

    return array(
      'value'  =>
        (isset($img[ 'data-' . $type ]) && strlen($img[ 'data-' . $type ])) ?
          $img[ 'data-' . $type ]
          : (isset($img[ 'data-' . $type ]) && strlen($img[ 'data-' . $type ]) ?
          $img[ 'data-' . $type ]
          : (isset($img[ $type ]) && strlen($img[ $type ]) ? $img[ $type ] : false)),
      'prefix' =>
        (isset($img[ 'data-' . $type ]) && strlen($img[ 'data-' . $type ])) ? 'data-'
          : (isset($img[ 'data-' . $type ]) && strlen($img[ 'data-' . $type ]) ? 'data-'
          : (isset($img[ $type ]) && strlen($img[ $type ]) ? '' : false))
    );
  }


  public static function get_image_size($url) {
    preg_match("/([0-9]+)x([0-9]+)\.[a-zA-Z0-9]+/", $url, $matches); //the filename suffix way
    if (isset($matches[1]) && isset($matches[2])) {
      $sizes = array($matches[1], $matches[2]);
    } elseif ( ! $sizes = self::url_to_path_to_sizes($url)) { //the file way
      $sizes = self::url_to_metadata_to_sizes($url);//the DB way
    }

    return $sizes;
  }


  public static function url_to_path_to_sizes($image_url) {
    $updir          = wp_upload_dir();
    $baseUrlPattern = "/" . str_replace("/", "\/", preg_replace("/^http[s]{0,1}:\/\//", "^http[s]{0,1}://", $updir['baseurl'])) . "/";

    //if(strpos($image_url, 'feat1')) file_put_contents("/tmp/shortpixelai_log", "\n\nMATCH? $image_url $baseUrlPattern " . json_encode($updir), FILE_APPEND);

    if (preg_match($baseUrlPattern, $image_url)) {
      $path = preg_replace($baseUrlPattern, $updir['basedir'], $image_url);
    } elseif ($image_url[0] == '/') {
      $path = dirname(dirname($updir['basedir'])) . $image_url;
    } else {
      $path = dirname(dirname($updir['basedir'])) . '/' . $image_url;
    }

    //if(strpos($image_url, 'feat1')) file_put_contents("/tmp/shortpixelai_log", "\n\nUUUUNDDD? $path", FILE_APPEND );

    if (file_exists($path)) {
      return getimagesize($path);
    }

    return false;
  }


  public static function url_to_metadata_to_sizes($image_url) {
    // Thx to https://github.com/kylereicks/picturefill.js.wp/blob/master/inc/class-model-picturefill-wp.php
    global $wpdb;
    $prefix             = $wpdb->prefix;
    $sql                = "SELECT m.meta_value FROM {$prefix}posts p INNER JOIN {$prefix}postmeta m on p.id = m.post_id WHERE m.meta_key = '_wp_attachment_metadata' AND ";
    $original_image_url = $image_url;
    $image_url          = preg_replace('/^(.+?)(-\d+x\d+)?\.(jpg|jpeg|png|gif)((?:\?|#).+)?$/i', '$1.$3', $image_url);
    $meta               = $wpdb->get_var($wpdb->prepare("$sql p.guid='%s';", $image_url));

    //try the other proto (https - http) if full urls are used
    if (empty($meta) && strpos($image_url, 'http://') === 0) {
      $image_url_other_proto = strpos($image_url, 'https') === 0 ?
        str_replace('https://', 'http://', $image_url) :
        str_replace('http://', 'https://', $image_url);
      $meta                  = $wpdb->get_var($wpdb->prepare("$sql p.guid='%s';", $image_url_other_proto));
    }

    //try using only path
    if (empty($meta)) {
      $image_path = parse_url($image_url, PHP_URL_PATH); //some sites have different domains in posts guid (site changes, etc.)
      //keep only the last two elements of the path because some CDN's add path elements in front ( Google Cloud adds the project name, etc. )
      $image_path_elements = explode('/', $image_path);
      $image_path_elements = array_slice($image_path_elements, max(0, count($image_path_elements) - 3));
      $meta                = $wpdb->get_var($wpdb->prepare("$sql p.guid like'%%%s';", implode('/', $image_path_elements)));
    }

    //try using the initial URL
    if (empty($meta)) {
      $meta = $wpdb->get_var($wpdb->prepare("$sql p.guid='%s';", $original_image_url));
    }

    if ( ! empty($meta)) { //get the sizes from meta
      $meta = unserialize($meta);
      if (preg_match("/" . preg_quote($meta['file'], '/') . "$/", $original_image_url)) {
        return array($meta['width'], $meta['height']);
      }
      foreach ($meta['sizes'] as $size) {
        if ($size['file'] == wp_basename($original_image_url)) {
          return array($size['width'], $size['height']);
        }
      }
    }

    return array(1, 1);
  }


  public function find_thumb_size($size) {
    $thumb_name = '';
    $thumb_size = '';
    $sizes      = array(180, 360, 570, 652, 720, 1140, 1440, 2280, 2880);
    foreach ($sizes as $k => $s) {
      if ($s == $size) {
        $thumb_name = 'img-' . $sizes[ $k ];
        $thumb_size = $sizes[ $k ];
        break;
      } else {
        if ($size >= $s) {
          if (isset($sizes[ $k + 1 ])) {
            $thumb_name = 'img-' . $sizes[ $k + 1 ];
            $thumb_size = $sizes[ $k + 1 ];
          } else {
            $thumb_name = 'img-' . $sizes[ $k ];
            $thumb_size = $sizes[ $k ];
          }
        }
      }
    }

    return array('thumb_name' => $thumb_name, 'size' => $thumb_size);
  }


  public function find_retina_size($non_retina_input) {
    $sizes = array(180 => 360, 360 => 720, 570 => 1140, 652 => 1140, 720 => 1440, 1140 => 2280, 1440 => 2280, 2280 => 2280, 2880 => 2880);

    $non_retina_input = $non_retina_input * 2;
    foreach ($sizes as $non_retina => $retina) {
      if ($retina >= $non_retina_input) {
        return 'img-' . $retina;
      }
    }

    return 'full';
  }


  public function convert_image_webp($match) {
    global $post;

    $options      = get_option(WPS_IC_SETTINGS);
    $url          = preg_match('@src="([^"]+)"@', $match[0], $match_url);
    $attachmentID = self::attachment_url_to_id($match_url[1]);
    $compressed   = get_post_meta($attachmentID, 'wps_ic_data', true);
    if (empty($compressed) || $compressed == 'not_able') {
      return $match[0];
    }

    $img = self::get_attributes($match[0]);

    if (empty($img['srcset'])) {
      $img['srcset'] = self::get_srcset($attachmentID);
    }

    if (empty($img['sizes'])) {
      $max_width = self::get_max_width($attachmentID, $match_url[1]);
      $img['sizes'] = '(max-width: ' . $max_width . ') 100vw, ' . $max_width . '';
    } else {
      $max_width = explode(',', $img['sizes']);
      $max_width = trim($max_width[1]);
    }

    $lazy   = '';
    $src    = $img['src'];
    $srcset = $img['srcset'];

    $screen_width     = $_COOKIE[ "ic_window_resolution_" . $post->ID ];
    $max_width_string = str_replace('px', '', $max_width);

    if (empty($screen_width)) {
      $screen_width = 1680;
    }

    if ($max_width_string < $screen_width) {
      $screen_width = $max_width_string;
    }

    $sizes = explode(',', $srcset);
    if (is_array($sizes)) {

      $order_sizes = array();
      foreach ($sizes as $key => $size) {
        $size                    = trim($size);
        $size                    = explode(' ', $size);
        $size[0]                 = trim($size[0]);
        $size[1]                 = trim($size[1]);
        $size[1]                 = str_replace('w', '', $size[1]);
        $order_sizes[ $size[1] ] = $size[0];
      }

      ksort($order_sizes);

      foreach ($order_sizes as $key => $image) {
        $size     = $key;
        $width    = $size;
        $width_s  = str_replace('w', '', $width);
        $width_px = str_replace('w', 'px', $width);

        if ($width_s >= $screen_width) {
          $selected_size   = $image;
          $selected_size   = trim($selected_size);
          $non_retina_size = $width_s;
          $image           = $selected_size;
          break;
        }

      }

    }

    $webp = self::webp_generator($image);

    $webp_srcset = $img['srcset'];
    if ( $webp && ! empty($webp_srcset)) {
      $webp_srcset = str_replace(array('.png', '.jpg'), '.webp', $webp_srcset);
    }


    if (!empty($_GET['ic_debug_img'])) {
      var_dump($match_url[1]);
      var_dump($img['sizes']);
      var_dump($max_width);
    }


    if ( ! $webp) {
      return '<img src="' . $img['src'] . '" class="' . $img['class'] . ' ' . $lazy . '" data-src="' . $image . '" data-srcset="' . $img['srcset'] . '" alt="' . $img['alt'] . '" sizes="' . $img['sizes'] . '"/>';
    } else {
      return '<picture class="' . $lazy . '">
      <source data-srcset="' . $webp_srcset . '" class="' . $img['class'] . '" sizes="' . $img['sizes'] . '" type="image/webp"> 
      <img src="' . $img['src'] . '" data-src="' . $image . '" data-srcset="' . $img['srcset'] . '" alt="' . $img['alt'] . '" class="' . $img['class'] . '" sizes="' . $img['sizes'] . '"/>
      </picture>';
    }

  }


  public static function rewrite_to_cdn($input) {
    $upload_dir = wp_upload_dir();
    $upload_dir = $upload_dir['baseurl'];
    $output     = '';

    if (strpos($input, $upload_dir) !== false) {

      $input      = preg_replace('#' . $upload_dir . '\/\d{4}\/\d{2}\/#', '', $input);
      $upload_dir = preg_replace('#' . $upload_dir . '\/\d{4}\/\d{2}\/#', '', $upload_dir);

      // http://dev55.brickrdev.com/wp-content/uploads/2019/06/g-0dp2ycCw0-2-1024x683.jpg 1024w,
      $urls = explode(',', $input);

      if (is_array($urls)) {
        foreach ($urls as $key => $url) {
          $url = trim($url);

          $cdn_url = self::$protocol . self::$domain . '.wpcompress.xyz/';
          $output  .= $cdn_url . $url . ',';
        }
      }

      $output = rtrim($output, ',');

      #var_dump($output);
      return $output;
    } else {
      return $input;
    }
  }


  public static function get_srcset($attachmentID) {
    $srcset = '';

		$image_path = get_attached_file($attachmentID);

		if (preg_match('/\.svg/', $image_path, $matches)) {
			$image  = wp_get_attachment_image_src($attachmentID, $size);
			$image  = $image[0];
			$srcset .= $image;
			return $srcset;
		}

		$image_dim = getimagesize($image_path);

		$max_width = 2880;
		if ($image_dim[0] < $max_width) {
			$max_width = $image_dim[0];
		}

		$image = wp_get_attachment_image_src($attachmentID, 'full');
		$srcset .= $image[0] . ' ' . $max_width . 'w,';

		if ($image_dim[0] > 180) {
			$sizes = array(2880, 2280, 1440, 1140, 720, 652, 570, 360, 180);
			foreach ($sizes as $key => $size) {
				if ($size > $max_width) {
					continue;
				}
				$image  = wp_get_attachment_image_src($attachmentID, 'img-' . $size);
				$image  = $image[0];
				$srcset .= $image . ' ' . $size . 'w,';
			}
		}

    $sizes = get_intermediate_image_sizes();
    foreach ($sizes as $k => $size) {
      if ($size == 'thumbnail') {
        $image  = wp_get_attachment_image_src($attachmentID, $size);
        $image  = $image[0];
        $srcset .= $image . ' 150w,';
        break;
      }
    }

    $srcset = rtrim($srcset, ',');

    return $srcset;
  }


  public static function get_max_width($attachmentID, $image) {
    $max_width = '';

    $sizes = get_intermediate_image_sizes();

    if ($sizes) {
      foreach ($sizes as $k => $size) {
        $image_search = wp_get_attachment_image_src($attachmentID, $size);
        $image_search = $image_search[0];

        if ($image == $image_search) {
          $width = get_option("{$size}_size_w");
          if ( ! $width) {
            $width = get_option("img-1440_size_w");
          }

          $max_width = $width . 'px';
          break;
        }
      }
    }

    $max_width = trim($max_width);

    if (empty($max_width) || $max_width == 'px') {

      if (preg_match('/\-(\d*[x]\d*)\./', $image, $matches)) {
        $size = $matches[1]; // i.e. 250x250
        $size = explode('x',$size);
        $max_width = $size[0].'px';
      } else {
        $max_width = '1440px';
      }
    }

    return $max_width;
  }


  public function convert_image_cdn($match) {
    global $post;

    $options = get_option(WPS_IC_SETTINGS);
    $url     = preg_match('@src="([^"]+)"@', $match[0], $match_url);
    #var_dump($match_url[1]);
		$attachmentID = self::attachment_url_to_id($match_url[1]);
		$compressed = get_post_meta($attachmentID, 'wps_ic_data', true);

		if (empty($compressed) || $compressed == 'not_able') {
			return $match[0];
		}

    if (preg_match('/\.svg/', $match[0], $matches)) {
      return $match[0];
    }

    $html     = $match[0];
    $site_url = site_url();

    $regex_rule = '/(http(s?):)\/\/' . self::$uploads_dir . '\/\d{4}\/\d{2}\//';
    $html       = preg_replace($regex_rule, self::$protocol . self::$domain . '.wpcompress.xyz/', $html);

    return $html;
  }


  public function convert_image_lazy($match) {
    global $post;

    $options      = get_option(WPS_IC_SETTINGS);
    $class          = preg_match('@class="([^"]+)"@', $match[0], $match_class);
    $url          = preg_match('@src="([^"]+)"@', $match[0], $match_url);
    $attachmentID = self::attachment_url_to_id($match_url[1]);

    $is_standalone_image = preg_match('/attachment/', $match_class[1], $standalone_image);
    if (!$standalone_image && $standalone_image[0] == '') {
      return $match[0];
    }

    $sizes_img = self::get_image_size($match_url[1]);

    $img = self::get_attributes($match[0]);
    if ( ! empty($img['sizes'])) {
      $max_width = explode(',', $img['sizes']);
      $max_width = trim($max_width[1]);
    } else {
      // Smallest - thumb
      #$max_width = '150px';
      $max_width = $img['width'] . 'px';
    }

    $srcInfo = self::setup_lazy($img, 'src');
    $src     = $srcInfo['value'];

    $srcsetInfo = self::setup_lazy($img, 'srcset');
    $srcset     = $srcsetInfo['value'];

    $srcsetInfo = self::setup_lazy($img, 'sizes');
    $sizes_src  = $srcsetInfo['value'];

    if (empty($srcset)) {
      $srcset = self::get_srcset($attachmentID);
    }

    if (empty($max_width)) {
      $max_width = self::get_max_width($attachmentID, $match_url[1]);
    }

    #echo '<h1>' . $_COOKIE["ic_window_resolution_4"] . '</h1>';
    $screen_width     = $_COOKIE[ "ic_window_resolution_" . $post->ID ];
    $max_width_string = str_replace('px', '', $max_width);

    if (empty($screen_width)) {
      $screen_width = 1680;
    }

    if ($max_width_string < $screen_width) {
      $screen_width = $max_width_string;
    }

    $sizes = explode(',', $srcset);
    if (is_array($sizes)) {

      $order_sizes = array();
      foreach ($sizes as $key => $size) {
        $size                    = trim($size);
        $size                    = explode(' ', $size);
        $size[0]                 = trim($size[0]);
        $size[1]                 = trim($size[1]);
        $size[1]                 = str_replace('w', '', $size[1]);
        $order_sizes[ $size[1] ] = $size[0];
      }

      ksort($order_sizes);

      foreach ($order_sizes as $key => $image) {
        $size     = $key;
        $width    = $size;
        $width_s  = str_replace('w', '', $width);
        $width_px = str_replace('w', 'px', $width);

        if ($width_s >= $screen_width) {
          $selected_size   = $image;
          $selected_size   = trim($selected_size);
          $non_retina_size = $width_s;
          $image           = $selected_size;
          break;
        }

      }

    }

    $svg = 'data:image/svg+xml;u=aHR0cHMlM0ElMkYlMkZ3b3JkcHJlc3MtMTY4NTc4LTg5MzQ3Mi5jbG91ZHdheXNhcHBzLmNvbSUyRndwLWNvbnRlbnQlMkZ1cGxvYWRzJTJGMjAxOSUyRjA2JTJGQVhfVldjN09Sd1kuanBn;w=' . $sizes_img[0] . ';h=' . $sizes_img[1] . ';base64,
    PHN2ZyB2aWV3Qm94PSIwIDAgMjU2MCAyMDQ4IiB3aWR0aD0iMjU2MCIgaGVpZ2h0PSIyMDQ4IiB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciPjwvc3ZnPg==';

    return '<img src="' . $svg . '" data-src="' . $image . '" data-srcset="' . $srcset . '" alt="' . $img['alt'] . '" class="' . $img['class'] . ' lazy" sizes="' . $sizes_src . '"/>';

  }


  public function convert_image_ai($match) {
    global $post;

    $options      = get_option(WPS_IC_SETTINGS);
    $url          = preg_match('@src="([^"]+)"@', $match[0], $match_url);
    $attachmentID = self::attachment_url_to_id($match_url[1]);

    $compressed = get_post_meta($attachmentID, 'wps_ic_data', true);
    if (empty($compressed) || $compressed == 'not_able') {
      return $match[0];
    }

    $sizes_img = self::get_image_size($match_url[1]);

    $img = self::get_attributes($match[0]);

    if ( ! empty($img['sizes'])) {
      $max_width = explode(',', $img['sizes']);
      $max_width = trim($max_width[1]);
    } else {
      // Smallest - thumb
      #$max_width = '150px';
      $max_width = $img['width'] . 'px';
    }


    if (empty($srcset)) {
      $srcset = self::get_srcset($attachmentID);
    }

    if (empty($max_width)) {
      $max_width = self::get_max_width($attachmentID, $match_url[1]);
    }

    $screen_width     = $_COOKIE[ "ic_window_resolution_" . $post->ID ];
    $max_width_string = str_replace('px', '', $max_width);

    if (empty($screen_width)) {
      $screen_width = 1380;
    }

    if ($max_width_string < $screen_width) {
      $screen_width = $max_width_string;
    }

    $sizes = explode(',', $srcset);
    if (is_array($sizes)) {

      $order_sizes = array();
      foreach ($sizes as $key => $size) {
        $size                    = trim($size);
        $size                    = explode(' ', $size);
        $size[0]                 = trim($size[0]);
        $size[1]                 = trim($size[1]);
        $size[1]                 = str_replace('w', '', $size[1]);
        $order_sizes[ $size[1] ] = $size[0];
      }

      ksort($order_sizes);

      foreach ($order_sizes as $key => $image) {
        $size     = $key;
        $width    = $size;
        $width_s  = str_replace('w', '', $width);
        $width_px = str_replace('w', 'px', $width);

        if ($width_s >= $screen_width) {
          $selected_size   = $image;
          $selected_size   = trim($selected_size);
          $non_retina_size = $width_s;
          $image           = $selected_size;
          break;
        }

      }
    }

    return '<img src="' . $img['src'] . '" data-src="' . $img['src'] . '" data-srcset="' . $srcset . '" alt="' . $img['alt'] . '" class="' . $img['class'] . '" sizes="' . $img['sizes'] . '"/>';

  }


  public function is_speed_test() {
    $ip_list = array(
      0   => '5.172.196.188',
      1   => '13.232.220.164',
      2   => '23.22.2.46',
      3   => '23.83.129.219',
      4   => '23.111.152.74',
      5   => '23.111.159.174',
      6   => '27.122.14.7',
      7   => '37.252.231.50',
      8   => '43.225.198.122',
      9   => '43.229.84.12',
      10  => '46.20.45.18',
      11  => '46.165.195.139',
      12  => '46.246.122.10',
      13  => '50.16.153.186',
      14  => '50.23.28.35',
      15  => '52.0.204.16',
      16  => '52.24.42.103',
      17  => '52.48.244.35',
      18  => '52.52.34.158',
      19  => '52.52.95.213',
      20  => '52.52.118.192',
      21  => '52.57.132.90',
      22  => '52.59.46.112',
      23  => '52.59.147.246',
      24  => '52.62.12.49',
      25  => '52.63.142.2',
      26  => '52.63.164.147',
      27  => '52.63.167.55',
      28  => '52.67.148.55',
      29  => '52.73.209.122',
      30  => '52.89.43.70',
      31  => '52.194.115.181',
      32  => '52.197.31.124',
      33  => '52.197.224.235',
      34  => '52.198.25.184',
      35  => '52.201.3.199',
      36  => '52.209.34.226',
      37  => '52.209.186.226',
      38  => '52.210.232.124',
      39  => '54.68.48.199',
      40  => '54.70.202.58',
      41  => '54.94.206.111',
      42  => '64.237.49.203',
      43  => '64.237.55.3',
      44  => '66.165.229.130',
      45  => '66.165.233.234',
      46  => '72.46.130.18',
      47  => '72.46.130.44',
      48  => '76.72.167.90',
      49  => '76.72.167.154',
      50  => '76.72.172.208',
      51  => '76.164.234.106',
      52  => '76.164.234.170',
      53  => '81.17.62.205',
      54  => '82.103.136.16',
      55  => '82.103.139.165',
      56  => '82.103.145.126',
      57  => '83.170.113.210',
      58  => '85.195.116.134',
      59  => '89.163.146.247',
      60  => '89.163.242.206',
      61  => '94.75.211.73',
      62  => '94.75.211.74',
      63  => '94.247.174.83',
      64  => '95.141.32.46',
      65  => '95.211.198.87',
      66  => '96.47.225.18',
      67  => '103.47.211.210',
      68  => '104.129.24.154',
      69  => '104.129.30.18',
      70  => '109.123.101.103',
      71  => '138.219.43.186',
      72  => '148.72.170.233',
      73  => '148.72.171.17',
      74  => '151.106.52.134',
      75  => '162.218.67.34',
      76  => '168.1.92.58',
      77  => '169.51.2.22',
      78  => '169.56.174.147',
      79  => '172.241.112.86',
      80  => '173.248.147.18',
      81  => '173.254.206.242',
      82  => '174.34.156.130',
      83  => '175.45.132.20',
      84  => '178.255.152.2',
      85  => '178.255.153.2',
      86  => '178.255.155.2',
      87  => '179.50.12.212',
      88  => '184.75.208.210',
      89  => '184.75.209.18',
      90  => '184.75.210.90',
      91  => '184.75.210.226',
      92  => '184.75.214.66',
      93  => '185.39.146.214',
      94  => '185.39.146.215',
      95  => '185.70.76.23',
      96  => '185.93.3.92',
      97  => '185.136.156.82',
      98  => '185.152.65.167',
      99  => '185.180.12.65',
      100 => '185.246.208.82',
      101 => '188.172.252.34',
      102 => '199.87.228.66',
      103 => '201.33.21.5',
      104 => '207.244.80.239',
      105 => '209.58.139.193',
      106 => '209.58.139.194',
      107 => '78.0.31.33'
    );

    $pingdom  = strpos(strtolower($_SERVER['HTTP_USER_AGENT']), 'pingdom');
    $gtmetrix = strpos(strtolower($_SERVER['HTTP_USER_AGENT']), 'gtmetrix');

    if (!empty($_GET['simulate_test'])) {
      return true;
    }

    if ($pingdom !== false) {
      $fp = fopen(WPS_IC_DIR . 'test-pingdom.txt', 'a');
      fwrite($fp, 'Pingdom ' . $pingdom);

      return true;
    }

    if ($gtmetrix !== false) {
      $fp = fopen(WPS_IC_DIR . 'test-gtmetrix.txt', 'a');
      fwrite($fp, 'GTMetrix ' . $gtmetrix);

      return true;
    }

    $userIP = $_SERVER['REMOTE_ADDR'];
    if (in_array($userIP, $ip_list)) {
      return true;
    } else {
      return false;
    }
  }


  public function rewrite_ai($html) {
    global $post;

    if ($this->is_speed_test() || ! empty($_GET['simulate_test'])) {

      $options     = get_option(WPS_IC_SETTINGS);
      $output      = '';
      $image_count = 0;
      $thumb_size  = 'large';
      preg_match_all('/<img[^>]*>/', $html, $matches);

      if ($matches) {
        foreach ($matches as $key => $match) {
          foreach ($match as $k => $image) {
            $image_count ++;

            $find = $image;
            $img  = self::get_attributes($image);

            $url          = preg_match('@src="([^"]+)"@', $image, $match_url);
            $attachmentID = self::attachment_url_to_id($match_url[1]);
            $sizes_img    = self::get_image_size($match_url[1]);
            #$compressed   = get_post_meta($attachmentID, 'wps_ic_data', true);

            $svg = 'data:image/svg+xml;u=aHR0cHMlM0ElMkYlMkZ3b3JkcHJlc3MtMTY4NTc4LTg5MzQ3Mi5jbG91ZHdheXNhcHBzLmNvbSUyRndwLWNvbnRlbnQlMkZ1cGxvYWRzJTJGMjAxOSUyRjA2JTJGQVhfVldjN09Sd1kuanBn;w=' . $sizes_img[0] . ';h=' . $sizes_img[1] . ';base64,
    PHN2ZyB2aWV3Qm94PSIwIDAgMjU2MCAyMDQ4IiB3aWR0aD0iMjU2MCIgaGVpZ2h0PSIyMDQ4IiB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciPjwvc3ZnPg==';

            if ($image_count <= 2) {

              $smallest = wp_get_attachment_image_src($attachmentID, $thumb_size);
              $smallest = $smallest[0];
              if ( ! empty($options['cdn']) && $options['cdn'] == '1') {
                $smallest = self::rewrite_to_cdn($smallest);
              }

              $replace = '<img src="' . $svg . '" class="' . $img['class'] . '" data-src="' . $smallest . '"/>';
              $html    = str_replace($find, $replace, $html);
            } else {
              $replace = '<img src="' . $svg . '" class="' . $img['class'] . '"/>';
              $html    = str_replace($find, $replace, $html);
            }

          }
        }
      }

      $content = $html;
    } else {
      $content = preg_replace_callback('/<img[^>]*>/', array('self', 'convert_image_ai'), $html);
    }

    return $content;
  }


  public function rewrite_cdn($html) {
    global $post;
    $content = preg_replace_callback('/<img[^>]*>/', array('self', 'convert_image_cdn'), $html);
    $content = preg_replace_callback('/<source[^>]*>/', array('self', 'convert_image_cdn'), $content);

    return $content;
  }


  public function rewrite_lazy($html) {
    global $post;

    if ($this->is_speed_test()) {

      $options     = get_option(WPS_IC_SETTINGS);
      $output      = '';
      $image_count = 0;
      $thumb_size  = 'large';
      preg_match_all('/<img[^>]*>/', $html, $matches);

      if ($matches) {
        foreach ($matches as $key => $match) {
          foreach ($match as $k => $image) {
            $image_count ++;

            $find = $image;
            $img  = self::get_attributes($image);

            $url          = preg_match('@src="([^"]+)"@', $image, $match_url);
            $attachmentID = self::attachment_url_to_id($match_url[1]);
            $sizes_img    = self::get_image_size($match_url[1]);

            $svg = 'data:image/svg+xml;u=aHR0cHMlM0ElMkYlMkZ3b3JkcHJlc3MtMTY4NTc4LTg5MzQ3Mi5jbG91ZHdheXNhcHBzLmNvbSUyRndwLWNvbnRlbnQlMkZ1cGxvYWRzJTJGMjAxOSUyRjA2JTJGQVhfVldjN09Sd1kuanBn;w=' . $sizes_img[0] . ';h=' . $sizes_img[1] . ';base64,
    PHN2ZyB2aWV3Qm94PSIwIDAgMjU2MCAyMDQ4IiB3aWR0aD0iMjU2MCIgaGVpZ2h0PSIyMDQ4IiB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciPjwvc3ZnPg==';

            if ($image_count <= 2) {

              $smallest = wp_get_attachment_image_src($attachmentID, $thumb_size);
              $smallest = $smallest[0];
              if ( ! empty($options['cdn']) && $options['cdn'] == '1') {
                $smallest = self::rewrite_to_cdn($smallest);
              }

              $replace = '<img src="' . $svg . '" class="' . $img['class'] . '" data-src="' . $smallest . '"/>';
              $html    = str_replace($find, $replace, $html);
            } else {
              $replace = '<img src="' . $svg . '" class="' . $img['class'] . '"/>';
              $html    = str_replace($find, $replace, $html);
            }

          }
        }
      }

      $content = $html;
    } else {
      $content = preg_replace_callback('/<img[^>]*>/', array('self', 'convert_image_lazy'), $html);
    }

    return $content;
  }


  public function rewrite_webp($html) {
    global $post;

    if ($this->is_speed_test()) {

      $options     = get_option(WPS_IC_SETTINGS);
      $output      = '';
      $image_count = 0;
      $thumb_size  = 'large';
      preg_match_all('/<img[^>]*>/', $html, $matches);

      if ($matches) {
        foreach ($matches as $key => $match) {
          foreach ($match as $k => $image) {
            $image_count ++;

            $find = $image;
            $img  = self::get_attributes($image);

            $url          = preg_match('@src="([^"]+)"@', $image, $match_url);
            $attachmentID = self::attachment_url_to_id($match_url[1]);
            $sizes_img    = self::get_image_size($match_url[1]);

            $svg = 'data:image/svg+xml;u=aHR0cHMlM0ElMkYlMkZ3b3JkcHJlc3MtMTY4NTc4LTg5MzQ3Mi5jbG91ZHdheXNhcHBzLmNvbSUyRndwLWNvbnRlbnQlMkZ1cGxvYWRzJTJGMjAxOSUyRjA2JTJGQVhfVldjN09Sd1kuanBn;w=' . $sizes_img[0] . ';h=' . $sizes_img[1] . ';base64,
    PHN2ZyB2aWV3Qm94PSIwIDAgMjU2MCAyMDQ4IiB3aWR0aD0iMjU2MCIgaGVpZ2h0PSIyMDQ4IiB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciPjwvc3ZnPg==';

            if ($image_count <= 2) {

              $smallest = wp_get_attachment_image_src($attachmentID, $thumb_size);
              $smallest = $smallest[0];
              if ( ! empty($options['cdn']) && $options['cdn'] == '1') {
                $smallest = self::rewrite_to_cdn($smallest);
              }

              $replace = '<img src="' . $svg . '" class="' . $img['class'] . '" data-src="' . $smallest . '"/>';
              $html    = str_replace($find, $replace, $html);
            } else {
              $replace = '<img src="' . $svg . '" class="' . $img['class'] . '"/>';
              $html    = str_replace($find, $replace, $html);
            }

          }
        }
      }

      $content = $html;
    } else {
      $content = preg_replace_callback('/<img[^>]*>/', array('self', 'convert_image_webp'), $html);
    }

    return $content;
  }


  public static function rewrite($html) {
    global $post;

    $ic_cdn_content = get_post_meta($post->ID, '_ic_cdn_content', true);
    if ( ! $ic_cdn_content) {
      $found_compressed = false;

      // regex rule start
      $regex_rule = '/src\=\"(http(s?):)\/\/' . self::$uploads_dir . '\/\d{4}\/\d{2}\/([a-zA-Z0-9\-\_\.]+)\"/';

      $match = preg_match_all($regex_rule, $html, $matches, PREG_SET_ORDER);
      if ($matches) {

        foreach ($matches as $key => $match) {
          $image     = $match[3];
          $image_url = $match[0];

          $url_no_src = preg_match('/src=\"(.*)\"/', $image_url, $match_url);
          if ( ! empty($match_url[1])) {
            $attachment_ID = self::attachment_url_to_id($match_url[1]);
            $compressed    = get_post_meta($attachment_ID, 'wps_ic_data', true);

            if ($compressed) {
              $found_compressed = true;
              $html             = str_replace($image_url, 'src="' . self::$protocol . self::$domain . '.wpcompress.xyz/' . $image . '"', $html);
            }
          }

        }
        if ($found_compressed) {
          #update_post_meta($post->ID, '_ic_cdn_content', $html);
        }
      }

      return $html;
    } else {
      return $ic_cdn_content;
    }
  }


  public static function attachment_url_to_id($attachment_url) {
    global $wpdb;
    $attachment_id = false;

    // If there is no url, return.
    if ($attachment_url == '') {
      return;
    }

    // Get the upload directory paths
    $upload_dir_paths = wp_upload_dir();

    if (false !== strpos($attachment_url, $upload_dir_paths['baseurl'])) {

      // If this is the URL of an auto-generated thumbnail, get the URL of the original image
      $attachment_url = preg_replace('/-\d+x\d+(?=\.(jpg|jpeg|png|gif)$)/i', '', $attachment_url);

      // Remove the upload path base directory from the attachment URL
      $attachment_url = str_replace($upload_dir_paths['baseurl'] . '/', '', $attachment_url);

      // Finally, run a custom database query to get the attachment ID from the modified attachment URL
      $attachment_id = $wpdb->get_var($wpdb->prepare("SELECT wposts.ID FROM $wpdb->posts wposts, $wpdb->postmeta wpostmeta WHERE wposts.ID = wpostmeta.post_id AND wpostmeta.meta_key = '_wp_attached_file' AND wpostmeta.meta_value = '%s' AND wposts.post_type = 'attachment'",
                                                     $attachment_url));
    }

    return $attachment_id;
  }


}
