<?php


/**
 * Class - Ajax
 */
class wps_ic_ajax extends wps_ic {

	public $options;


	public function __construct() {
		if (is_admin()) {

			if ( ! empty(parent::$response_key)) {

				$this->add_ajax('wps_ic_any_thumbnails_to_rebuild');
				$this->add_ajax('wps_ic_compress_fetch_thumbnails');

				$this->add_ajax('get_queued_images');
				$this->add_ajax('get_uncompressed_images');
				$this->add_ajax('get_compressed_images');
				$this->add_ajax('get_restored_images');

				$this->add_ajax('wps_ic_check_compress_count');
				$this->add_ajax('wps_ic_api_check_credits');
				$this->add_ajax('wps_ic_compress_hidden_bulk');

				// Bulk Actions
				$this->add_ajax('wps_ic_unlock');

				// Library Actions
				$this->add_ajax('get_library_queued_images');
				$this->add_ajax('get_library_compressed_images');
				$this->add_ajax('get_library_restored_images');

				// Change Setting
				$this->add_ajax('wps_ic_settings_change');
				$this->add_ajax('wps_ic_get_setting');
				$this->add_ajax('wps_ic_get_thumbnail_checkboxes');
				$this->add_ajax('wps_ic_notify');

				// CDN
				$this->add_ajax('check_cdn_status');
				$this->add_ajax('reset_cdn_cache');
				$this->add_ajax('reset_stats');

				// Regenerate
				$this->add_ajax('wps_ic_regenerate_thumbnails_bulk');
				$this->add_ajax('wps_ic_regenerate_thumbnails_prep');

				// Compress
				$this->add_ajax('wps_ic_compress_bulk_prep_hidden_background');
				$this->add_ajax('wps_ic_compress_bulk_prep_background');

				// Restore Bulk
				$this->add_ajax('wps_ic_restore_bulk_prep_background_hidden');
				$this->add_ajax('wps_ic_restore_hidden_bulk');

				// Exclude Image from Compress
				$this->add_ajax('wps_ic_exclude_image');

				// Get the Queue Status
				$this->add_ajax('wps_ic_compress_next_in_queue');
				$this->add_ajax('wps_ic_compress_next_in_queue_last_three');
				$this->add_ajax('wps_ic_library_compress_next_in_queue');
				$this->add_ajax('wps_ic_get_queue');

				// TODO: Maybe delete
				$this->add_ajax('wps_ic_wake_up_cron');

				// Restore the image - single
				$this->add_ajax('wps_ic_single_compress_btn');
				$this->add_ajax('wps_ic_restore_single_btn');

			} else {
				// API Signup
				$this->add_ajax('wps_ic_api_signup');
				$this->add_ajax('wps_ic_api_connect');
				$this->add_ajax('wps_ic_api_test');

				// Authorize with remote API
				$this->add_ajax('wps_ic_authorize_api');
			}
		}

		#delete_option('wps_ic_rebuild_started');
	}


	/**
	 * @since 4.0.0
	 */
	public function wps_ic_unlock() {
		global $wpdb;

		$two_minutes_ago = time() - 60 * 2;
		$two_minutes_ago = date('Y-m-d H:i:s', $two_minutes_ago);

		wp_send_json_error('Unlock deactivated');

		$queue = $wpdb->prepare("SELECT * FROM " . $wpdb->prefix . "ic_queue WHERE created<=%s AND (status=%s OR status=%s)", $two_minutes_ago, 'compressing', 'queued');
		$queue = $wpdb->get_results($queue);

		if ($queue) {
			foreach ($queue as $item) {
				$update = $wpdb->prepare("UPDATE " . $wpdb->prefix . "ic_queue SET status=%s, message=%s, created=%s WHERE attachment_ID=%s", 'compressed', array('code' => 'not_able', 'exited_by' => 'Unlock script ajax'), current_time('mysql'),
																 $item->attachment_ID);
				$update = $wpdb->query($update);
			}

			wp_send_json_success('Unlocked Some Images');
		}

		wp_send_json_error('No images to unlock');
	}


	public function reset_stats() {
		global $wpdb;
		// First check if CDN Zone already exists
		$options = get_option(WPS_IC_OPTIONS);
		$call    = wp_remote_get(WPS_IC_APIURL . '?stats_purge=true&domain=' . site_url() . '&apikey=' . $options['api_key'], array('timeout' => '10', 'sslverify' => false));

		if (wp_remote_retrieve_response_code($call) == 200) {
			$body = wp_remote_retrieve_body($call);
			$body = json_decode($body);

			if ($body->success == 'true') {

				// Stats Purged
				$output = '';
				$output .= '<img src="' . WPS_IC_URI . 'assets/thumbnails-config.png" style="margin-top: 15px;"/>';

				$output .= '<h3>Your stats have been successfully reset.</h3>';
				$output .= '<p>Please allow a few minutes for stats to refresh on the portal.</p>';

				wp_send_json_success(array('html' => $output));
			} else {
				// Stats not purged
				$output = '';
				$output .= '<img src="' . WPS_IC_URI . 'assets/thumbnails-config.png" style="margin-top: 15px;"/>';

				$output .= '<h3>We had problems with resetting your Stats.</h3>';
				$output .= '<p>Please contact our support for further details.</p>';
				wp_send_json_error(array('html' => $output));
			}
		}

		wp_send_json_success(array('html' => $output, 'found' => ''));
	}


	public function reset_cdn_cache() {
		global $wpdb;
		// First check if CDN Zone already exists
		$options = get_option(WPS_IC_OPTIONS);
		$call    = wp_remote_get(WPS_IC_APIURL . '?cdn_purge=true&domain=' . site_url() . '&apikey=' . $options['api_key'], array('timeout' => '10', 'sslverify' => false));

		if (wp_remote_retrieve_response_code($call) == 200) {
			$body = wp_remote_retrieve_body($call);
			$body = json_decode($body);

			if ($body->success == 'true') {
				$cache = $wpdb->query("DELETE FROM " . $wpdb->postmeta . " WHERE meta_key='_ic_sources' OR meta_key='_ic_cdn_content'");
				set_transient('ic_cdn_reset_cache', 'true', 60);

				// CDN Suspended (bandwidth)
				$output = '';
				$output .= '<img src="' . WPS_IC_URI . 'assets/thumbnails-config.png" style="margin-top: 15px;"/>';

				$output .= '<h3>Your CDN Account has been successfully purged.</h3>';
				$output .= '<p>Please allow few minutes for cache reset on global network.</p>';

				wp_send_json_success(array('html' => $output));
			} else {
				// CDN Does not exist,error
				$output = '';
				$output .= '<img src="' . WPS_IC_URI . 'assets/thumbnails-config.png" style="margin-top: 15px;"/>';

				$output .= '<h3>We had problems with resetting your CDN account.</h3>';
				$output .= '<p>Please contact our support for further details.</p>';
				wp_send_json_error(array('html' => $output));
			}
		}

		wp_send_json_success(array('html' => $output, 'found' => ''));
	}


	public function check_cdn_status() {
		// First check if CDN Zone already exists
		$options = get_option(WPS_IC_OPTIONS);

		$request_params           = array();
		$request_params['apiv3']  = 'true';
		$request_params['apikey'] = $options['api_key'];
		$request_params['action'] = 'cdn_check';
		$request_params['url']    = site_url();

		$params = array(
			'method'      => 'POST',
			'timeout'     => 30,
			'redirection' => 3,
			'sslverify'   => false,
			'httpversion' => '1.0',
			'blocking'    => true, // TODO: Mozda true?
			'headers'     => array(),
			'body'        => $request_params,
			'cookies'     => array()
		);

		// Send call to API
		$call = wp_remote_post('http://206.189.251.204/', $params);

		if (wp_remote_retrieve_response_code($call) == 200) {
			$body = wp_remote_retrieve_body($call);
			$body = json_decode($body);

			if ($body->success == 'true') {
				// CDN Does exist or we just created it
				if ($body->data == 'suspended') {
					// CDN Suspended (bandwidth)
					$output = '';
					$output .= '<img src="' . WPS_IC_URI . 'assets/thumbnails-config.png" style="margin-top: 15px;"/>';

					$output .= '<h3>Your CDN account has used all of it\'s bandwidth quota.</h3>';
					$output .= '<p>Please contact our support for further details.</p>';

					wp_send_json_error(array('html' => $output));
				}

			} else {
				// CDN Does not exist,error
				$output = '';
				$output .= '<img src="' . WPS_IC_URI . 'assets/thumbnails-config.png" style="margin-top: 15px;"/>';

				$output .= '<h3>We had problems with creating your CDN account.</h3>';
				$output .= '<p>Please contact our support for further details.</p>';

				wp_send_json_error(array('html' => $output));
			}
		}

		$output .= '<img src="' . WPS_IC_URI . 'assets/thumbnails-config.png" style="margin-top: 15px;"/>';

		$output .= '<h3>Your images are up to date on the CDN</h3>';
		$output .= '<p>Look for significantly faster global performance.</p>';

		wp_send_json_success(array('html' => $output));
	}


	/**
	 * @since 4.0.0
	 */
	public function get_library_restored_images() {
		global $wpdb, $wps_ic;
		$logo_uncompressed = WPS_IC_URI . 'assets/images/not-compressed.png';

		$queue = $wpdb->prepare("SELECT * FROM " . $wpdb->prefix . "ic_queue WHERE (status=%s OR status=%s) AND (type=%s OR type=%s)", 'regenerate', 'restored', 'restore', 'library_restore');
		$queue = $wpdb->get_results($queue);

		if ($queue) {
			$output = array();

			foreach ($queue as $attachment) {

				$attach_id = $attachment->attachment_ID;
				$info      = '';
				$actions   = '';

				$dimmensions        = get_post_meta($attach_id, 'wps_ic_dimmensions', true);
				$uncompressed_value = get_post_meta($attach_id, 'wps_ic_noncompressed_size', true);

				if ( ! $dimmensions) {
					$uploadfile                   = get_attached_file($attach_id);
					$imagesize                    = getimagesize($uploadfile);
					$dimmensions['old']['width']  = $imagesize[0];
					$dimmensions['old']['height'] = $imagesize[1];
				}

				// Image is compressed
				$uncompressed_value = size_format($uncompressed_value, 2);

				$info .= '<div class="wps-ic-uncompressed">';
				$info .= '<img src="' . $logo_uncompressed . '" />';
				$info .= '<h5>Not Compressed</h5>';
				$info .= '</div>';

				$actions .= '<div class="wps-ic-media-actions">';
				$actions .= '<ul class="wps-ic-noncompressed-icon">';
				$actions .= '<li class="wps-ic-weight"><span>' . $uncompressed_value . '</span></li>';
				$actions .= '<li class="wps-ic-size"><span>' . $dimmensions['old']['width'] . 'x' . $dimmensions['old']['height'] . '</span></li>';
				$actions .= '<li class="wps-ic-li-no-padding">';
				$actions .= '<div class="wps-ic-media-actions-toolbox">';
				$actions .= '<ul class="wps-ic-noncompressed-icon">';
				$actions .= '<li class="wps-ic-action">';

				$actions .= '<div class="btn-group">';
				$actions .= '<button type="button" class="btn btn-success wps-ic-compress-single" data-image_id="' . $attach_id . '">Compress</button>';
				$actions .= '<button type="button" class="btn btn-success dropdown-toggle" data-toggle="dropdown" aria-haspopup="true" aria-expanded="false">';
				$actions .= '<span class="caret"></span>';
				$actions .= '<span class="sr-only">Toggle Dropdown</span>';
				$actions .= '</button>';
				$actions .= '<ul class="dropdown-menu">';
				$actions .= apply_filters('wps_ic_pro_exclude', $attach_id);
				$actions .= '</ul>';
				$actions .= '</div>';

				$actions .= '</li>';
				$actions .= '</ul>';
				$actions .= '</div>';

				$output[ $attachment->attachment_ID ]['info']    = $info;
				$output[ $attachment->attachment_ID ]['actions'] = $actions;

				#$wps_ic->queue->remove_queue($attachment->attachment_ID, false, 'restore');
				#$wps_ic->queue->remove_queue($attachment->attachment_ID, false, 'hidden_restore_bulk');
				#$wps_ic->queue->change_queue_status($attachment->attachment_ID, 'done', 'echoed', '');

				$update = $wpdb->prepare("UPDATE " . $wpdb->prefix . "ic_queue SET status=%s, message=%s WHERE ID=%s", 'done', 'echoed', $attachment->ID);
				$wpdb->query($update);

				wp_send_json_success($output);
			}

		} else {
			$queue = $wpdb->prepare("SELECT * FROM " . $wpdb->prefix . "ic_queue WHERE type=%s AND message=%s", 'restore', '{"code":"restoring"}');
			$queue = $wpdb->get_results($queue);

			if ($queue) {
				wp_send_json_success('restoring');
			} else {
				$queue = $wpdb->prepare("SELECT * FROM " . $wpdb->prefix . "ic_queue WHERE type=%s AND message=%s", 'restore', '');
				$queue = $wpdb->get_results($queue);

				if ($queue) {
					wp_send_json_success('compressing');
				}
			}
		}

		wp_send_json_error();
	}


	/**
	 * @since 4.0.0
	 */
	public function get_restored_images() {
		global $wpdb, $wps_ic;
		$logo_uncompressed = WPS_IC_URI . 'assets/images/not-compressed.png';

		$queue = $wpdb->prepare("SELECT * FROM " . $wpdb->prefix . "ic_queue WHERE (status=%s OR status=%s) AND (type=%s OR type=%s)", 'regenerate', 'restored', 'restore', 'hidden_restore_bulk');
		$queue = $wpdb->get_results($queue);

		if ($queue) {
			$output = array();

			foreach ($queue as $attachment) {

				$attach_id = $attachment->attachment_ID;
				$info      = '';
				$actions   = '';

				$dimmensions        = get_post_meta($attach_id, 'wps_ic_dimmensions', true);
				$uncompressed_value = get_post_meta($attach_id, 'wps_ic_noncompressed_size', true);

				if ( ! $dimmensions) {
					$uploadfile                   = get_attached_file($attach_id);
					$imagesize                    = getimagesize($uploadfile);
					$dimmensions['old']['width']  = $imagesize[0];
					$dimmensions['old']['height'] = $imagesize[1];
				}

				// Image is compressed
				$uncompressed_value = size_format($uncompressed_value, 2);

				$info .= '<div class="wps-ic-uncompressed">';
				$info .= '<img src="' . $logo_uncompressed . '" />';
				$info .= '<h5>Not Compressed</h5>';
				$info .= '</div>';

				$actions .= '<div class="wps-ic-media-actions">';
				$actions .= '<ul class="wps-ic-noncompressed-icon">';
				$actions .= '<li class="wps-ic-weight"><span>' . $uncompressed_value . '</span></li>';
				$actions .= '<li class="wps-ic-size"><span>' . $dimmensions['old']['width'] . 'x' . $dimmensions['old']['height'] . '</span></li>';
				$actions .= '<li class="wps-ic-li-no-padding">';
				$actions .= '<div class="wps-ic-media-actions-toolbox">';
				$actions .= '<ul class="wps-ic-noncompressed-icon">';
				$actions .= '<li class="wps-ic-action">';

				$actions .= '<div class="btn-group">';
				$actions .= '<button type="button" class="btn btn-success wps-ic-compress-single" data-image_id="' . $attach_id . '">Compress</button>';
				$actions .= '<button type="button" class="btn btn-success dropdown-toggle" data-toggle="dropdown" aria-haspopup="true" aria-expanded="false">';
				$actions .= '<span class="caret"></span>';
				$actions .= '<span class="sr-only">Toggle Dropdown</span>';
				$actions .= '</button>';
				$actions .= '<ul class="dropdown-menu">';
				$actions .= apply_filters('wps_ic_pro_exclude', $attach_id);
				$actions .= '</ul>';
				$actions .= '</div>';

				$actions .= '</li>';
				$actions .= '</ul>';
				$actions .= '</div>';

				$output[ $attachment->attachment_ID ]['info']    = $info;
				$output[ $attachment->attachment_ID ]['actions'] = $actions;

				#$wps_ic->queue->remove_queue($attachment->attachment_ID, false, 'restore');
				#$wps_ic->queue->remove_queue($attachment->attachment_ID, false, 'hidden_restore_bulk');
				#$wps_ic->queue->change_queue_status($attachment->attachment_ID, 'done', 'echoed', '');

				$update = $wpdb->prepare("UPDATE " . $wpdb->prefix . "ic_queue SET status=%s, message=%s WHERE ID=%s", 'done', 'echoed', $attachment->ID);
				$wpdb->query($update);

				wp_send_json_success($output);
			}

		} else {
			$queue = $wpdb->prepare("SELECT * FROM " . $wpdb->prefix . "ic_queue WHERE type=%s AND message=%s", 'restore', '{"code":"restoring"}');
			$queue = $wpdb->get_results($queue);

			if ($queue) {
				wp_send_json_success('restoring');
			} else {
				$queue = $wpdb->prepare("SELECT * FROM " . $wpdb->prefix . "ic_queue WHERE type=%s AND message=%s", 'restore', '');
				$queue = $wpdb->get_results($queue);

				if ($queue) {
					wp_send_json_success('compressing');
				}
			}
		}

		wp_send_json_error();
	}


	/**
	 * @since 4.0.0
	 */
	public function get_library_queued_images() {
		global $wps_ic;
		$restore_queue  = $wps_ic->queue->get_next_queue('library_restore');
		$compress_queue = $wps_ic->queue->get_next_queue('library_compress');

		if ($restore_queue || $compress_queue) {
			wp_send_json_success();
		} else {
			wp_send_json_error();
		}
	}


	public function get_queued_images() {
		global $wps_ic;
		$restore_queue    = $wps_ic->queue->get_next_queue('hidden_restore_bulk');
		$compress_queue   = $wps_ic->queue->get_next_queue('hidden_compress_bulk');
		$regenerate_queue = $wps_ic->queue->get_next_queue('hidden_regenerate');

		if ($restore_queue || $compress_queue || $regenerate_queue) {
			wp_send_json_success();
		} else {
			wp_send_json_error();
		}
	}


	public function get_uncompressed_images() {
		global $wpdb, $wps_ic;
		$logo_uncompressed = WPS_IC_URI . 'assets/images/not-compressed.png';
		$logo_compressed   = WPS_IC_URI . 'assets/images/compressed.png';

		$queue = $wpdb->prepare("SELECT * FROM " . $wpdb->prefix . "ic_queue WHERE status=%s AND (type=%s OR type=%s)", 'not_able', 'compress', 'hidden_compress_bulk');
		$queue = $wpdb->get_results($queue);

		if ($queue) {
			$output = array();

			foreach ($queue as $attachment) {

				$attach_id = $attachment->attachment_ID;
				$info      = '';
				$actions   = '';

				$file_data          = get_attached_file($attach_id);
				$compress           = get_post_meta($attach_id, 'wps_ic_data', true);
				$dimmensions        = get_post_meta($attach_id, 'wps_ic_dimmensions', true);
				$uncompressed_value = get_post_meta($attach_id, 'wps_ic_noncompressed_size', true);

				if ( ! $dimmensions) {
					$uploadfile                   = get_attached_file($attach_id);
					$imagesize                    = getimagesize($uploadfile);
					$dimmensions['new']['width']  = $imagesize[0];
					$dimmensions['new']['height'] = $imagesize[1];
				}

				$attachment_dimensions_new = $dimmensions['new']['width'] . 'x' . $dimmensions['new']['height'];

				if ( ! empty($compress) && ! empty($compress['old']['size'])) {
					// Image is compressed
					$compressed_value_num = $compressed_value = $compress['new']['size'];
					$uncompressed_value   = $compress['old']['size'];
				} else {
					// Image is not compressed
					$compressed_value_num = $compressed_value = 0;

					if (empty($uncompressed_value)) {
						$uncompressed_value = filesize($file_data);
						update_post_meta($attach_id, 'wps_ic_noncompressed_size', $uncompressed_value);
					}

				}

				if ($compress != 'not_able') {

					// Old size
					$uncompressed_value_num = $compress['old']['size'] / 1000;

					// Saved total
					$saved_total = $compress['old']['size'] - $compressed_value_num;
					$saved_total = size_format($saved_total, 2);

					// Saved
					$saved = ($compressed_value_num / ($uncompressed_value_num * 1000)) * 100;
					$saved = round(100 - $saved, 0) . '';

					if ($saved == '0') {

						// Not Further Savings
						$info = '';
						$info .= '<div class="wps-ic-uncompressed">';
						$info .= '<img src="' . $logo_compressed . '" />';
						$info .= '<h5 class="no-further-savings">No Further Savings</h5>';
						$info .= '</div>';

					} else {

						// x% saved on compression
						$info = '';
						$info .= '<div class="wps-ic-compressed">';
						$info .= '<img src="' . $logo_compressed . '" />';
						$info .= '<h5>' . $saved . '%</h5>';
						$info .= '<h5>' . $saved_total . '</h5>';
						$info .= '<h5>Saved</h5>';
						$info .= '</div>';

					}

					$actions .= '<div class="wps-ic-media-actions">';
					$actions .= '<div class="wps-ic-half-media-actions">';
					$actions .= '<ul class="wps-ic-noncompressed-icon">';

					$actions .= '<li class="wps-ic-weight"><span class="strike">' . size_format($uncompressed_value, 2) . '</span></li>';
					$actions .= '<li class="wps-ic-size"><span>' . $attachment_dimensions_new . '</span></li>';

					$actions .= '</ul>';
					$actions .= '</div>';
					$actions .= '<div class="wps-ic-half-media-actions">';
					$actions .= '<ul class="wps-ic-noncompressed-icon">';
					$actions .= '</li>';

					$actions .= '<li class="wps-ic-size-compressed"><span>' . size_format($compressed_value, 2) . '</span></li>';
					#$actions .= '<li class="wps-ic-time"><span>' . $time . '</span></li>';

					$actions .= '</ul>';
					$actions .= '</div>';

					$actions .= '<div class="wps-ic-media-actions-toolbox">';
					$actions .= '<ul class="wps-ic-noncompressed-icon">';
					$actions .= '<li class="wps-ic-action">';

					$actions .= '<button type="button" class="btn btn-info wps-ic-restore-single" data-image_id="' . $attach_id . '">Restore</button>';

					$actions .= '</li>';
					$actions .= '</ul>';
					$actions .= '</div>';

					$actions .= '</div>';

				} else {
					// Compressed
					$file_data        = get_attached_file($attach_id);
					$compressed_value = filesize($file_data);
					$compressed_value = size_format($compressed_value, 2);
					$dimensions       = getimagesize($file_data);

					$info .= '<div class="wps-ic-compressed">';
					$info .= '<img src="' . $logo_compressed . '" />';
					$info .= '<h5 class="no-further-savings">No Further Savings</h5>';
					$info .= '</div>';

					$actions .= '<div class="wps-ic-media-actions">';
					$actions .= '<ul class="wps-ic-noncompressed-icon">';
					$actions .= '<li class="wps-ic-weight"><span>' . $compressed_value . '</span></li>';
					$actions .= '<li class="wps-ic-size"><span>' . $dimensions[0] . 'x' . $dimensions[1] . '</span></li>';
					$actions .= '<li class="wps-ic-li-no-padding">';
					$actions .= '<div class="wps-ic-media-actions-toolbox">';
					$actions .= '<ul class="wps-ic-noncompressed-icon">';
					$actions .= '<li class="wps-ic-action">';

					$actions .= '<button type="button" class="btn btn-info wps-ic-restore-single" data-type="reset" data-image_id="' . $attach_id . '">Reset</button>';

					$actions .= '</li>';
					$actions .= '</ul>';
					$actions .= '</div>';

					$actions .= '<div id="wps-ic-compare' . $attach_id . '" class="lightbox wp-ic-compare"></div>';
					$actions .= '<div id="wps-ic-pro-compare' . $attach_id . '" class="lightbox wp-ic-compare"></div>';
				}

				$output[ $attachment->attachment_ID ]['info']    = $info;
				$output[ $attachment->attachment_ID ]['actions'] = $actions;

				#$wps_ic->queue->remove_queue($attachment->attachment_ID, false, 'compress');
				#$wps_ic->queue->remove_queue($attachment->attachment_ID, false, 'hidden_compress_bulk');

				$update = $wpdb->prepare("UPDATE " . $wpdb->prefix . "ic_queue SET status=%s, message=%s WHERE ID=%s", 'done', 'echoed', $attachment->ID);
				$wpdb->query($update);
			}

			$cache = $wpdb->query("DELETE FROM " . $wpdb->postmeta . " WHERE meta_key='_ic_sources' OR meta_key='_ic_cdn_content'");

			wp_send_json_success($output);
		} else {

			$queue = $wpdb->prepare("SELECT * FROM " . $wpdb->prefix . "ic_queue WHERE status=%s AND type=%s", 'queued', 'compress');
			$queue = $wpdb->get_results($queue);

			if ($queue) {
				wp_send_json_success('compressing');
			} else {
				/*$queue = $wpdb->prepare("SELECT * FROM " . $wpdb->prefix . "ic_queue WHERE status=%s AND type=%s", 'queued', 'compress');
				$queue = $wpdb->get_results($queue);

				if ($queue) {
					wp_send_json_success('compressing');
				}*/
			}

		}

		wp_send_json_error();
	}


	/**
	 * @since 4.0.0
	 */
	public function get_library_compressed_images() {
		global $wpdb, $wps_ic;
		$logo_uncompressed = WPS_IC_URI . 'assets/images/not-compressed.png';
		$logo_compressed   = WPS_IC_URI . 'assets/images/compressed.png';

		$queue = $wpdb->prepare("SELECT * FROM " . $wpdb->prefix . "ic_queue WHERE status=%s AND (type=%s OR type=%s)", 'compressed', 'compress', 'library_compress');
		$queue = $wpdb->get_results($queue);

		if ($queue) {
			$output = array();

			foreach ($queue as $attachment) {

				$attach_id = $attachment->attachment_ID;
				$info      = '';
				$actions   = '';

				$file_data          = get_attached_file($attach_id);
				$compress           = get_post_meta($attach_id, 'wps_ic_data', true);
				$dimmensions        = get_post_meta($attach_id, 'wps_ic_dimmensions', true);
				$uncompressed_value = get_post_meta($attach_id, 'wps_ic_noncompressed_size', true);

				if ( ! $compress || empty($compress['new']['size'])) {
					#continue;
				}

				if ( ! $dimmensions['new']['width']) {
					$uploadfile = get_attached_file($attach_id);
					clearstatcache();
					$imagesize                    = getimagesize($uploadfile);
					$dimmensions['new']['width']  = $imagesize[0];
					$dimmensions['new']['height'] = $imagesize[1];
				}

				$attachment_dimensions_new = $dimmensions['new']['width'] . 'x' . $dimmensions['new']['height'];

				if ( ! empty($compress) && ! empty($compress['old']['size'])) {
					// Image is compressed
					$compressed_value_num = $compressed_value = $compress['new']['size'];
					$uncompressed_value   = $compress['old']['size'];
				} else {
					// Image is not compressed
					$compressed_value_num = $compressed_value = 0;

					if (empty($uncompressed_value)) {
						$uncompressed_value = filesize($file_data);
						update_post_meta($attach_id, 'wps_ic_noncompressed_size', $uncompressed_value);
					}

				}

				if ($compress == 'not_able') {
					// Compressed
					$file_data        = get_attached_file($attach_id);
					$compressed_value = filesize($file_data);
					$compressed_value = size_format($compressed_value, 2);
					$dimensions       = getimagesize($file_data);

					$info .= '<div class="wps-ic-compressed">';
					$info .= '<img src="' . $logo_compressed . '" />';
					$info .= '<h5 class="no-further-savings">No Further Savings</h5>';
					$info .= '</div>';

					$actions .= '<div class="wps-ic-media-actions">';
					$actions .= '<ul class="wps-ic-noncompressed-icon">';
					$actions .= '<li class="wps-ic-weight"><span>' . $compressed_value . '</span></li>';
					$actions .= '<li class="wps-ic-size"><span>' . $dimensions[0] . 'x' . $dimensions[1] . '</span></li>';
					$actions .= '<li class="wps-ic-li-no-padding">';
					$actions .= '<div class="wps-ic-media-actions-toolbox">';
					$actions .= '<ul class="wps-ic-noncompressed-icon">';
					$actions .= '<li class="wps-ic-action">';

					$actions .= '<button type="button" class="btn btn-info wps-ic-restore-single" data-type="reset" data-image_id="' . $attach_id . '">Reset</button>';

					$actions .= '</li>';
					$actions .= '</ul>';
					$actions .= '</div>';

					$actions .= '<div id="wps-ic-compare' . $attach_id . '" class="lightbox wp-ic-compare"></div>';
					$actions .= '<div id="wps-ic-pro-compare' . $attach_id . '" class="lightbox wp-ic-compare"></div>';
				} else if ($compress == 'no_credits') {
					// No credits
					$file_data          = get_attached_file($attach_id);
					$uncompressed_value = filesize($file_data);
					$uncompressed_value = size_format($uncompressed_value, 2);
					$dimensions         = getimagesize($file_data);

					$info .= '<div class="wps-ic-compressed">';
					$info .= '<img src="' . $logo_uncompressed . '" />';
					$info .= '<h5 class="no-further-savings">Not Enough Credits</h5>';
					$info .= '</div>';

					$actions .= '<div class="wps-ic-media-actions">';
					$actions .= '<ul class="wps-ic-noncompressed-icon">';
					$actions .= '<li class="wps-ic-weight"><span>' . $uncompressed_value . '</span></li>';
					$actions .= '<li class="wps-ic-size"><span>' . $dimensions[0] . 'x' . $dimensions[1] . '</span></li>';
					$actions .= '<li class="wps-ic-li-no-padding">';
					$actions .= '<div class="wps-ic-media-actions-toolbox">';
					$actions .= '<ul class="wps-ic-noncompressed-icon">';
					$actions .= '<li class="wps-ic-action">';

					$actions .= '<button type="button" class="btn btn-success wps-ic-compress-single" data-image_id="' . $attach_id . '">Compress</button>';

					$actions .= '</li>';
					$actions .= '</ul>';
					$actions .= '</div>';
					$actions .= '<script type="text/javascript">swal({
                        title: \'\',
                        html: jQuery(\'#ic_no_credits_popup\').html(),
                        width: 900,
                        showCloseButton: true,
                        showCancelButton: false,
                        showConfirmButton: false,
                        onClose: function () {
                            //window.location.reload();
                        }
                    });</script>';
				} else {

					// Old size
					$uncompressed_value_num = $compress['old']['size'] / 1000;

					// Saved total
					$saved_total = $compress['old']['size'] - $compressed_value_num;
					$saved_total = size_format($saved_total, 2);

					// Saved
					$saved = ($compressed_value_num / ($uncompressed_value_num * 1000)) * 100;
					$saved = round(100 - $saved, 0) . '';

					if ($saved == '0') {

						// Not Further Savings
						$info = '';
						$info .= '<div class="wps-ic-uncompressed">';
						$info .= '<img src="' . $logo_compressed . '" />';
						$info .= '<h5 class="no-further-savings">No Further Savings</h5>';
						$info .= '</div>';

					} else {

						// x% saved on compression
						$info = '';
						$info .= '<div class="wps-ic-compressed">';
						$info .= '<img src="' . $logo_compressed . '" />';
						$info .= '<h5>' . $saved . '%</h5>';
						$info .= '<h5>' . $saved_total . '</h5>';
						$info .= '<h5>Saved</h5>';
						$info .= '</div>';

					}

					$actions .= '<div class="wps-ic-media-actions">';
					$actions .= '<div class="wps-ic-half-media-actions">';
					$actions .= '<ul class="wps-ic-noncompressed-icon">';

					$actions .= '<li class="wps-ic-weight"><span class="strike">' . size_format($uncompressed_value, 2) . '</span></li>';
					$actions .= '<li class="wps-ic-size"><span>' . $attachment_dimensions_new . '</span></li>';

					$actions .= '</ul>';
					$actions .= '</div>';
					$actions .= '<div class="wps-ic-half-media-actions">';
					$actions .= '<ul class="wps-ic-noncompressed-icon">';
					$actions .= '</li>';

					$actions .= '<li class="wps-ic-size-compressed"><span>' . size_format($compressed_value, 2) . '</span></li>';
					#$actions .= '<li class="wps-ic-time"><span>' . $time . '</span></li>';

					$actions .= '</ul>';
					$actions .= '</div>';

					$actions .= '<div class="wps-ic-media-actions-toolbox">';
					$actions .= '<ul class="wps-ic-noncompressed-icon">';
					$actions .= '<li class="wps-ic-action">';

					$actions .= '<button type="button" class="btn btn-info wps-ic-restore-single" data-image_id="' . $attach_id . '">Restore</button>';

					$actions .= '</li>';
					$actions .= '</ul>';
					$actions .= '</div>';

					$actions .= '</div>';
				}

				$output[ $attachment->attachment_ID ]['info']    = $info;
				$output[ $attachment->attachment_ID ]['actions'] = $actions;

				#$wps_ic->queue->remove_queue($attachment->attachment_ID, false, 'compress');
				#$wps_ic->queue->remove_queue($attachment->attachment_ID, false, 'hidden_compress_bulk');

				$update = $wpdb->prepare("UPDATE " . $wpdb->prefix . "ic_queue SET status=%s, message=%s WHERE ID=%s", 'done', 'echoed', $attachment->ID);
				$wpdb->query($update);
			}

			$cache = $wpdb->query("DELETE FROM " . $wpdb->postmeta . " WHERE meta_key='_ic_sources' OR meta_key='_ic_cdn_content'");

			wp_send_json_success($output);
		} else {

			$queue = $wpdb->prepare("SELECT * FROM " . $wpdb->prefix . "ic_queue WHERE status=%s AND type=%s", 'queued', 'compress');
			$queue = $wpdb->get_results($queue);

			if ($queue) {
				wp_send_json_success('compressing');
			} else {
				/*$queue = $wpdb->prepare("SELECT * FROM " . $wpdb->prefix . "ic_queue WHERE status=%s AND type=%s", 'queued', 'compress');
				$queue = $wpdb->get_results($queue);

				if ($queue) {
					wp_send_json_success('compressing');
				}*/
			}

		}

		wp_send_json_error();
	}


	/**
	 * @since 4.0.0
	 */
	public function get_compressed_images() {
		global $wpdb, $wps_ic;
		$logo_uncompressed = WPS_IC_URI . 'assets/images/not-compressed.png';
		$logo_compressed   = WPS_IC_URI . 'assets/images/compressed.png';

		$queue = $wpdb->prepare("SELECT * FROM " . $wpdb->prefix . "ic_queue WHERE status=%s AND (type=%s OR type=%s)", 'compressed', 'compress', 'hidden_compress_bulk');
		$queue = $wpdb->get_results($queue);

		if ($queue) {
			$output = array();

			foreach ($queue as $attachment) {

				$attach_id = $attachment->attachment_ID;
				$info      = '';
				$actions   = '';

				$file_data          = get_attached_file($attach_id);
				$compress           = get_post_meta($attach_id, 'wps_ic_data', true);
				$dimmensions        = get_post_meta($attach_id, 'wps_ic_dimmensions', true);
				$uncompressed_value = get_post_meta($attach_id, 'wps_ic_noncompressed_size', true);

				if ( ! $compress || empty($compress['new']['size'])) {
					#continue;
				}

				if ( ! $dimmensions) {
					$uploadfile                   = get_attached_file($attach_id);
					$imagesize                    = getimagesize($uploadfile);
					$dimmensions['new']['width']  = $imagesize[0];
					$dimmensions['new']['height'] = $imagesize[1];
				}

				$attachment_dimensions_new = $dimmensions['new']['width'] . 'x' . $dimmensions['new']['height'];

				if ( ! empty($compress) && ! empty($compress['old']['size'])) {
					// Image is compressed
					$compressed_value_num = $compressed_value = $compress['new']['size'];
					$uncompressed_value   = $compress['old']['size'];
				} else {
					// Image is not compressed
					$compressed_value_num = $compressed_value = 0;

					if (empty($uncompressed_value)) {
						$uncompressed_value = filesize($file_data);
						update_post_meta($attach_id, 'wps_ic_noncompressed_size', $uncompressed_value);
					}

				}

				if ($compress == 'not_able') {
					// Compressed
					$file_data        = get_attached_file($attach_id);
					$compressed_value = filesize($file_data);
					$compressed_value = size_format($compressed_value, 2);
					$dimensions       = getimagesize($file_data);

					$info .= '<div class="wps-ic-compressed">';
					$info .= '<img src="' . $logo_compressed . '" />';
					$info .= '<h5 class="no-further-savings">No Further Savings</h5>';
					$info .= '</div>';

					$actions .= '<div class="wps-ic-media-actions">';
					$actions .= '<ul class="wps-ic-noncompressed-icon">';
					$actions .= '<li class="wps-ic-weight"><span>' . $compressed_value . '</span></li>';
					$actions .= '<li class="wps-ic-size"><span>' . $dimensions[0] . 'x' . $dimensions[1] . '</span></li>';
					$actions .= '<li class="wps-ic-li-no-padding">';
					$actions .= '<div class="wps-ic-media-actions-toolbox">';
					$actions .= '<ul class="wps-ic-noncompressed-icon">';
					$actions .= '<li class="wps-ic-action">';

					$actions .= '<button type="button" class="btn btn-info wps-ic-restore-single" data-type="reset" data-image_id="' . $attach_id . '">Reset</button>';

					$actions .= '</li>';
					$actions .= '</ul>';
					$actions .= '</div>';

					$actions .= '<div id="wps-ic-compare' . $attach_id . '" class="lightbox wp-ic-compare"></div>';
					$actions .= '<div id="wps-ic-pro-compare' . $attach_id . '" class="lightbox wp-ic-compare"></div>';
				} else if ($compress == 'no_credits') {
					// No credits
					$file_data          = get_attached_file($attach_id);
					$uncompressed_value = filesize($file_data);
					$uncompressed_value = size_format($uncompressed_value, 2);
					$dimensions         = getimagesize($file_data);

					$info .= '<div class="wps-ic-compressed">';
					$info .= '<img src="' . $logo_uncompressed . '" />';
					$info .= '<h5 class="no-further-savings">Not Enough Credits</h5>';
					$info .= '</div>';

					$actions .= '<div class="wps-ic-media-actions">';
					$actions .= '<ul class="wps-ic-noncompressed-icon">';
					$actions .= '<li class="wps-ic-weight"><span>' . $uncompressed_value . '</span></li>';
					$actions .= '<li class="wps-ic-size"><span>' . $dimensions[0] . 'x' . $dimensions[1] . '</span></li>';
					$actions .= '<li class="wps-ic-li-no-padding">';
					$actions .= '<div class="wps-ic-media-actions-toolbox">';
					$actions .= '<ul class="wps-ic-noncompressed-icon">';
					$actions .= '<li class="wps-ic-action">';

					$actions .= '<button type="button" class="btn btn-success wps-ic-compress-single" data-image_id="' . $attach_id . '">Compress</button>';

					$actions .= '</li>';
					$actions .= '</ul>';
					$actions .= '</div>';
					$actions .= '<script type="text/javascript">swal({
                        title: \'\',
                        html: jQuery(\'#ic_no_credits_popup\').html(),
                        width: 900,
                        showCloseButton: true,
                        showCancelButton: false,
                        showConfirmButton: false,
                        onClose: function () {
                            //window.location.reload();
                        }
                    });</script>';
				} else {

					// Old size
					$uncompressed_value_num = $compress['old']['size'] / 1000;

					// Saved total
					$saved_total = $compress['old']['size'] - $compressed_value_num;
					$saved_total = size_format($saved_total, 2);

					// Saved
					$saved = ($compressed_value_num / ($uncompressed_value_num * 1000)) * 100;
					$saved = round(100 - $saved, 0) . '';

					if ($saved == '0') {

						// Not Further Savings
						$info = '';
						$info .= '<div class="wps-ic-uncompressed">';
						$info .= '<img src="' . $logo_compressed . '" />';
						$info .= '<h5 class="no-further-savings">No Further Savings</h5>';
						$info .= '</div>';

					} else {

						// x% saved on compression
						$info = '';
						$info .= '<div class="wps-ic-compressed">';
						$info .= '<img src="' . $logo_compressed . '" />';
						$info .= '<h5>' . $saved . '%</h5>';
						$info .= '<h5>' . $saved_total . '</h5>';
						$info .= '<h5>Saved</h5>';
						$info .= '</div>';

					}

					$actions .= '<div class="wps-ic-media-actions">';
					$actions .= '<div class="wps-ic-half-media-actions">';
					$actions .= '<ul class="wps-ic-noncompressed-icon">';

					$actions .= '<li class="wps-ic-weight"><span class="strike">' . size_format($uncompressed_value, 2) . '</span></li>';
					$actions .= '<li class="wps-ic-size"><span>' . $attachment_dimensions_new . '</span></li>';

					$actions .= '</ul>';
					$actions .= '</div>';
					$actions .= '<div class="wps-ic-half-media-actions">';
					$actions .= '<ul class="wps-ic-noncompressed-icon">';
					$actions .= '</li>';

					$actions .= '<li class="wps-ic-size-compressed"><span>' . size_format($compressed_value, 2) . '</span></li>';
					#$actions .= '<li class="wps-ic-time"><span>' . $time . '</span></li>';

					$actions .= '</ul>';
					$actions .= '</div>';

					$actions .= '<div class="wps-ic-media-actions-toolbox">';
					$actions .= '<ul class="wps-ic-noncompressed-icon">';
					$actions .= '<li class="wps-ic-action">';

					$actions .= '<button type="button" class="btn btn-info wps-ic-restore-single" data-image_id="' . $attach_id . '">Restore</button>';

					$actions .= '</li>';
					$actions .= '</ul>';
					$actions .= '</div>';

					$actions .= '</div>';
				}

				$output[ $attachment->attachment_ID ]['info']    = $info;
				$output[ $attachment->attachment_ID ]['actions'] = $actions;

				#$wps_ic->queue->remove_queue($attachment->attachment_ID, false, 'compress');
				#$wps_ic->queue->remove_queue($attachment->attachment_ID, false, 'hidden_compress_bulk');

				$update = $wpdb->prepare("UPDATE " . $wpdb->prefix . "ic_queue SET status=%s, message=%s WHERE ID=%s", 'done', 'echoed', $attachment->ID);
				$wpdb->query($update);
			}

			$cache = $wpdb->query("DELETE FROM " . $wpdb->postmeta . " WHERE meta_key='_ic_sources' OR meta_key='_ic_cdn_content'");

			wp_send_json_success($output);
		} else {

			$queue = $wpdb->prepare("SELECT * FROM " . $wpdb->prefix . "ic_queue WHERE status=%s AND type=%s", 'queued', 'compress');
			$queue = $wpdb->get_results($queue);

			if ($queue) {
				wp_send_json_success('compressing');
			} else {
				/*$queue = $wpdb->prepare("SELECT * FROM " . $wpdb->prefix . "ic_queue WHERE status=%s AND type=%s", 'queued', 'compress');
				$queue = $wpdb->get_results($queue);

				if ($queue) {
					wp_send_json_success('compressing');
				}*/
			}

		}

		wp_send_json_error();
	}


	public function add_ajax($hook) {
		add_action('wp_ajax_' . $hook, array($this, $hook));
	}


	public function wps_ic_get_thumbnail_checkboxes() {
		global $wps_ic;

		$output   = '';
		$settings = $wps_ic->options->get_settings();
		$sizes    = get_intermediate_image_sizes();
		/*
		$all_checked = 1;
		foreach ($sizes as $key => $value) {
			if (checked('1', $settings['thumbnails'][ $value ], false) != " checked='checked'") {
				$all_checked = 0;
			}
		}
		*/

		foreach ($sizes as $key => $value) {

			if ($settings['thumbnails'][ $value ] == '1') {
				$output .= '<div class="checkbox-container popup-checkbox wp-ic-setting[thumbnails][' . $value . ']">
          <h3> ' . $value . '</h3>
          <div>
            <input type="checkbox" class="checkbox" id="wp-ic-setting[thumbnails][' . $value . ']" style="display: none;" ' . checked('1', $settings['thumbnails'][ $value ], false) . ' value="' . $value . '"
                   name="wp-ic-setting[thumbnails][' . $value . ']"/>
            <label for="wp-ic-setting[thumbnails][' . $value . ']" class="check checkbox-live">
              <svg width="18px" height="18px" viewBox="0 0 18 18">
                <path d="M1,9 L1,3.5 C1,2 2,1 3.5,1 L14.5,1 C16,1 17,2 17,3.5 L17,14.5 C17,16 16,17 14.5,17 L3.5,17 C2,17 1,16 1,14.5 L1,9 Z"></path>
                <polyline points="1 9 7 14 15 4"></polyline>
              </svg>
            </label>
          </div>
        </div>';
			} else {
				$output .= '<div class="checkbox-container popup-checkbox popup-deactivated wp-ic-setting[thumbnails][' . $value . ']">
          <h3> ' . $value . '</h3>
          <div>
            <input type="checkbox" class="checkbox" id="wp-ic-setting[thumbnails][' . $value . ']" style="display: none;" ' . checked('1', $settings['thumbnails'][ $value ], false) . ' value="' . $value . '"
                   name="wp-ic-setting[thumbnails][' . $value . ']"/>
            <label for="wp-ic-setting[thumbnails][' . $value . ']" class="check checkbox-live">
              <svg width="18px" height="18px" viewBox="0 0 18 18">
                <path d="M1,9 L1,3.5 C1,2 2,1 3.5,1 L14.5,1 C16,1 17,2 17,3.5 L17,14.5 C17,16 16,17 14.5,17 L3.5,17 C2,17 1,16 1,14.5 L1,9 Z"></path>
                <polyline points="1 9 7 14 15 4"></polyline>
              </svg>
            </label>
          </div>
        </div>';
			}
		}

		$output .= '<div class="clearfix"></div>
        <div style="height: 30px"></div>
        <a href="#" class="popup-save" data-action="thumbnails">Save</a>';

		wp_send_json_success(array('html' => $output));
	}


	/**
	 * @since 4.1.4
	 */
	public static function wps_ic_check_compress_count() {
		global $wpdb;

		$attachment_list   = $wpdb->get_results("SELECT ID FROM " . $wpdb->posts . " WHERE post_type='attachment' AND post_status='inherit'");
		$attachments_count = count($attachment_list);

		$already_compressed = $wpdb->get_var($wpdb->prepare("SELECT COUNT(ID) as files FROM " . $wpdb->prefix . "ic_compressed WHERE restored=%s", 0));

		if ($already_compressed >= $attachments_count) {
			wp_send_json_error(array('compressed' => $already_compressed, 'images' => $attachments_count));
		} else {
			wp_send_json_success(array('compressed' => $already_compressed, 'images' => $attachments_count));
		}
	}


	/**
	 * @since 3.3.0
	 */
	public static function wps_ic_api_check_credits() {
		// Check credits
		$options       = get_option(WPS_IC_OPTIONS);
		$check_credits = wp_remote_get(WPS_IC_APIURL . '?apikey=' . $options['api_key'] . '&check_credits=true', array('timeout' => '25', 'sslverify' => false));
		$credits       = 0;

		if (wp_remote_retrieve_response_code($check_credits) == 200) {
			$body = wp_remote_retrieve_body($check_credits);
			$body = json_decode($body, true);
		}

		wp_send_json_success($body['data']['credits']);
	}


	/**
	 * @since 3.3.0
	 */
	public function wps_ic_wake_up_cron($response = true) {
		$working = get_transient('ic_cron_working');

		if ( ! empty($working) && $working == 'true') {
			wp_send_json_success();
		}

		foreach (_get_cron_array() as $timestamp => $events) {

			foreach ($events as $event_hook => $event_args) {
				foreach ($event_args as $event) {

					if ($event_hook == 'wps_ic_regen_thumbs_by_array') {
						foreach ($event_args as $hash => $event_Args) {

							if ( ! empty($event_Args['args'])) {
								delete_transient('doing_cron');
								set_transient('ic_cron_working', 'true', 20);
								do_action_ref_array($event_hook, $event_Args['args']);
							}

							wp_send_json_success('regen_thumbs');
						}
					}

					if ($event_hook == 'wps_ic_compress_single') {
						foreach ($event_args as $hash => $event_Args) {

							if ( ! empty($event_Args['args'])) {
								delete_transient('doing_cron');
								set_transient('ic_cron_working', 'true', 20);
								do_action_ref_array($event_hook, $event_Args['args']);
							}

							wp_send_json_success('compress');
						}
					}

					if ($event_hook == 'wps_ic_restore_single') {
						foreach ($event_args as $hash => $event_Args) {

							if ( ! empty($event_Args['args'])) {
								delete_transient('doing_cron');
								#set_transient('ic_cron_working', 'true', 6);
								do_action_ref_array($event_hook, $event_Args['args']);
							}

							wp_send_json_success('restore');
						}
					}

				}

			}

		}

		if ($response) {
			wp_send_json_success();
		} else {
			update_option('no_cron_set', 'true');
			wp_send_json_error();
		}
	}


	/**
	 * @since 4.0.0
	 */
	public function wps_ic_restore_bulk_prep_background_hidden($response = 'json') {
		global $wps_ic, $wpdb;

		$output = array();

		$count_files = 0;

		update_option('wps_ic_hidden_bulk_running', 'true');
		delete_option(WPS_IC_BULK_RESTORE_TOTAL);
		delete_option(WPS_IC_BULK_RESTORE_PROCESSED);
		update_option('wps_ic_bulk_started', microtime(true));

		// Clear whole queue
		$wps_ic->queue->clear_queue('hidden_regenerate');
		$wps_ic->queue->clear_queue('hidden_compress_bulk');
		$wps_ic->queue->clear_queue('compress_bulk');
		$wps_ic->queue->clear_queue('restore_bulk');
		$wps_ic->queue->clear_queue('hidden_restore_bulk');
		$wps_ic->queue->clear_queue('generate_thumbnail');

		/*
		 * Check if it's a single file clicked or bulk compress button
		 */
		$attachments = $wpdb->get_results("SELECT ID FROM " . $wpdb->prefix . "posts WHERE post_type='attachment' AND post_status='inherit' ORDER BY post_date DESC");

		if ($attachments) {
			foreach ($attachments as $attachment) {

				$data = get_post_meta($attachment->ID, 'wps_ic_data', true);

				if ($data == 'not_able') {
					update_post_meta($attachment->ID, 'wps_ic_reset', 'true');
				} else {

					// If not compressed skip
					$compressed = get_post_meta($attachment->ID, 'wps_ic_compressed', true);
					if (empty($compressed) && (empty($data) || $data != 'not_able')) {

						// Delete compress data
						delete_post_meta($attachment->ID, 'wps_ic_compressed_size');
						delete_post_meta($attachment->ID, 'wps_ic_reset');
						delete_post_meta($attachment->ID, 'wps_ic_times');
						delete_post_meta($attachment->ID, 'wps_ic_compressed');
						delete_post_meta($attachment->ID, 'wps_ic_data');
						delete_post_meta($attachment->ID, 'wps_ic_cdn');
						delete_post_meta($attachment->ID, 'wps_ic_in_bulk');
						delete_post_meta($attachment->ID, 'wps_ic_compressing');
						delete_post_meta($attachment->ID, 'wps_ic_restoring');
						continue;
					}

					// If excluded skip
					$excluded = get_post_meta($attachment->ID, 'wps_ic_exclude', true);
					if ( ! empty($excluded) && $excluded == 'true') {
						continue;
					}

					// If not compressed skip
					if (empty($data) && empty($data['new'])) {
						continue;
					}

				}

				// Allowed type
				$file_data = get_attached_file($attachment->ID);
				$type      = wp_check_filetype($file_data);
				$exif      = exif_imagetype($file_data);

				$allowed_types         = array();
				$allowed_types['jpg']  = 'jpg';
				$allowed_types['jpeg'] = 'jpeg';
				$allowed_types['gif']  = 'gif';
				$allowed_types['png']  = 'png';

				// If not allowed type, skip
				if ( ! in_array(strtolower($type['ext']), $allowed_types) || ! $exif) {
					continue;
				}

				// Set compressing
				update_post_meta($attachment->ID, 'wps_ic_in_bulk', 'true');

				// Add queue
				$wps_ic->queue->remove_queue($attachment->ID);
				$queue = $wps_ic->queue->add_queue($attachment->ID, 'hidden_restore_bulk');
				$wps_ic->log->write_log($attachment->ID, 'Added to queue', '0', 'restore');

				if ($queue) {
					// Count files
					$count_files ++;
				}
			}

			// Cron Details
			update_option('wps_ic_compress_info', array('total' => $count_files, 'action' => 'Restoring'), 60 * 60 * 24);

			if ($response == 'none') {
				return;
			} else {
				if (empty($_POST['attachment_id'])) {
					$queue = $wps_ic->queue->get_hidden_restore_bulk_queue();
					wp_send_json_success($queue);
				} else {
					wp_send_json_success();
				}
			}
		}

		wp_send_json_error();
	}


	/**
	 * @since 4.0.0
	 */
	public function wps_ic_restore_hidden_bulk($response = true) {
		global $wps_ic, $wpdb;
		update_option('wps_ic_hidden_bulk_running', 'true');

		$queue = $wps_ic->queue->get_queue('hidden_restore_bulk');

		if ($queue) {
			wp_send_json_success(array('total_queued' => count($queue)));
		}

		if ($response) {
			wp_send_json_success(array('total_queued' => count($queue)));
		}
	}


	/**
	 * @since 4.0.3
	 */
	public function wps_ic_compress_fetch_thumbnails() {
		global $wps_ic, $wpdb;
		$apikey = $wps_ic::$api_key;

		$attachments = $_POST['attachments'];
		if (empty($attachments)) {
			wp_send_json_error();
		}

		$request_params                = array();
		$request_params['apiv3']       = 'true';
		$request_params['bulk']        = 'true';
		$request_params['apikey']      = $apikey;
		$request_params['action']      = 'fetch_thumbnails';
		$request_params['url']         = site_url();
		$request_params['attachments'] = json_encode($attachments);

		$params = array(
			'method'      => 'POST',
			'timeout'     => 30,
			'redirection' => 3,
			'sslverify'   => false,
			'httpversion' => '1.0',
			'blocking'    => true, // TODO: Mozda true?
			'headers'     => array(),
			'body'        => $request_params,
			'cookies'     => array()
		);

		// Send call to API
		$call = wp_remote_post('http://206.189.251.204/', $params);

		if (wp_remote_retrieve_response_code($call) == 200) {
			$body        = wp_remote_retrieve_body($call);
			$body        = json_decode($body, true);
			$attachments = $body['data'];

			if ( ! empty($attachments)) {
				foreach ($attachments as $attachment_ID => $data) {
					$c = $wps_ic->compress->update_thumbnail($attachment_ID, $data);
				}
			}

		}

		wp_send_json_success();
	}


	/**
	 * @since 4.0.0
	 */
	public function wps_ic_compress_next_in_queue() {
		global $wps_ic, $wpdb;

		ini_set('memory_limit', '2024M');
		ini_set('max_execution_time', '180');
		$start = microtime(true);

		$restore_queue    = $wps_ic->queue->get_next_queue('hidden_restore_bulk');
		$compress_queue   = $wps_ic->queue->get_next_queue('hidden_compress_bulk');
		$regenerate_queue = $wps_ic->queue->get_next_queue('hidden_regenerate');

		// Regenerate Process
		if ( ! empty($regenerate_queue)) {
			if ( ! function_exists('wp_generate_attachment_metadata')) {
				require_once ABSPATH . 'wp-admin/includes/image.php';
			}

			if ( ! function_exists('download_url')) {
				require_once(ABSPATH . "wp-admin" . '/includes/image.php');
				require_once(ABSPATH . "wp-admin" . '/includes/file.php');
				require_once(ABSPATH . "wp-admin" . '/includes/media.php');
			}

			$fetch = $wpdb->prepare("SELECT attachment_ID, type, status FROM " . $wpdb->prefix . "ic_queue WHERE type=%s AND (status=%s OR status=%s) ORDER BY attachment_ID DESC LIMIT 3", 'hidden_regenerate', 'queued', 'compressed');
			$items = $wpdb->get_results($fetch);

			$compress_queue = array();
			if ($items) {
				foreach ($items as $item) {
					if (in_array($item->attachment_ID, $compress_queue)) {
						continue;
					}

					$compress_queue[] = $item->attachment_ID;
					$fullsizepath     = get_attached_file($item->attachment_ID);
					wp_update_attachment_metadata((int)$item->attachment_ID, wp_generate_attachment_metadata((int)$item->attachment_ID, $fullsizepath));
					#$wps_ic->queue->remove_queue($item->attachment_ID);
					$wps_ic->queue->change_queue_status($item->attachment_ID, 'regenerated', array('code' => 'regenerated'));
				}

				#$wps_ic->compress->hidden_restore($item->attachment_ID);
				#$wps_ic->compress->bulk_restore(array('attachments' => $compress_queue));

				$end = microtime(true);
				wp_send_json_success(array('action' => 'regenerate', 'done' => $compress_queue, 'attachments' => $compress_queue, 'time' => ($end - $start)));
			}

			wp_send_json_error(array('regenerate_queue' => 'true', 'items' => print_r($items, true)));
		}

		if ( ! empty($restore_queue)) {
			$fetch = $wpdb->prepare("SELECT attachment_ID, type, status FROM " . $wpdb->prefix . "ic_queue WHERE type=%s AND (status=%s OR status=%s) ORDER BY attachment_ID DESC LIMIT 3", 'hidden_restore_bulk', 'queued', 'compressed');
			$items = $wpdb->get_results($fetch);

			$compress_queue = array();
			if ($items) {
				foreach ($items as $item) {
					if (in_array($item->attachment_ID, $compress_queue)) {
						continue;
					}
					$compress_queue[] = $item->attachment_ID;
				}

				#$wps_ic->compress->hidden_restore($item->attachment_ID);
				$wps_ic->compress->bulk_restore(array('attachments' => $compress_queue));

				$end = microtime(true);
				wp_send_json_success(array('action' => 'restore', 'done' => $compress_queue, 'attachments' => $compress_queue, 'time' => ($end - $start)));
			}

			wp_send_json_error(array('restore_queue' => 'true', 'items' => print_r($items, true)));
		}

		if ( ! empty($compress_queue)) {
			$fetch = $wpdb->prepare("SELECT attachment_ID, type, status FROM " . $wpdb->prefix . "ic_queue WHERE type=%s AND (status=%s OR status=%s) ORDER BY attachment_ID DESC LIMIT 3", 'hidden_compress_bulk', 'queued', 'restored');
			$items = $wpdb->get_results($fetch);

			$compress_queue = array();
			if ($items) {
				foreach ($items as $item) {
					if (in_array($item->attachment_ID, $compress_queue)) {
						continue;
					}
					$compress_queue[] = $item->attachment_ID;
				}

				$wps_ic->compress->bulk(array('attachments' => $compress_queue));

				$end = microtime(true);
				wp_send_json_success(array('action' => 'compress', 'done' => $compress_queue, 'attachments' => $compress_queue, 'time' => ($end - $start)));
			}

			wp_send_json_error(array('compress_queue' => 'true', 'items' => $items, 'time' => date('H:i:s')));

		}

		$end = microtime(true);
		update_option('wps_ic_bulk_ended', $end);

		delete_option('wps_ic_hidden_bulk_running');
		delete_option('wps_ic_compress_info');
		delete_option('ic_remote_action');
		delete_option('wps_ic_bulk_started');

		wp_send_json_error(array('restore_queue' => $restore_queue, 'compress_queue' => $compress_queue));
	}


	/**
	 * @since 4.0.0
	 */
	public function wps_ic_compress_next_in_queue_last_three() {
		global $wps_ic, $wpdb;

		$apikey = $wps_ic::$api_key;
		if (empty($apikey) || $apikey == 0) {
			wp_send_json_error();
		}

		ini_set('memory_limit', '2024M');
		ini_set('max_execution_time', '180');
		$start = microtime(true);

		$compress_queue = $wps_ic->queue->get_next_queue('hidden_compress_bulk');

		if ( ! empty($compress_queue)) {
			$fetch = $wpdb->prepare("SELECT attachment_ID, type, status FROM " . $wpdb->prefix . "ic_queue WHERE type=%s AND status=%s ORDER BY attachment_ID DESC LIMIT 3", 'hidden_compress_bulk', 'compressing');
			$items = $wpdb->get_results($fetch);

			$compress_queue = array();
			if ($items) {
				foreach ($items as $item) {
					if (in_array($item->attachment_ID, $compress_queue)) {
						continue;
					}
					$compress_queue[] = $item->attachment_ID;
				}

				$wps_ic->compress->bulk(array('attachments' => $compress_queue));

				$end = microtime(true);
				wp_send_json_success(array('action' => 'compress', 'done' => $compress_queue, 'attachments' => $compress_queue, 'time' => ($end - $start)));
			}

		}

		wp_send_json_error();
	}


	/**
	 * @since 4.0.0
	 */
	public function wps_ic_library_compress_next_in_queue() {
		global $wps_ic, $wpdb;

		ini_set('memory_limit', '2024M');
		ini_set('max_execution_time', '180');
		$start = microtime(true);

		$restore_queue  = $wps_ic->queue->get_next_queue('library_restore');
		$compress_queue = $wps_ic->queue->get_next_queue('library_compress');

		if ( ! empty($restore_queue)) {
			$fetch = $wpdb->prepare("SELECT attachment_ID, type, status FROM " . $wpdb->prefix . "ic_queue WHERE type=%s AND status=%s ORDER BY attachment_ID DESC LIMIT 3", 'library_restore', 'queued');
			$items = $wpdb->get_results($fetch);

			$compress_queue = array();
			if ($items) {
				foreach ($items as $item) {
					$compress_queue[] = $item->attachment_ID;
				}

				#$wps_ic->compress->hidden_restore($item->attachment_ID);
				$wps_ic->compress->bulk_restore(array('attachments' => $compress_queue));

				$end = microtime(true);
				wp_send_json_success(array('action' => 'restore', 'done' => $compress_queue, 'attachments' => $compress_queue, 'time' => ($end - $start)));
			}
		}

		if ( ! empty($compress_queue)) {
			$fetch = $wpdb->prepare("SELECT attachment_ID, type, status FROM " . $wpdb->prefix . "ic_queue WHERE type=%s AND status=%s ORDER BY attachment_ID DESC LIMIT 3", 'library_compress', 'queued');
			$items = $wpdb->get_results($fetch);

			$compress_queue = array();
			if ($items) {
				foreach ($items as $item) {
					$compress_queue[] = $item->attachment_ID;
				}

				$wps_ic->compress->bulk(array('attachments' => $compress_queue));

				$end = microtime(true);
				wp_send_json_success(array('action' => 'compress', 'done' => $compress_queue, 'attachments' => $compress_queue, 'time' => ($end - $start)));
			}

		}

		wp_send_json_error();
	}


	/**
	 * @since 4.0.0
	 */
	public function wps_ic_get_queue() {
		global $wps_ic, $wpdb;

		#$transient = get_transient('wps_ic_compress_info');

		$restore_queue  = $wps_ic->queue->get_next_queue('hidden_restore_bulk');
		$compress_queue = $wps_ic->queue->get_next_queue('hidden_compress_bulk');
		$regenerate     = $wps_ic->queue->get_next_queue('hidden_regenerate');

		$no_cron_set = get_option('no_cron_set');

		if ($regenerate) {
			$wps_ic->log->write_log('0', 'Get Queue');
			$queue      = $wps_ic->compress->get_queue();
			$queue_type = 'hidden_regenerate';
			$queue_done = $wps_ic->queue->get_done_queue($queue_type);

			if ($queue_done >= $queue['queue'] && $queue['queue'] > 0) {
				$queue_transient = get_option('wps_ic_compress_info');

				$started = get_option('wps_ic_bulk_started');
				$end     = microtime(true);
				update_option('wps_ic_bulk_ended', $end);

				delete_option('wps_ic_hidden_bulk_running');
				delete_option('wps_ic_compress_info');
				delete_option('ic_remote_action');

				wp_send_json_error(array(
														 'error'      => 'queue done >= queue',
														 'action'     => $queue_transient['action'],
														 'queue_done' => $queue_done,
														 'queue'      => $queue['queue'],
														 'cron_set'   => $no_cron_set,
														 'time'       => ($end - $started),
														 'started'    => $started,
														 'ended'      => $end
													 ));
			}

			// Prepare query
			$attachments = array();
			$fetch       = $wpdb->prepare("SELECT attachment_ID, type, status FROM " . $wpdb->prefix . "ic_queue WHERE type=%s AND status=%s AND in_cron=%s ORDER BY attachment_ID DESC", $queue_type, 'queued', '0');
			$fetch       = $wpdb->get_results($fetch);

			if ($fetch) {
				foreach ($fetch as $item) {
					$attachments[] = $item->attachment_ID;
				}
			}

			// Check for attachments already in cron
			if ( ! empty($queue) && $queue['total'] > 0) {
				wp_send_json_success(array(
															 'total'  => $queue['total'],
															 'queue'  => $queue['queue'],
															 'done'   => $queue['done'],
															 'action' => $queue['action']
														 ));
			}

			if (empty($attachments) || count($attachments) == 0) {
				$queue_transient = get_option('wps_ic_compress_info');
				delete_option('wps_ic_hidden_bulk_running');
				delete_option('wps_ic_compress_info');
				delete_option('ic_remote_action');
				delete_option('wps_ic_bulk_started');
				wp_send_json_error(array('error' => 'empty attachments', 'action' => $queue_transient['action'], 'queue_done' => $queue['done'], 'queue' => $queue['queue'], 'cron_set' => $no_cron_set));
			}

		}

		if ($restore_queue || $compress_queue) {

			$wps_ic->log->write_log('0', 'Get Queue');
			$queue = $wps_ic->compress->get_queue();

			if ($restore_queue) {
				$queue_type = 'hidden_restore_bulk';
			} else if ($compress_queue) {
				$queue_type = 'hidden_compress_bulk';
			}

			$queue_done = $wps_ic->queue->get_done_queue($queue_type);

			if ($queue_done >= $queue['queue'] && $queue['queue'] > 0) {
				$queue_transient = get_option('wps_ic_compress_info');

				$started = get_option('wps_ic_bulk_started');
				$end     = microtime(true);
				update_option('wps_ic_bulk_ended', $end);

				delete_option('wps_ic_hidden_bulk_running');
				delete_option('wps_ic_compress_info');
				delete_option('ic_remote_action');

				wp_send_json_error(array(
														 'error'      => 'queue done >= queue',
														 'action'     => $queue_transient['action'],
														 'queue_done' => $queue_done,
														 'queue'      => $queue['queue'],
														 'cron_set'   => $no_cron_set,
														 'time'       => ($end - $started),
														 'started'    => $started,
														 'ended'      => $end
													 ));
			}

			if ($no_cron_set == 'true') {

				// Prepare query
				$attachments = array();
				$fetch       = $wpdb->prepare("SELECT attachment_ID, type, status FROM " . $wpdb->prefix . "ic_queue WHERE type=%s AND status=%s AND in_cron=%s ORDER BY attachment_ID DESC", $queue_type, 'queued', '0');
				$fetch       = $wpdb->get_results($fetch);

				if ($fetch) {
					foreach ($fetch as $item) {
						$attachments[] = $item->attachment_ID;
					}
				}

				// Check for attachments already in cron
				if ( ! empty($queue) && $queue['total'] > 0) {
					wp_send_json_success(array(
																 'cron_set' => $no_cron_set,
																 'total'    => $queue['total'],
																 'queue'    => $queue['queue'],
																 'done'     => $queue['done'],
																 'action'   => $queue['action']
															 ));
				}

				if (empty($attachments) || count($attachments) == 0) {
					$queue_transient = get_option('wps_ic_compress_info');
					delete_option('wps_ic_hidden_bulk_running');
					delete_option('wps_ic_compress_info');
					delete_option('ic_remote_action');
					delete_option('wps_ic_bulk_started');
					wp_send_json_error(array('error' => 'empty attachments', 'action' => $queue_transient['action'], 'queue_done' => $queue['done'], 'queue' => $queue['queue'], 'cron_set' => $no_cron_set));
				}

			} else {

				// Prepare query
				$attachments = array();
				$fetch       = $wpdb->prepare("SELECT attachment_ID, type, status FROM " . $wpdb->prefix . "ic_queue WHERE type=%s AND status=%s AND in_cron=%s ORDER BY attachment_ID DESC", $queue_type, 'queued', '0');
				$fetch       = $wpdb->get_results($fetch);

				if ($fetch) {
					foreach ($fetch as $item) {
						$attachments[] = $item->attachment_ID;
					}
				}

				wp_send_json_success(array(
															 'cron_set'          => $no_cron_set,
															 'attachments_array' => $attachments,
															 'total'             => $queue['total'],
															 'queue'             => $queue['queue'],
															 'done'              => $queue['done'],
															 'action'            => $queue['action']
														 ));

			}
		} else {
			/*
			$queue_transient = get_option('wps_ic_compress_info');
			delete_option('wps_ic_hidden_bulk_running');
			delete_option('wps_ic_compress_info');
			delete_option('ic_remote_action');

			$started = get_option('wps_ic_bulk_started');
			$end     = microtime(true);
			update_option('wps_ic_bulk_ended', $end);
*/

			wp_send_json_error(array('action' => 'nothing_to_compress'));
		}
	}


	/**
	 * Exclude the image
	 * @since 4.0.0
	 */
	public
	function wps_ic_exclude_image() {
		global $wps_ic;
		do_action('ic_debug', $_POST['attachment_id'], 'Triggered Exclude');
		$wps_ic->compress->exclude($_POST, 'html');
	}


	/**
	 * Connect Multsites With API
	 */
	public
	function wps_ic_api_mu_connect() {
		global $wps_ic;

		// Is localhost?
		$whitelist = array('127.0.0.1', '::1');
		if (in_array($_SERVER['REMOTE_ADDR'], $whitelist)) {
			#wp_send_json_error(array('msg' => 'Sorry, localhost installs are not supported.'));
		}

		$sites = get_sites();

		// API Key
		$apikey         = sanitize_text_field($_POST['apikey']);
		$affiliate_code = get_option('wps_ic_affiliate_code');

		if ($sites && is_multisite()) {
			$error = false;

			foreach ($sites as $key => $site) {

				// Setup URI
				$uri = WPS_IC_KEYSURL;
				$uri .= '?action=connect';
				$uri .= '&apikey=' . $apikey;
				$uri .= '&site=' . urlencode($site->domain . $site->path);
				$uri .= '&affiliate_code=' . $affiliate_code;

				// Verify API Key is our database and user has is confirmed getresponse
				$get = wp_remote_get($uri, array('timeout' => 20, 'sslverify' => false));

				if (wp_remote_retrieve_response_code($get) == 200) {
					$body = wp_remote_retrieve_body($get);
					$body = json_decode($body);

					if ($body->success == true && $body->data->api_key != '' && $body->data->response_key != '') {
						$wps_ic->options->set_option('api_key', $body->data->api_key);
						$wps_ic->options->set_option('response_key', $body->data->response_key);
						$wps_ic->options->set_option('orp', $body->data->orp);

						$settings = get_option(WPS_IC_SETTINGS);

						$sizes = get_intermediate_image_sizes();
						foreach ($sizes as $key => $value) {
							$settings['thumbnails'][ $value ] = 1;
						}

						update_option(WPS_IC_SETTINGS, $settings);

						//delete_option('wps_ic_affiliate_code');
					}
				} else {
					$error = true;
				}

			}

			if ($error == true) {
				wp_send_json_error($body->data);
			} else {
				wp_send_json_success();
			}

		}

		wp_send_json_error('0');
	}


	/**
	 * API Tests
	 */
	public
	function wps_ic_api_test() {
		global $wps_ic;
		$wps_ic = new wps_ic();

		$test = sanitize_text_field($_POST['test_id']);

		$wps_ic->tests->call_test($test);
	}


	/**
	 * Connect With API
	 */
	public
	function wps_ic_api_connect() {
		global $wps_ic;

		delete_option('wps_ic_test');

		// Is localhost?
		$whitelist = array('127.0.0.1', '::1');
		if (in_array($_SERVER['REMOTE_ADDR'], $whitelist)) {
			#wp_send_json_error(array('msg' => 'Sorry, localhost installs are not supported.'));
		}

		// API Key
		$apikey         = sanitize_text_field($_POST['apikey']);
		$affiliate_code = get_option('wps_ic_affiliate_code');

		// Setup URI
		$uri = WPS_IC_KEYSURL;
		$uri .= '?action=connect';
		$uri .= '&apikey=' . $apikey;
		$uri .= '&site=' . urlencode(site_url());
		$uri .= '&affiliate_code=' . $affiliate_code;

		// Verify API Key is our database and user has is confirmed getresponse
		$get = wp_remote_get($uri, array('timeout' => 20, 'sslverify' => false));

		if (wp_remote_retrieve_response_code($get) == 200) {
			$body = wp_remote_retrieve_body($get);
			$body = json_decode($body);

			if ($body->success == true && $body->data->api_key != '' && $body->data->response_key != '') {
				$wps_ic->options->set_option('api_key', $body->data->api_key);
				$wps_ic->options->set_option('response_key', $body->data->response_key);
				$wps_ic->options->set_option('orp', $body->data->orp);

				// First check if CDN Zone already exists
				$call = wp_remote_get(WPS_IC_APIURL . '?cdn_exists=true&domain=' . site_url(), array('timeout' => '10', 'sslverify' => false));
				if (wp_remote_retrieve_response_code($call) == 200) {
					$cdn_body = wp_remote_retrieve_body($call);
					$cdn_body = json_decode($cdn_body);

					if ($cdn_body->success == 'true') {
						// CDN Does exist or we just created it
					} else {
						// CDN Does not exist,error
					}
				}

				$settings = get_option(WPS_IC_SETTINGS);

				$sizes = get_intermediate_image_sizes();
				foreach ($sizes as $key => $value) {
					$settings['thumbnails'][ $value ] = 1;
				}

				update_option(WPS_IC_SETTINGS, $settings);

				//delete_option('wps_ic_affiliate_code');
				wp_send_json_success();
			}

			wp_send_json_error($body->data);
		}

		wp_send_json_error('0');
	}


	/**
	 * Signup to API
	 */
	public
	function wps_ic_api_signup() {
		global $wps_ic;

		// Sanitize
		$email = sanitize_email($_POST['email']);
		#$name  = sanitize_text_field($_POST['name']);
		$name           = '';
		$affiliate_code = sanitize_text_field($_POST['affiliate_code']);
		update_option('wps_ic_affiliate_code', $affiliate_code);

		// Is valid email
		if ( ! is_email($email)) {
			wp_send_json_error(array('code' => 100, 'msg' => 'Invalid e-mail!'));
		}

		$uri = WPS_IC_KEYSURL;
		$uri .= '?action=signup';
		$uri .= '&email=' . $email;
		$uri .= '&name=' . $name;
		$uri .= '&site=' . urlencode(site_url());

		$get = wp_remote_get($uri, array('timeout' => 25, 'sslverify' => false));

		if (wp_remote_retrieve_response_code($get) == 200) {
			$body = wp_remote_retrieve_body($get);
			$body = json_decode($body);

			if ($body->success == true) {

				$response      = $body->data->code;
				$referral_code = $body->data->referral_code;

				if ( ! empty($referral_code)) {
					update_option('wps_ic_ref_id', $referral_code);
				}

				if ($response == 110) {
					// Already subscribed and confirmed
					wp_send_json_error(array('code' => 110, 'msg' => 'You are already subscribed!<br/><a href="https://wpcompress.com/lost-api-key/" target="_blank">Retrieve API Key</a>'));
				} else {
					if ($response == 100) {
						// Subscribed but not confirmed
						wp_send_json_success(array('code' => 100, 'msg' => 'We’ve sent your API key to your email - please copy it and confirm!'));
					} else {
						if ($response == 111) {
							// Subscribed but was not in our database
							wp_send_json_error(array('code' => 111, 'msg' => 'You are already subscribed, but you were not found in our database!'));
						} else {
							if ($response == 200) {
								// Subscribed, waiting for confirmation
								wp_send_json_success(array('code' => 200, 'msg' => 'Thanks! Sent confirmation!'));
							} else {
								if ($response == 202) {
									// Subscribed, waiting for confirmation
									wp_send_json_error(array('code' => 202, 'msg' => 'You are already subscribed and confirmed!<br/><a href="https://wpcompress.com/lost-api-key/" target="_blank">Retrieve API Key</a>'));
								} else {
									// Subscribed, waiting for confirmation
									wp_send_json_error(array('code' => 202, 'msg' => 'Unkown error occured!'));
								}
							}
						}
					}
				}

			}

			// Uknown Error
			wp_send_json_error(array('code' => 0));
		}

		wp_send_json_error(array('code' => wp_remote_retrieve_response_code($get), 'body' => wp_remote_retrieve_body($get)));
	}


	/**
	 * Get settings
	 */
	public
	function wps_ic_get_setting() {
		global $wps_ic;
		$what     = sanitize_text_field($_POST['what']);
		$settings = $wps_ic->options->get_settings();

		wp_send_json_success($settings[ $what ]);
	}


	/**
	 * Change Settings Value
	 */
	public
	function wps_ic_settings_change() {
		global $wps_ic;

		$what     = sanitize_text_field($_POST['what']);
		$value    = sanitize_text_field($_POST['value']);
		$checked  = sanitize_text_field($_POST['checked']);
		$checkbox = sanitize_text_field($_POST['checkbox']);

		$settings = $wps_ic->options->get_settings();

		if ($what == 'thumbnails') {

			if ( ! isset($value) || empty($value)) {
				$settings['thumbnails'] = array();
			} else {
				$settings['thumbnails'] = array();
				$value                  = rtrim($value, ',');
				$value                  = explode(',', $value);
				foreach ($value as $i => $thumb_size) {
					$settings['thumbnails'][ $thumb_size ] = 1;
				}

			}

		} else {

			if ($what == 'autopilot') {
				if ($checked == 'checked') {

				} else {
					$settings['otto'] = 'automated';
				}
			}

			if ($checkbox == 'true') {
				if ($checked === 'false') {
					$settings[ $what ] = 0;
				} else {
					$settings[ $what ] = 1;
				}
			} else {
				$settings[ $what ] = $value;
			}
		}

		if ($what == 'generate_webp' || $what == 'adaptive_images' || $what == 'cdn') {
			global $wpdb;
			$cache = $wpdb->query("DELETE FROM " . $wpdb->postmeta . " WHERE meta_key='_ic_sources' OR meta_key='_ic_cdn_content'");

			$attachments = $wpdb->get_results("SELECT ID FROM " . $wpdb->posts . " WHERE post_type='attachment' AND post_status='inherit'");

			$queued       = 0;
			$total_queued = 0;
			$regen_array  = array();
			$delay        = 0;
			if ($attachments) {
				foreach ($attachments as $attachment) {

					$delay += 25;

					$compressed = get_post_meta($attachment->ID, 'wps_ic_data', true);
					if ( ! $compressed || $compressed == 'not_able') {
						continue;
					}

					$queued ++;
					$total_queued ++;

					$regen_array[] = $attachment->ID;

					if ($queued >= 3) {
						wp_schedule_single_event(time(), 'wps_ic_regen_thumbs_by_array', array('attachments' => $regen_array));

						$regen_array = array();
						$queued      = 0;
					}
				}

				if ($queued <= 3) {
					if ( ! empty($regen_array) && count($regen_array) > 0) {
						wp_schedule_single_event(time(), 'wps_ic_regen_thumbs_by_array', array('attachments' => $regen_array));
					}
				}
			}
		}

		update_option(WPS_IC_SETTINGS, $settings);

		wp_send_json_success();
	}


	/**
	 * Change Notify Setting Value
	 */
	public
	function wps_ic_notify() {
		global $wps_ic;

		$checkbox = 'true';
		$checked  = sanitize_text_field($_POST['checked']);

		$settings = $wps_ic->options->get_settings();

		if ($checked === 'false') {
			$settings['notify'] = 0;
		} else {
			$settings['notify'] = 1;
		}

		update_option(WPS_IC_SETTINGS, $settings);

		wp_send_json_success();
	}


	/**
	 * @since 4.0.0
	 */
	public function wps_ic_regenerate_thumbnails_bulk($response = true) {
		global $wps_ic, $wpdb;
		update_option('wps_ic_hidden_bulk_running', 'true');

		$queue = $wps_ic->queue->get_queue('hidden_regenerate');

		if ($queue) {
			wp_send_json_success(array('total_queued' => count($queue)));
		}

		if ($response) {
			wp_send_json_success(array('total_queued' => count($queue)));
		}
	}


	/**
	 * @since 4.0.0
	 */
	public function wps_ic_compress_hidden_bulk($response = true) {
		global $wps_ic, $wpdb;
		update_option('wps_ic_hidden_bulk_running', 'true');

		$queue = $wps_ic->queue->get_queue('hidden_compress_bulk');

		if ($queue) {
			wp_send_json_success(array('total_queued' => count($queue)));
		}

		if ($response) {
			wp_send_json_success(array('total_queued' => count($queue)));
		}
	}


	/**
	 * @since 4.0.0
	 */
	public function wps_ic_compress_bulk_prep_background() {
		global $wps_ic, $wpdb;

		$count_files = 0;

		delete_option(WPS_IC_STATS_BULK_FILES);
		delete_option(WPS_IC_STATS_BULK_TOTAL_FILES);
		delete_option(WPS_IC_STATS_BULK_SAVINGS);
		delete_option(WPS_IC_STATS_BULK_AVG);

		// Clear whole queue
		if ( ! empty($_POST['attachment_id'])) {
			$queue = $wps_ic->queue->get_compress_queue();
			wp_send_json_success(array('queue' => $queue));
		} else {
			$queue = $wps_ic->queue->get_compress_bulk_queue();
			update_option(WPS_IC_BULK_COMPRESS_QUEUE, $queue);
			wp_send_json_success(array('queue' => $queue));
		}

		wp_send_json_error();
	}


	/**
	 * @since 4.0.0
	 */
	public function wps_ic_regenerate_thumbnails_prep() {
		global $wps_ic, $wpdb;

		update_option('wps_ic_bulk_started', microtime(true));
		update_option('wps_ic_hidden_bulk_running', 'true');

		$count_files = 0;

		delete_option(WPS_IC_STATS_BULK_FILES);
		delete_option(WPS_IC_STATS_BULK_TOTAL_FILES);
		delete_option(WPS_IC_STATS_BULK_SAVINGS);
		delete_option(WPS_IC_STATS_BULK_AVG);

		// Clear whole queue
		$wps_ic->queue->clear_queue('hidden_regenerate');
		$wps_ic->queue->clear_queue('hidden_compress_bulk');
		$wps_ic->queue->clear_queue('compress_bulk');
		$wps_ic->queue->clear_queue('restore_bulk');
		$wps_ic->queue->clear_queue('hidden_restore_bulk');
		$wps_ic->queue->clear_queue('generate_thumbnail');

		/*
		 * Check if it's a single file clicked or bulk compress button
		 */
		$attachments = $wpdb->get_results("SELECT ID FROM " . $wpdb->prefix . "posts WHERE post_type='attachment' AND post_status='inherit' ORDER BY post_date DESC");

		// Count attachments and check if user has enough credits
		$count_attachments = count($attachments);

		if ($attachments) {
			foreach ($attachments as $attachment) {

				// Allowed type
				$file_data = get_attached_file($attachment->ID);
				$type      = wp_check_filetype($file_data);

				$allowed_types         = array();
				$allowed_types['jpg']  = 'jpg';
				$allowed_types['jpeg'] = 'jpeg';
				$allowed_types['gif']  = 'gif';
				$allowed_types['png']  = 'png';

				if ( ! in_array(strtolower($type['ext']), $allowed_types)) {
					continue;
				}

				// Add queue
				$wps_ic->queue->remove_queue($attachment->ID);
				$queue = $wps_ic->queue->add_queue($attachment->ID, 'hidden_regenerate');

				$count_files ++;
			}

			// Cron Details
			update_option('wps_ic_compress_info', array('total' => $count_files, 'action' => 'Regenerating'), 60 * 60 * 24);

			if ( ! empty($_POST['attachment_id'])) {
				$queue = $wps_ic->queue->get_hidden_regenerate_bulk_queue();
				wp_send_json_success(array('queue' => $queue));
			} else {
				$queue = $wps_ic->queue->get_hidden_regenerate_bulk_queue();
				wp_send_json_success(array('queue' => $queue));
			}

		}

		wp_send_json_error();
	}


	/**
	 * @param string $response
	 *
	 * @since 4.0.0
	 */
	public function wps_ic_compress_bulk_prep_hidden_background($response = 'json') {
		global $wps_ic, $wpdb;

		update_option('wps_ic_bulk_started', microtime(true));
		update_option('wps_ic_hidden_bulk_running', 'true');

		$count_files = 0;

		delete_option(WPS_IC_STATS_BULK_FILES);
		delete_option(WPS_IC_STATS_BULK_TOTAL_FILES);
		delete_option(WPS_IC_STATS_BULK_SAVINGS);
		delete_option(WPS_IC_STATS_BULK_AVG);

		// Clear whole queue
		$wps_ic->queue->clear_queue('hidden_regenerate');
		$wps_ic->queue->clear_queue('hidden_compress_bulk');
		$wps_ic->queue->clear_queue('compress_bulk');
		$wps_ic->queue->clear_queue('restore_bulk');
		$wps_ic->queue->clear_queue('hidden_restore_bulk');
		$wps_ic->queue->clear_queue('generate_thumbnail');

		/*
		 * Check if it's a single file clicked or bulk compress button
		 */
		$attachments = $wpdb->get_results("SELECT ID FROM " . $wpdb->prefix . "posts WHERE post_type='attachment' AND post_status='inherit' ORDER BY post_date DESC");

		// Count attachments and check if user has enough credits
		$count_attachments = count($attachments);

		// Total to compress
		$required_credits = 1;

		update_option('wps_ic_reserved_credits', $count_attachments);

		// Does user have enough credits?
		$check_credits = wp_remote_get(WPS_IC_APIURL . '?apikey=' . $wps_ic::$api_key . '&check_credits=true', array('timeout' => '25', 'sslverify' => false));

		if (wp_remote_retrieve_response_code($check_credits) == 200) {
			$body = wp_remote_retrieve_body($check_credits);
			$body = json_decode($body, true);

			if ($body['success'] == 'true') {
				// OK
				$credits = $body['data']['credits'];

				if ($credits == 0) {
					// Write Log
					wp_send_json_error('not_enough_credits');
				}

				// User does not have enough credits
				if ($required_credits > $credits) {
					// Write Log
				}

			}
		} else {
			wp_send_json_error('api_offline');
		}

		$available_credits = $credits;
		$credits_flag      = 0;

		if ($attachments) {
			foreach ($attachments as $attachment) {

				$required_credits_per_image = 1;
				if ($available_credits < $required_credits_per_image) {
					$queue        = $wps_ic->queue->get_compress_bulk_queue();
					$credits_flag = 1;
					break;
				}

				$excluded = get_post_meta($attachment->ID, 'wps_ic_exclude', true);
				if ( ! empty($excluded) && $excluded == 'true') {
					continue;
				}

				$data = get_post_meta($attachment->ID, 'wps_ic_data', true);
				if ( ! empty($data) && ! empty($data['new'])) {
					continue;
				}

				if ($data == 'not_able') {
					continue;
				}

				$available_credits = $available_credits - $required_credits_per_image;

				// Allowed type
				$file_data = get_attached_file($attachment->ID);
				$type      = wp_check_filetype($file_data);

				$allowed_types         = array();
				$allowed_types['jpg']  = 'jpg';
				$allowed_types['jpeg'] = 'jpeg';
				$allowed_types['gif']  = 'gif';
				$allowed_types['png']  = 'png';

				if ( ! in_array(strtolower($type['ext']), $allowed_types)) {
					continue;
				}

				// Set compressing
				update_post_meta($attachment->ID, 'wps_ic_in_bulk', 'true');

				// Add queue
				$wps_ic->queue->remove_queue($attachment->ID);
				$queue = $wps_ic->queue->add_queue($attachment->ID, 'hidden_compress_bulk');

				$count_files ++;
			}

			// Cron Details
			update_option('wps_ic_compress_info', array('total' => $count_files, 'action' => 'Compressing'), 60 * 60 * 24);

			if ($response == 'none') {
				return;
			} else {

				if ($credits_flag) {
					wp_send_json_success(array('queue' => $queue, 'msg' => 'not_enough_credits_notification'));
				}

				if ( ! empty($_POST['attachment_id'])) {
					$queue = $wps_ic->queue->get_hidden_compress_bulk_queue();
					wp_send_json_success(array('queue' => $queue));
				} else {
					$queue = $wps_ic->queue->get_hidden_compress_bulk_queue();
					wp_send_json_success(array('queue' => $queue));
				}

			}
		}

		wp_send_json_success(array('queue' => $queue, 'msg' => 'not_enough_credits_notification'));
	}


	/**
	 * Restore the image
	 */
	public function wps_ic_restore_single_btn() {
		global $wps_ic;

		// Change memory limits
		$wps_ic->change_limits();

		if (empty($_POST['type'])) {
			$type = 'restore';
		} else {
			$type = sanitize_text_field($_POST['type']);
		}

		if ($type == 'reset') {
			# Working but trying better solution
			#$wps_ic->compress->reset($_POST['attachment_id'], 'table');
		}

		/*
		 * Check if it's a single file clicked or bulk compress button
		 */
		$queued = array();

		$attachments[] = (object)array('ID' => sanitize_text_field($_POST['attachment_id']));
		$attachments   = (object)$attachments;

		if ($attachments) {
			foreach ($attachments as $attachment) {

				if ($type == 'reset') {
					update_post_meta($attachment->ID, 'wps_ic_reset', 'true');
				}

				// Set compressing
				update_post_meta($attachment->ID, 'wps_ic_restoring', 'true');
				update_post_meta($attachment->ID, 'wps_ic_in_bulk', 'true');

				// Add to Queue
				$wps_ic->queue->remove_queue($attachment->ID);
				$wps_ic->queue->add_queue($attachment->ID, 'library_restore');
				$wps_ic->compress->bulk_restore(array('attachments' => array($attachment->ID)));

				$queued[] = $attachment->ID;
			}

			wp_send_json_success(array('list' => $attachments, 'queued' => $queued));
		}

		wp_send_json_error();
	}


	/**
	 * Authorize site with remote api
	 */
	public function wps_ic_authorize_api() {
		global $wps_ic;

		$site    = site_url();
		$api_key = $this->options->get_option('api_key');

		$call = wp_remote_get(WPS_IC_APIURL . '?authorize=true&site=' . urlencode($site) . '&api_key=' . $api_key, array('timeout' => 25, 'sslverify' => false));

		if (wp_remote_retrieve_response_code($call) == 200) {
			$body = wp_remote_retrieve_body($call);
			$body = json_decode($body);

			$wps_ic->options->set_defaults();
			$wps_ic->options->set_option('response_key', $body->data);
			$wps_ic->options->api_key('response_key', $api_key);
			wp_send_json_success();
		}

		wp_send_json_error('Oops! There has been an error in authorize api!');
	}


	/**
	 * Deauthorize site with remote api
	 */
	public function wps_ic_deauthorize_api() {
		global $wps_ic;

		// Vars
		$site   = site_url();
		$apikey = $wps_ic->options->get_option('api_key');

		// Setup URI
		$uri = WPS_IC_KEYSURL;
		$uri .= '?action=disconnect';
		$uri .= '&apikey=' . $apikey;
		$uri .= '&site=' . urlencode($site);

		// Verify API Key is our database and user has is confirmed getresponse
		$get = wp_remote_get($uri, array('timeout' => 20, 'sslverify' => false));

		#if (wp_remote_retrieve_response_code($get) == 200) {
		#$body = wp_remote_retrieve_body($get);
		#$body = json_decode($body);

		#if ($body->success == true) {
		$wps_ic->options->set_option('api_key', '');
		$wps_ic->options->set_option('response_key', '');
		$wps_ic->options->set_option('orp', '');
		//
		#wp_send_json_success();
		#}

		#wp_send_json_success();
		#}

		#wp_send_json_error('We were not able to communicate with api!');
	}


	/**
	 * Compress the image
	 */
	public function wps_ic_single_compress_btn() {
		global $wps_ic;

		// Change memory limits
		$wps_ic->change_limits();

		/*
		 * Check if it's a single file clicked or bulk compress button
		 */
		$queued = array();

		$attachments[] = (object)array('ID' => sanitize_text_field($_POST['attachment_id']));
		$attachments   = (object)$attachments;

		if ($attachments) {
			foreach ($attachments as $attachment) {

				$wps_ic->queue->remove_queue($attachment->ID);

				// Set compressing
				#update_post_meta($attachment->ID, 'wps_ic_compressing', 'true');
				#update_post_meta($attachment->ID, 'wps_ic_in_bulk', 'true');

				// Add to Queue
				$wps_ic->queue->add_queue($attachment->ID, 'library_compress');
				$response = $wps_ic->compress->bulk(array('attachments' => array($attachment->ID)), '', true);

				$queued[] = $attachment->ID;
			}

			wp_send_json_success(array('attachments' => $queued, 'thumbs' => $response));
		}

		wp_send_json_error();
	}


	/**
	 * Verify Orp With API
	 */
	public function wps_ic_orp() {
		global $wps_ic;

		$package = get_transient('wps_ic_pro');

		if (empty($package)) {

			// Setup URI
			$uri = WPS_IC_KEYSURL;
			$uri .= '?action=orp';
			$uri .= '&apikey=' . $wps_ic::$api_key;
			$uri .= '&site=' . urlencode(site_url());

			// Verify API Key is our database and user has is confirmed getresponse
			$get = wp_remote_get($uri, array('timeout' => 20, 'sslverify' => false));

			if (wp_remote_retrieve_response_code($get) == 200) {
				$body = wp_remote_retrieve_body($get);
				$body = json_decode($body);
				if ($body->success == true) {
					return $body->data;
				}

				return false;
			}

			return false;
		} else {
			return true;
		}
	}

}