<?php


/**
 * Class - Cron
 */
class wps_ic_cron {

  public function __construct() {
    $options = get_option(WPS_IC_OPTIONS);

    if ( ! empty($options['response_key'])) {
      $this->response_key = $options['response_key'];
    }

    if ( ! empty($this->response_key)) {
      add_action('wps_ic_upload_image', array($this, 'cron_on_upload'), 10, 1);
      add_action('wps_ic_fetch_thumbnails', array($this, 'cron_fetch_thumbnails'), 10, 1);
      add_action('wps_ic_rebuild_thumbs', array($this, 'cron_regenerate_thumbnails'), 10, 1);

      add_action('wps_ic_owl', array($this, 'trigger_owl'), 10, 1);

      // Cron that fetches Queue and adds Cron Schedule
      add_action('wps_ic_compress_queue', array($this, 'trigger_compress_cron'), 1, 1);

      // Cron that fetches Queue and runs Compress
      add_action('wps_ic_regen_thumbs_by_array', array($this, 'wps_ic_regen_thumbs_by_array'), 1, 1);
      add_action('wps_ic_compress_single', array($this, 'trigger_compress_single'), 1, 1);

      add_action('wps_ic_restore_single', array($this, 'trigger_restore_single'), 1, 1);
      add_action('wps_ic_restore_queue', array($this, 'trigger_restore_cron'), 1, 1);

      // Cron for remote actions
      add_action('wps_ic_remote_restore_all', array($this, 'remote_restore_all'));
      add_action('wps_ic_remote_compress_all', array($this, 'remote_compress_all'));

      // CDN
      add_action('wps_ic_regen_thumbs', array($this, 'regen_thumbs'), 1, 1);
      add_action('wps_ic_trigger_push_cdn', array($this, 'push_cdn'), 1, 1);
      add_action('wps_ic_rebuild_cdn', array($this, 'rebuild_cdn'));
      add_action('wps_ic_check_cdn_status', array($this, 'check_cdn_status'));
      add_action('wps_ic_wake_up_cron_visit', array($this, 'wake_up_cron'));
    }
  }


  public function wake_up_cron() {
    foreach (_get_cron_array() as $timestamp => $events) {

      foreach ($events as $event_hook => $event_args) {
        foreach ($event_args as $event) {

          if ($event_hook == 'wps_ic_regen_thumbs_by_array') {
            foreach ($event_args as $hash => $event_Args) {

              if ( ! empty($event_Args['args'])) {
                delete_transient('doing_cron');
                set_transient('ic_cron_working', 'true', 20);
                do_action_ref_array($event_hook, $event_Args['args']);
              }

              wp_send_json_success('regen_thumbs');
            }
          }

          if ($event_hook == 'wps_ic_compress_single') {
            foreach ($event_args as $hash => $event_Args) {

              if ( ! empty($event_Args['args'])) {
                delete_transient('doing_cron');
                set_transient('ic_cron_working', 'true', 20);
                do_action_ref_array($event_hook, $event_Args['args']);
              }

              wp_send_json_success('compress');
            }
          }

          if ($event_hook == 'wps_ic_restore_single') {
            foreach ($event_args as $hash => $event_Args) {

              if ( ! empty($event_Args['args'])) {
                delete_transient('doing_cron');
                #set_transient('ic_cron_working', 'true', 6);
                do_action_ref_array($event_hook, $event_Args['args']);
              }

              wp_send_json_success('restore');
            }
          }

        }

      }

    }

    $get = wp_remote_get(site_url('wp-cron.php?doing_wp_cron'), array('timeout' => 20));
    wp_send_json_success();
  }


  public function check_cdn_status() {
    #delete_transient('wps_ic_cdn_check');
    $cdn_check_transient = get_transient('wps_ic_cdn_check');

    if ( ! $cdn_check_transient) {
      // Check CDN Status
      $call = wp_remote_get(WPS_IC_APIURL . '?cdn_status=true&domain=' . site_url(), array('timeout' => '10', 'sslverify' => false));
      if (wp_remote_retrieve_response_code($call) == 200) {
        set_transient('wps_ic_cdn_check', 'true', 60 * 2);
        $body = wp_remote_retrieve_body($call);
        $body = json_decode($body);

        if ( ! $body->success) {

          // CDN Does exist or we just created it
          $settings        = get_option(WPS_IC_SETTINGS);
          $settings['cdn'] = 0;
          update_option(WPS_IC_SETTINGS, $settings);
          set_transient('wps_ic_cdn_check_status', 'failed', 60 * 2);

        } else {

        }
      }
    }

  }


  public function wps_ic_regen_thumbs_by_array($attachments) {
    return true;
  }


  public function regen_thumbs() {
    return true;
  }


  public function rebuild_cdn($attachmentID = '', $return = false) {
    global $wps_ic;

    if ( ! defined('ABSPATH')) {
      /** Set up WordPress environment */
      require_once(dirname(__FILE__) . '/wp-load.php');
    }

    if ( ! defined('WP_MEMORY_LIMIT')) {
      define('WP_MEMORY_LIMIT', '1024M');
    }

    ini_set('memory_limit', '1024M');
    ini_set('max_execution_time', '3600');

    if (empty($attachmentID)) {
      $attachments = get_posts(array(
                                 'post_type'      => 'attachment',
                                 'posts_per_page' => '50',
                                 'meta_query'     => array(
                                   array(
                                     'key'   => 'wps_ic_docdn',
                                     'value' => 'true'
                                   )
                                 )
                               ));
    } else {
      $attachment  = get_post($attachmentID);
      $attachments = array($attachment);
      $attachments = (object)$attachments;
    }

    if ( ! empty($attachments)) {
      foreach ($attachments as $attachment) {
        $attachment_id = $attachmentID = $attachment->ID;

        $compressed = get_post_meta($attachment_id, 'wps_ic_data', true);
        if (empty($compressed) || $compressed == 'not_able') {
          continue;
        }

        $thumbs = array();
        $sizesa = get_intermediate_image_sizes();

        // Addition for doing only selected sizes
        $settings = get_option(WPS_IC_SETTINGS);
        $sizes    = $settings['thumbnails'];

        if (empty($sizes) || ! isset($sizes)) {
          $thumbs['full'] = wp_get_attachment_image_src($attachment_id, 'full');
          $uri            = explode('?', $thumbs['full'][0]);
          $uri[0]         = str_replace('https://', 'http://', $uri[0]);
          $thumbs['full'] = $uri[0];

          return json_encode($thumbs);
        }

        $thumbs['full'] = wp_get_attachment_image_src($attachment_id, 'full');
        $uri            = explode('?', $thumbs['full'][0]);
        $uri[0]         = str_replace('https://', 'http://', $uri[0]);
        $thumbs['full'] = $uri[0];

        // WebP
        $thumbs['webp'] = str_replace(array('jpg', 'jpeg', 'png'), 'webp', $thumbs['full']);

        if (is_array($sizesa)) {
          foreach ($sizesa as $size => $value) {
            $thumbs[ $value ] = wp_get_attachment_image_src($attachment_id, $value);
            $uri              = explode('?', $thumbs[ $value ][0]);
            $uri[0]           = str_replace('https://', 'http://', $uri[0]);
            $thumbs[ $value ] = $uri[0];
          }
        }

        if (class_exists('WooCommerce')) {
          $thumbs['woocommerce_thumbnail']         = wp_get_attachment_image_src($attachment_id, 'woocommerce_thumbnail');
          $thumbs['woocommerce_thumbnail']         = $thumbs['woocommerce_thumbnail'][0];
          $thumbs['woocommerce_single']            = wp_get_attachment_image_src($attachment_id, 'woocommerce_single');
          $thumbs['woocommerce_single']            = $thumbs['woocommerce_single'] [0];
          $thumbs['woocommerce_gallery_thumbnail'] = wp_get_attachment_image_src($attachment_id, 'woocommerce_gallery_thumbnail');
          $thumbs['woocommerce_gallery_thumbnail'] = $thumbs['woocommerce_gallery_thumbnail'][0];
        }

        $thumbs = json_encode($thumbs);

        $ch = curl_init(WPS_IC_APIURL . '?resync_cdn=true&attachmentID=' . $attachmentID . '&images=' . $thumbs);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($ch, CURLOPT_USERAGENT, 'api');
        curl_setopt($ch, CURLOPT_TIMEOUT, 10);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 10);
        $curl_output = curl_exec($ch);
        curl_close($ch);

        update_post_meta($attachmentID, 'wps_ic_cdn', 'true');
        delete_post_meta($attachmentID, 'wps_ic_docdn');
      }
    }

    if ($return) {
      wp_send_json_success();
    } else {
      return true;
    }
  }


  /**
   * @since 3.3.2
   */
  public function remote_restore_all() {
    global $wps_ic;

    if ( ! defined('ABSPATH')) {
      /** Set up WordPress environment */
      require_once(dirname(__FILE__) . '/wp-load.php');
    }

    update_option('ic_remote_action', 'restoring');
    $wps_ic->ajax->wps_ic_restore_bulk_prep_background_hidden('none');
    $wps_ic->ajax->wps_ic_restore_hidden_bulk();
  }


  /**
   * @since 3.3.2
   */
  public function remote_compress_all() {
    global $wps_ic, $wpdb;

    if ( ! defined('ABSPATH')) {
      /** Set up WordPress environment */
      require_once(dirname(__FILE__) . '/wp-load.php');
    }

    if ( ! function_exists('update_option')) {
      require_once(ABSPATH . "wp-includes" . '/option.php');
    }

    update_option('ic_remote_action', 'compressing');
    $wps_ic->ajax->wps_ic_compress_bulk_prep_hidden_background('none');
    $wps_ic->ajax->wps_ic_compress_hidden_bulk();

    $fetch = $wpdb->prepare("SELECT attachment_ID, type, status, hash FROM " . $wpdb->prefix . "ic_queue WHERE (type=%s OR type=%s) AND status=%s AND in_cron='0' ORDER BY attachment_ID DESC", 'hidden_compress_bulk', 'compress', '0');
    $fetch = $wpdb->get_results($fetch);

    if ($fetch) {
      foreach ($fetch as $row) {
        $queue[ $row->attachment_ID ] = array('ID' => $row->attachment_ID, 'type' => $row->type, 'hash' => $row->hash, 'status' => $row->status);
      }
    }

    $attachments = array();
    $queued      = 0;

    if ($queue) {

      foreach ($queue as $attID => $item) {
        $wps_ic->queue->change_queue_status($attID, 'queued', array('code' => 'compressing'));
        $queued ++;

        $attachments[] = $attID;
        $update        = $wpdb->prepare("UPDATE " . $wpdb->prefix . "ic_queue SET in_cron='1' WHERE attachment_ID=%s", $attID);
        $wpdb->query($update);

        if ($queued >= WPS_IC_IMAGES_PER_QUEUE) {
          $wps_ic->log->write_log('0', 'Exiting Compress Queue - Reached Queue Limit ' . WPS_IC_IMAGES_PER_QUEUE);
          $wps_ic->log->write_log('0', print_r($attachments, true));

          if ( ! empty($attachments) && count($attachments) > 0) {
            wp_schedule_single_event(time(), 'wps_ic_compress_single', array('attachments' => $attachments));
          }
          $attachments = array();
          $queued      = 0;
        }
      }

      if ($queued <= WPS_IC_IMAGES_PER_QUEUE) {
        if ( ! empty($attachments) && count($attachments) > 0) {
          wp_schedule_single_event(time(), 'wps_ic_compress_single', array('attachments' => $attachments));
        }
      }
    }
  }


  /**
   * @since 3.3.0
   */
  public function trigger_restore_cron() {
    global $wps_ic, $wpdb;

    #$queue = $wps_ic->queue->get_queue('hidden_restore_bulk');
    $fetch = $wpdb->prepare("SELECT attachment_ID, type, status, hash FROM " . $wpdb->prefix . "ic_queue WHERE (type=%s OR type=%s) AND status=%s AND in_cron='0' ORDER BY attachment_ID DESC", 'hidden_restore_bulk', 'restore', '0');
    $fetch = $wpdb->get_results($fetch);

    if ($fetch) {
      foreach ($fetch as $row) {
        $queue[ $row->attachment_ID ] = array('ID' => $row->attachment_ID, 'type' => $row->type, 'hash' => $row->hash, 'status' => $row->status);
      }
    }

    $attachments = array();
    $queued      = 0;

    if ($queue) {

      foreach ($queue as $attID => $item) {
        $queued ++;

        $attachments[] = $attID;
        $update        = $wpdb->prepare("UPDATE " . $wpdb->prefix . "ic_queue SET in_cron='1' WHERE attachment_ID=%s", $attID);
        $wpdb->query($update);

        if ($queued >= WPS_IC_IMAGES_PER_QUEUE) {
          $wps_ic->log->write_log('0', 'Exiting Restore Queue - Reached Queue Limit ' . WPS_IC_IMAGES_PER_QUEUE);
          $wps_ic->log->write_log('0', print_r($attachments, true));

          if ( ! empty($attachments) && count($attachments) > 0) {
            wp_schedule_single_event(time(), 'wps_ic_restore_single', array('attachments' => $attachments));
          }

          $attachments = array();
          $queued      = 0;
        }
      }

      if ($queued <= WPS_IC_IMAGES_PER_QUEUE) {
        if ( ! empty($attachments) && count($attachments) > 0) {
          wp_schedule_single_event(time(), 'wps_ic_restore_single', array('attachments' => $attachments));
        }
      }

    }
  }


  /**
   * @since 3.3.0
   */
  public function trigger_compress_single($attachments) {
    global $wps_ic, $wpdb;

    ini_set('memory_limit', '2024M');
    ini_set('max_execution_time', WPS_IC_QUEUE_EXECUTION_TIME);


    if (is_array($attachments)) {
      $wps_ic->compress->bulk(array('attachments' => $attachments));
    }

    // Do we have any more left?
    $left = $wps_ic->queue->get_queue('hidden_compress_bulk');
    if ( ! $left || count($left) == 0) {
      delete_option('ic_remote_action');
    }

    delete_transient('ic_cron_working');
    wp_send_json_success();
  }


  /**
   * @since 3.3.0
   */
  public function trigger_restore_single($attachments) {
    global $wps_ic, $wpdb;

    ini_set('memory_limit', '2024M');
    ini_set('max_execution_time', WPS_IC_QUEUE_EXECUTION_TIME);

    $wps_ic->log->write_log('0', 'Started Restore ' . print_r($attachments, true), 'restore');

    if (is_array($attachments)) {
      foreach ($attachments as $index => $attachment_ID) {

        $wps_ic->log->write_log($attachment_ID, 'In Loop', 'restore');
        $queue = $wps_ic->queue->get_queue('hidden_restore_bulk', $attachment_ID);
        if ( ! $queue) {
          $queue = $wps_ic->queue->get_queue('restore', $attachment_ID);
        }

        if ($queue) {
          $wps_ic->log->write_log($attachment_ID, 'In Queue - Restoring', 'restore');
          $wps_ic->compress->hidden_restore($attachment_ID);
        }
      }
    }

    // Do we have any more left?
    $left = $wps_ic->queue->get_queue('hidden_restore_bulk');
    if ( ! $left || count($left) == 0) {
      delete_option('ic_remote_action');
    }

    $wps_ic->log->write_log('0', 'Exited Trigger Restore Single', 'restore');
    delete_transient('ic_cron_working');
    wp_send_json_success();
  }


  /**
   * @since 3.3.0
   */
  public function push_cdn() {
    global $wps_ic, $wpdb;

    $attachments = get_posts(array(
                               'post_type'      => 'attachment',
                               'posts_per_page' => '500',
                               'meta_query'     => array(
                                 array(
                                   'key'     => 'wps_ic_docdn',
                                   'compare' => 'NOT EXISTS',
                                 ),
                                 array(
                                   'key'     => 'wps_ic_data',
                                   'compare' => 'EXISTS'
                                 )
                               )
                             ));

    $output = '';
    $count  = array();

    if ($attachments) {
      foreach ($attachments as $attachment) {
        $count[] = $attachment->ID;
        update_post_meta($attachment->ID, 'wps_ic_docdn', 'true');
      }

      // Check again
      $attachments = get_posts(array(
                                 'post_type'      => 'attachment',
                                 'posts_per_page' => '500',
                                 'meta_query'     => array(
                                   array(
                                     'key'     => 'wps_ic_docdn',
                                     'compare' => 'NOT EXISTS',
                                   ),
                                   array(
                                     'key'     => 'wps_ic_data',
                                     'compare' => 'EXISTS'
                                   )
                                 )
                               ));

      if ($attachments) {
        wp_schedule_single_event(time(), 'wps_ic_trigger_push_cdn');
        wp_send_json_success('triggered_push');
      } else {
        wp_schedule_single_event(time(), 'wps_ic_rebuild_cdn');
        wp_send_json_success('triggered_rebuild');
      }

    }

    wp_send_json_success();
  }


  /**
   * @since 3.3.0
   */
  public function trigger_compress_cron() {
    global $wps_ic, $wpdb;

    #$queue = $wps_ic->queue->get_queue('hidden_compress_bulk');
    // Prepare query
    $fetch = $wpdb->prepare("SELECT attachment_ID, type, status, hash FROM " . $wpdb->prefix . "ic_queue WHERE (type=%s OR type=%s) AND status=%s AND in_cron='0' ORDER BY attachment_ID DESC", 'hidden_compress_bulk', 'compress', '0');
    $fetch = $wpdb->get_results($fetch);

    if ($fetch) {
      foreach ($fetch as $row) {
        $queue[ $row->attachment_ID ] = array('ID' => $row->attachment_ID, 'type' => $row->type, 'hash' => $row->hash, 'status' => $row->status);
      }
    }

    $attachments = array();
    $queued      = 0;

    if ($queue) {

      foreach ($queue as $attID => $item) {
        $queued ++;

        $attachments[] = $attID;
        $update        = $wpdb->prepare("UPDATE " . $wpdb->prefix . "ic_queue SET in_cron='1' WHERE attachment_ID=%s", $attID);
        $wpdb->query($update);

        if ($queued >= WPS_IC_IMAGES_PER_QUEUE) {
          $wps_ic->log->write_log('0', 'Exiting Compress Queue - Reached Queue Limit ' . WPS_IC_IMAGES_PER_QUEUE);
          $wps_ic->log->write_log('0', print_r($attachments, true));

          if ( ! empty($attachments) && count($attachments) > 0) {
            #wp_schedule_single_event(time(), 'wps_ic_compress_single', array('attachments' => $attachments));
          }
          $attachments = array();
          $queued      = 0;
        }
      }

      if ($queued <= WPS_IC_IMAGES_PER_QUEUE) {
        if ( ! empty($attachments) && count($attachments) > 0) {
          #wp_schedule_single_event(time(), 'wps_ic_compress_single', array('attachments' => $attachments));
        }
      }
    }
  }


  public function trigger_owl() {
    global $wps_ic;
    #$wps_ic = new wps_ic();

    if ( ! defined('ABSPATH')) {
      /** Set up WordPress environment */
      require_once(dirname(__FILE__) . '/wp-load.php');
    }

    // What is unlocked?
    $settings = get_option(WPS_IC_SETTINGS);

    if ( ! empty($settings) && ! empty($settings['otto']) && $settings['otto'] == 'automated') {
      $wps_ic->trigger_night_owl();
      wp_send_json_success();
    }

    wp_send_json_error();
  }


  public function cron_regenerate_thumbnails() {
    global $wps_ic;

    return;
    $wps_ic->ai->register_new_sizes();

    if ( ! defined('ABSPATH')) {
      /** Set up WordPress environment */
      require_once(dirname(__FILE__) . '/wp-load.php');
    }

    if ( ! function_exists('wp_generate_attachment_metadata')) {
      require_once ABSPATH . 'wp-admin/includes/image.php';
    }

    if ( ! function_exists('download_url')) {
      require_once(ABSPATH . "wp-admin" . '/includes/image.php');
      require_once(ABSPATH . "wp-admin" . '/includes/file.php');
      require_once(ABSPATH . "wp-admin" . '/includes/media.php');
    }

    if ( ! function_exists('update_option')) {
      require_once(ABSPATH . "wp-includes" . '/option.php');
    }

    $attachments = $_POST['attachments'];

    if ( ! empty($attachments)) {
      foreach ($attachments as $key => $attachment) {
        $wps_ic->ai->thumbnail_rebuild($attachment);
      }
    }
  }


  public function generate_attachment_metadata($attachment_id, $file, $thumbnails = null) {
    return;
    if ( ! function_exists('file_is_displayable_image')) {
      require_once(ABSPATH . "wp-admin" . '/includes/image.php');
      require_once(ABSPATH . "wp-admin" . '/includes/file.php');
      require_once(ABSPATH . "wp-admin" . '/includes/media.php');
    }

    $attachment = get_post($attachment_id);
    $meta_datas = get_post_meta($attachment_id, '_wp_attachment_metadata', true);

    $metadata = [];
    if (preg_match('!^image/!', get_post_mime_type($attachment)) && file_is_displayable_image($file)) {
      $imagesize          = getimagesize($file);
      $metadata['width']  = $imagesize[0];
      $metadata['height'] = $imagesize[1];
      list($uwidth, $uheight) = wp_constrain_dimensions($metadata['width'], $metadata['height'], 128, 96);
      $metadata['hwstring_small'] = "height='$uheight' width='$uwidth'";

      // Make the file path relative to the upload dir.
      $metadata['file'] = _wp_relative_upload_path($file);

      // make thumbnails and other intermediate sizes.
      global $_wp_additional_image_sizes;
      #var_dump($_wp_additional_image_sizes);
      #var_dump(get_intermediate_image_sizes());

      foreach (get_intermediate_image_sizes() as $s) {
        $sizes[ $s ] = [
          'width'  => '',
          'height' => '',
          'crop'   => false,
        ];
        if (isset($_wp_additional_image_sizes[ $s ]['width'])) {
          $sizes[ $s ]['width'] = intval($_wp_additional_image_sizes[ $s ]['width']);
        } // For theme-added sizes
        else {
          $sizes[ $s ]['width'] = get_option("{$s}_size_w");
        } // For default sizes set in options
        if (isset($_wp_additional_image_sizes[ $s ]['height'])) {
          $sizes[ $s ]['height'] = intval($_wp_additional_image_sizes[ $s ]['height']);
        } // For theme-added sizes
        else {
          $sizes[ $s ]['height'] = get_option("{$s}_size_h");
        } // For default sizes set in options
        if (isset($_wp_additional_image_sizes[ $s ]['crop'])) {
          $sizes[ $s ]['crop'] = intval($_wp_additional_image_sizes[ $s ]['crop']);
        } // For theme-added sizes
        else {
          $sizes[ $s ]['crop'] = get_option("{$s}_crop");
        } // For default sizes set in options
      }

      $sizes = apply_filters('intermediate_image_sizes_advanced', $sizes);

      // Only if not all sizes.
      if (isset($thumbnails) && is_array($thumbnails) && isset($meta_datas['sizes']) && ! empty($meta_datas['sizes'])) {
        // Fill the array with the other sizes not have to be done.
        foreach ($meta_datas['sizes'] as $name => $fsize) {
          $metadata['sizes'][ $name ] = $fsize;
        }
      }

      foreach ($sizes as $size => $size_data) {
        if (isset($thumbnails)) {
          if ( ! in_array($size, $thumbnails)) {
            continue;
          }
        }

        $resized = image_make_intermediate_size($file, $size_data['width'], $size_data['height'], $size_data['crop']);

        if (isset($meta_datas['size'][ $size ])) {
          // Remove the size from the orignal sizes for after work.
          unset($meta_datas['size'][ $size ]);
        }

        if ($resized) {
          $metadata['sizes'][ $size ] = $resized;
        }
      }

      // fetch additional metadata from exif/iptc.
      $image_meta = wp_read_image_metadata($file);
      if ($image_meta) {
        $metadata['image_meta'] = $image_meta;
      }
    }

    return apply_filters('wp_generate_attachment_metadata', $metadata, $attachment_id);
  }


  public function cron_fetch_thumbnails() {
    global $wps_ic, $wpdb;

    return;
    $options = get_option(WPS_IC_SETTINGS);

    if ( ! defined('ABSPATH')) {
      /** Set up WordPress environment */
      require_once(dirname(__FILE__) . '/wp-load.php');
    }

    if ( ! function_exists('wp_generate_attachment_metadata')) {
      require_once ABSPATH . 'wp-admin/includes/image.php';
    }

    if ( ! function_exists('download_url')) {
      require_once(ABSPATH . "wp-admin" . '/includes/image.php');
      require_once(ABSPATH . "wp-admin" . '/includes/file.php');
      require_once(ABSPATH . "wp-admin" . '/includes/media.php');
    }

    if ( ! function_exists('update_option')) {
      require_once(ABSPATH . "wp-includes" . '/option.php');
    }

    // Get COMPRESSED attachment
    $wps_ic->log->write_log('0', 'Fetch Thumbnails Queue');

    $compress_queue = $wps_ic->queue->get_queue('generate_thumbnail');
    foreach ($compress_queue as $attach_id => $file) {

      // Generate attachment metadata
      $fullsizepath  = get_attached_file($attach_id);
      $path_to_thumb = str_replace(basename($fullsizepath), '', $fullsizepath);

      $wps_ic->queue->remove_queue($attach_id, true, 'generate_thumbnail');

      if ( ! empty($options['thumbnails_locally']) && $options['thumbnails_locally'] == '1') {

        // Generate attachment metadata
        $fullsizepath = get_attached_file($attach_id);
        $attach_data  = wp_generate_attachment_metadata($attach_id, $fullsizepath);
        wp_update_attachment_metadata($attach_id, $attach_data);

        delete_post_meta($attach_id, 'thumbnail_generated');

        $wps_ic->queue->remove_queue($attach_id, true, 'generate_thumbnail');
        wp_send_json_success();

      } else {
        $check_thumbnails = wp_remote_get(WPS_IC_APIURL . '?fetch_thumbnails=true&api_key=' . $wps_ic::$api_key . '&attachment_ID=' . $attach_id, array('timeout' => 5, 'sslverify' => false));

        if (wp_remote_retrieve_response_code($check_thumbnails) == 200) {
          $body = wp_remote_retrieve_body($check_thumbnails);
          $body = json_decode($body, true);

          if ($body['success'] == true) {
            $data = $body['data'][ $attach_id ];

            if (is_array($data)) {
              foreach ($data as $thumb_size => $thumb_url) {
                $thumbnail_meta = wp_get_attachment_metadata($attach_id);

                if ($thumbnail_meta) {
                  $sizes                  = $thumbnail_meta['sizes'];
                  $current_size           = $sizes[ $thumb_size ];
                  $current_thumbnail      = $current_size['file'];
                  $current_thumbnail_path = $path_to_thumb . $current_thumbnail;

                  if (empty($current_thumbnail) || $current_thumbnail == '' || $current_thumbnail == '/') {
                    delete_post_meta($attach_id, 'thumbnail_generated');
                    continue;
                  }

                  if ($current_thumbnail !== basename($thumb_url)) {
                    delete_post_meta($attach_id, 'thumbnail_generated');
                    continue;
                  }

                  $temp_file = download_url($thumb_url, 60);
                  if ($temp_file) {

                    if (unlink($current_thumbnail_path)) {
                      copy($temp_file, $current_thumbnail_path);
                    }
                  }

                }

              }
            }

            $wps_ic->queue->remove_queue($attach_id, true, 'generate_thumbnail');
            delete_post_meta($attach_id, 'thumbnail_generated');

            $remove_thumanils = wp_remote_get(WPS_IC_APIURL . '?fetch_thumbnails=true&api_key=' . $wps_ic::$api_key . '&attachment_ID=' . $attach_id . '&remove=true', array('timeout' => 5, 'sslverify' => false));

          }

        } else {
          wp_send_json_error('Still processing.');
        }
      }

    }

    wp_send_json_success();
  }


  public function cron_on_upload($attach_id = '') {
    global $wps_ic;
    #$wps_ic = new wps_ic();

    if (empty($attach_id)) {
      return;
    }

    if ( ! defined('ABSPATH')) {
      /** Set up WordPress environment */
      require_once(dirname(__FILE__) . '/wp-load.php');
    }

    if ( ! function_exists('wp_generate_attachment_metadata')) {
      require_once ABSPATH . 'wp-admin/includes/image.php';
    }

    // Allowed type
    $file_data = get_attached_file($attach_id);
    $type      = wp_check_filetype($file_data);

    // Add generate thumbnail to queue
    $attach_data = wp_generate_attachment_metadata($attach_id, $file_data);
    wp_update_attachment_metadata($attach_id, $attach_data);

    // Mark attachment as compressing
    update_post_meta($attach_id, 'wps_ic_in_bulk', 'true');
    $queue = $wps_ic->queue->add_queue($attach_id, 'compress');

    // Compress Image
    $wps_ic->compress->single_bulk_v2(array('attachment_id' => $attach_id));
  }


}