<?php


/**
 * Class - Database
 */
class wps_ic_database extends wps_ic {

  public $database;


  public function __construct() {
    return $this;
  }


  public function delete_tables() {
    global $wpdb;
  }


  public function create_tables_per_site($blog_id) {
    global $wpdb;
    $prefix = $wpdb->get_blog_prefix($blog_id);
    $this->create_stats_table($prefix);
    $this->create_compressed_table($prefix);
    $this->create_compress_table($prefix);
    $this->create_log_table($prefix);
  }


  public function create_tables() {
    global $wpdb;

    if (is_multisite()) {
      $sites = get_sites();

      foreach ($sites as $index => $site) {
        $prefix = $wpdb->get_blog_prefix($site->blog_id);
        $this->create_stats_table($prefix);
        $this->create_compressed_table($prefix);
        $this->create_compress_table($prefix);
        $this->create_log_table($prefix);
      }
    } else {
      $prefix = $wpdb->prefix;
      $this->create_stats_table($prefix);
      $this->create_compressed_table($prefix);
      $this->create_compress_table($prefix);
      $this->create_log_table($prefix);
    }
  }


  /**
   * All logs database table
   */
  public function create_log_table($prefix) {

    global $wpdb;

    $charset_collate = $wpdb->get_charset_collate();

    $sql = "CREATE TABLE IF NOT EXISTS `" . $prefix . "ic_log` (
  `ID` int(11) NOT NULL AUTO_INCREMENT,
  `blogID` int(11) NOT NULL,
  `attachment_ID` int(11) NOT NULL,
  `status` tinyint(4) NOT NULL DEFAULT '0',
  `type` varchar(32) COLLATE utf8mb4_unicode_ci NOT NULL,
  `occured` datetime NOT NULL,
  `message` text COLLATE utf8mb4_unicode_ci NOT NULL,
   PRIMARY KEY (`ID`))" . $charset_collate . ";";

    require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
    dbDelta($sql);

  }


  /**
   * Handles stats
   */
  public function create_stats_table($prefix) {
    global $wpdb;
    $charset_collate = $wpdb->get_charset_collate();

    $sql = "CREATE TABLE IF NOT EXISTS `" . $prefix . "ic_stats` (`ID` int(11) NOT NULL AUTO_INCREMENT,`created` date NOT NULL,`attachment_ID` int(11) NOT NULL,`blogID` int(11) NOT NULL,`original` int(11) NOT NULL,`compressed` int(11) NOT NULL,`saved` int(11) NOT NULL,`percent_saved` int(11) NOT NULL, PRIMARY KEY (`ID`), UNIQUE KEY `attachment_ID` (`attachment_ID`))" . $charset_collate . ";";

    require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
    dbDelta($sql);
  }


  /**
   * Handles which images are in compressing queue
   */
  public function create_compress_table($prefix) {

    global $wpdb;

    $charset_collate = $wpdb->get_charset_collate();

    $sql = "CREATE TABLE IF NOT EXISTS `" . $prefix . "ic_queue` (
  `ID` int(11) NOT NULL AUTO_INCREMENT,
  `blogID` int(11) NOT NULL,
  `attachment_ID` int(11) NOT NULL,
  `hash` varchar(32) COLLATE utf8mb4_unicode_ci NOT NULL,
  `type` varchar(32) COLLATE utf8mb4_unicode_ci NOT NULL,
  `status` varchar(32) NOT NULL,
  `in_cron` tinyint(4) NOT NULL,
  `message` text COLLATE utf8mb4_unicode_ci NOT NULL,
  `old_metadata` text COLLATE utf8mb4_unicode_ci NOT NULL,
  `new_metadata` text COLLATE utf8mb4_unicode_ci NOT NULL,
  `created` datetime NOT NULL, PRIMARY KEY (`ID`),UNIQUE KEY `attachment_ID` (`attachment_ID`,`type`))" . $charset_collate . ";";

    require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
    dbDelta($sql);
  }


  /**
   * Handles which images are compressed
   */
  public function create_compressed_table($prefix) {

    global $wpdb;

    $charset_collate = $wpdb->get_charset_collate();

    $sql = "CREATE TABLE IF NOT EXISTS `" . $prefix . "ic_compressed` (`ID` int(11) NOT NULL AUTO_INCREMENT, `created` datetime NOT NULL, `attachment_ID` int(11) NOT NULL, `blogID` int(11) NOT NULL, `original` float NOT NULL, `saved` float NOT NULL, `percent_saved` float NOT NULL, `restored` tinyint(4) NOT NULL DEFAULT '0', `count` smallint(6) NOT NULL DEFAULT '0', PRIMARY KEY  (ID),UNIQUE KEY `attachment_ID` (`attachment_ID`)) $charset_collate;";

    require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
    dbDelta($sql);

  }


}