<?php


/**
 * Class - Options
 */
class wps_ic_options extends wps_ic {

  public $options;


  public function __construct() {
    return $this;
  }


  /**
   * Save settings
   */
  public function save_settings() {
    if ( ! empty($_POST)) {

      $options                           = get_option(WPS_IC_SETTINGS);
      $_POST['wp-ic-setting']['unlocks'] = $options['unlocks'];

      if (empty($_POST['wp-ic-setting']['optimization']) || $_POST['wp-ic-setting']['optimization'] == '0') {
        $_POST['wp-ic-setting']['optimization'] = 'maximum';
      }

      if (empty($_POST['wp-ic-setting']['optimize_upload'])) {
        $_POST['wp-ic-setting']['optimize_upload'] = '0';
      }

      if (empty($_POST['wp-ic-setting']['ignore_larger_images'])) {
        $_POST['wp-ic-setting']['ignore_larger_images'] = '0';
      }

      if (empty($_POST['wp-ic-setting']['resize_larger_images'])) {
        $_POST['wp-ic-setting']['resize_larger_images'] = '0';
      }

      if (empty($_POST['wp-ic-setting']['resize_larger_images_width'])) {
        $_POST['wp-ic-setting']['resize_larger_images_width'] = '2048';
      }

      if (empty($_POST['wp-ic-setting']['ignore_larger_images_width'])) {
        $_POST['wp-ic-setting']['ignore_larger_images_width'] = '2048';
      }

      if (empty($_POST['wps_no']['time'])) {
        $_POST['wp-ic-setting']['wps_no']['time'] = '';
      }

      if (empty($_POST['wp-ic-setting']['backup'])) {
        $_POST['wp-ic-setting']['backup'] = '0';
      }

      if (empty($_POST['wp-ic-setting']['hide_compress'])) {
        $_POST['wp-ic-setting']['hide_compress'] = '0';
      }

      if (empty($_POST['wp-ic-setting']['thumbnails_locally'])) {
        $_POST['wp-ic-setting']['thumbnails_locally'] = '0';
      }

      if (empty($_POST['wp-ic-setting']['debug'])) {
        $_POST['wp-ic-setting']['debug'] = '0';
      }

      if (empty($_POST['wp-ic-setting']['preserve_exif'])) {
        $_POST['wp-ic-setting']['preserve_exif'] = '0';
      }

      if (empty($_POST['wp-ic-setting']['night_owl'])) {
        $_POST['wp-ic-setting']['night_owl'] = '0';
      }

      if (empty($_POST['wp-ic-setting']['otto'])) {
        $_POST['wp-ic-setting']['otto'] = 'off';
      }

      if (empty($_POST['wp-ic-setting']['night_owl_upload'])) {
        $_POST['wp-ic-setting']['night_owl_upload'] = '0';
      }

      if ( ! empty($_POST['wp-ic-setting']['thumbnails'])) {
        foreach ($_POST['wp-ic-setting']['thumbnails'] as $key => $value) {
          $_POST['wp-ic-setting']['thumbnails'][ $key ] = 1;
        }
      }

      // Sanitize
      foreach ($_POST['wp-ic-setting'] as $key => $value) {
        $_POST['wp-ic-setting'][ $key ] = $value;
      }

      update_option(WPS_IC_SETTINGS, $_POST['wp-ic-setting']);
    }
  }


  /**
   * Get compress stats (total images, total saved)
   * @return mixed|void
   */
  public function get_stats() {
    global $wpdb;

    $query = $wpdb->prepare("SELECT COUNT(ID) as images, SUM(saved) as saved FROM " . $wpdb->prefix . "ic_compressed ORDER by ID");
    $query = $wpdb->get_results($query);

    return array('images' => $query[0]->images, 'saved' => $query[0]->saved);
  }


  /**
   * Update stats
   */
  public function update_stats($attachment_ID = 1, $saved = '', $action = 'add') {
    global $wpdb;

    $attachment_ID = (int)$attachment_ID;
    $saved         = sanitize_text_field($saved);

    if ($action == 'add') {
      $query = $wpdb->prepare("INSERT INTO " . $wpdb->prefix . "ic_compressed (created, attachment_ID, saved, count) VALUES (%s, %s, %s, %s) ON DUPLICATE KEY UPDATE created=%s, count=count+1, restored=0",
                              current_time('mysql'), $attachment_ID, $saved, current_time('mysql'), '1');
      $wpdb->query($query);
    } else {
      //
    }

  }


  /**
   * Get various settings for WP Compress
   * @return mixed|void
   */
  public function get_settings() {
    $settings = get_option(WPS_IC_SETTINGS);

    return $settings;
  }


  /**
   * Fetch specific option or all options if key is empty
   *
   * @param null $key
   *
   * @return bool|mixed|void
   */
  public function get_option($key = null) {
    $options = get_option(WPS_IC_OPTIONS);

    if ($key == null) {

      if (empty($options)) {
        return false;
      }

      return $options;
    } else {

      if (empty($options[ $key ])) {
        return false;
      }

      return $options[ $key ];
    }

  }


  /**
   * Set option with key and value
   *
   * @param $key
   * @param $value
   */
  public function set_option($key, $value) {
    $options         = get_option(WPS_IC_OPTIONS);
    $options[ $key ] = $value;
    update_option(WPS_IC_OPTIONS, $options);
  }


  /**
   * Setup default settings
   */
  public function set_defaults() {
    $defaults = array();

    $defaults['authorized'] = '';
    $defaults['api_key']    = md5(site_url() . mt_rand(1000, 9999));
    update_option(WPS_IC_OPTIONS, $defaults);

    $settings                         = array();
    $settings['optimize_upload']      = '0';
    $settings['resize_larger_images'] = '1';
    $settings['ignore_larger_images'] = '0';
    $settings['preserve_exif']        = '0';
    $settings['debug']                = '0';
    $settings['backup']               = '1';
    $settings['hide_compress']        = '0';
    $settings['thumbnails_locally']   = '0';
    $settings['night_owl']            = '0';
    $settings['night_owl_upload']     = '0';
    $settings['auto_optimize']        = '0';

    $settings['debug']                      = 'yes';
    $settings['resize_larger_images_width'] = '2048';
    $settings['ignore_larger_images_width'] = '2048';
    $settings['custom_quality']             = '0';
    $settings['optimize_types'][]           = 'png';
    $settings['optimize_types'][]           = 'jpg';
    $settings['wps_no']['time']             = '';
    $settings['unlocks']                    = '';

    $settings['autopilot']    = '1';
    $settings['otto']         = 'on-upload';
    $settings['optimization'] = 'maximum';

    $sizes = get_intermediate_image_sizes();
    foreach ($sizes as $key => $value) {
      $settings['thumbnails'][ $value ] = 1;
    }

    update_option(WPS_IC_SETTINGS, $settings);

    update_option('wps_ic_demo_connect', 'true');
    update_option('wps_ic_demo_settings', 'true');
    update_option('wps_ic_demo_media', 'true');

  }

}