<?php


/**
 * Class - Tests
 */
class wps_ic_tests extends wps_ic {


  public function __construct() {
    if ( ! empty($_GET['img'])) {
      $this->image_upload();
    }

    $this->listener();
  }


  public function listener() {

    if ( ! empty($_GET['debug'])) {
      foreach ($_GET as $key => $value) {
        $_POST[ $key ] = $value;
      }
    }

    if ( ! empty($_POST['test']) && ! empty($_POST['apikey'])) {
      update_option('wps_ic_test', 'true');
      $apikey  = sanitize_text_field($_POST['apikey']);
      $test    = sanitize_text_field($_POST['test']);
      $value   = sanitize_text_field($_POST['value']);
      $setting = sanitize_text_field($_POST['setting']);

      $tmp_key = get_option('wps_ic_tmp_apikey');
      if ($tmp_key != $apikey) {
        wp_send_json_error();
      }

      wp_send_json_success();
    }

  }


  public function call_test($test_name) {
    sleep(2);
    $this->$test_name();
  }


  public function verify_api_key() {
    // API Key
    $apikey         = sanitize_text_field($_POST['apikey']);

    // Setup URI
    $uri = WPS_IC_KEYSURL;
    $uri .= '?action=verify_api_key';
    $uri .= '&apikey=' . $apikey;
    $uri .= '&site=' . urlencode(site_url());

    if ( ! empty($_POST['fail_test'])) {
      if ($_POST['fail_test'] == '1') {
        wp_send_json_error(array('code' => 'Unable to compress'));
      }
    }

    update_option('wps_ic_tmp_apikey', $apikey);

    // Verify API Key is our database and user has is confirmed getresponse
    $get = wp_remote_get($uri, array('timeout' => 60, 'sslverify' => false));

    if (wp_remote_retrieve_response_code($get) == 200) {
      $body = wp_remote_retrieve_body($get);
      $body = json_decode($body);

      if ($body->success == true && $body->data->code == 'ok') {
        wp_send_json_success();
      } else {
        wp_send_json_error(array('code' => 'Key Failed', 'call' => $uri));
      }
    }

    wp_send_json_error(array('code' => 'API Failed', 'call' => $get, 'code' => wp_remote_retrieve_response_code($get), 'body' => wp_remote_retrieve_body($get)));
  }


  public function communication_with_portal() {
    // API Key
    $tmp_key = get_option('wps_ic_tmp_apikey');
    $apikey  = sanitize_text_field($tmp_key);
    $site    = urlencode(site_url());

    if ( ! empty($_POST['fail_test'])) {
      if ($_POST['fail_test'] == '2') {
        wp_send_json_error(array('code' => 'Portal not working'));
      }
    }

    // Setup URI
    $uri = WPS_IC_PORTALURL;

    $call = wp_remote_get(WPS_IC_PORTALURL . '?action=remote_setting_test&apikey=' . $apikey . '&site=' . $site, array('sslverify' => false, 'timeout' => 20));

    if (wp_remote_retrieve_response_code($call) == 200) {
      $body = wp_remote_retrieve_body($call);
      $body = json_decode($body);

      if ($body->success == true) {
        wp_send_json_success();
      } else {
        wp_send_json_error(array('code' => 'Portal not working', 'url' => WPS_IC_PORTALURL, 'body_raw' => wp_remote_retrieve_body($call), 'body' => $body, 'site' => $site, 'apikey' => $apikey));
      }

    }

    wp_send_json_error(array('code' => 'Portal is not accessible'));
  }


  public function image_upload() {
    global $wpdb;

    // Remove on upload
    $options      = get_option(WPS_IC_SETTINGS);
    $old_settings = $options;
    delete_option(WPS_IC_SETTINGS);

    require_once(ABSPATH . 'wp-admin/includes/media.php');
    require_once(ABSPATH . 'wp-admin/includes/file.php');
    require_once(ABSPATH . 'wp-admin/includes/image.php');

    // Test Image
    $image = 'https://plugin.wpcompress.xyz/test-image.jpg';

    // Was it already uploaded?
    $test_image = get_option('wps_ic_test_image');

    if (!empty($test_image)) {
      delete_option('wps_ic_test_image');
      wp_delete_attachment($test_image, true);
      $test_image = false;
    }

    if (empty($test_image) || !$test_image) {
      $media = media_sideload_image($image, 0, '', 'id');
      update_option('wps_ic_test_image', $media);
    }

    if ( ! empty($media) && ! is_wp_error($media)) {
      update_option(WPS_IC_SETTINGS, $old_settings);

      return true;
    } else {
      update_option(WPS_IC_SETTINGS, $old_settings);

      return false;
    }
  }


  public function image_compress() {
    global $wpdb, $wps_ic;
    $wps_ic = new wps_ic();

    $upload = $this->image_upload();
    if ( ! $upload) {
      delete_option('wps_ic_test_image');
      wp_send_json_error(array('code' => 'unable_upload'));
    }

    // API Key
    $tmp_key = get_option('wps_ic_tmp_apikey');
    $apikey  = sanitize_text_field($tmp_key);
    $site    = urlencode(site_url());

    if ( ! empty($_POST['fail_test'])) {
      if ($_POST['fail_test'] == '3') {
        wp_send_json_error(array('code' => 'unable_compress'));
      }
    }

    $compress_test = get_option('wps_ic_test_image');
    #$queue = $wps_ic->queue->add_queue($compress_test, mt_rand(1000, 9999), 'hidden_compress_bulk');
    $wps_ic->compress->bulk(array('attachments' => array($compress_test)), $apikey, false);

    $data = get_post_meta($compress_test, 'wps_ic_data', true);

    if ( ! empty($data)) {
      wp_send_json_success(array('attachment_id' => $compress_test));
    } else {
      wp_delete_attachment($compress_test);
      delete_option('wps_ic_test_image');
      wp_send_json_error(array('code' => 'unable_compress', 'attachment_id' => $compress_test, 'apikey' => $apikey));
    }

    delete_option('wps_ic_test_image');
    wp_send_json_error(array('code' => 'no_attachments'));
  }


  public function image_restore() {
    global $wps_ic;
    $wps_ic = new wps_ic();

    // API Key
    $test_image = get_option('wps_ic_test_image');

    if ( ! empty($_POST['fail_test'])) {
      if ($_POST['fail_test'] == '4') {
        wp_send_json_error(array('code' => 'unable_restore'));
      }
    }

    if ($test_image) {
      $queue = $wps_ic->queue->add_queue($test_image, 'hidden_restore_bulk');
      $restore = $wps_ic->compress->bulk_restore(array('attachments' => array($test_image)));
      $data = get_post_meta($test_image, 'wps_ic_data', true);
      wp_delete_attachment($test_image, true);
      delete_option('wps_ic_test_image');

      if ( ! empty($data)) {
        wp_send_json_error(array('code' => 'unable_restore', 'data' => $data, 'attID' => $test_image, 'restore' => print_r($restore, true)));
      } else {
        wp_send_json_success(array('ID' => $test_image));
      }

    }

    delete_option('wps_ic_test_image');
    wp_send_json_error(array('code' => 'test_image_not_found'));
  }

}