<?php
/*
 * Plugin name: WP Compress | Image Optimizer
 * Plugin URI: https://www.wpcompress.com
 * Author: WP Compress
 * Version: 4.1.71
 * Description: Automatically compress and optimize images to shrink image file size, improve load times and boost SEO ranks - all without lifting a finger after setup.
 */

if ( ! empty($_GET['ic_debug'])) {
  update_option('wps_ic_debug', $_GET['ic_debug']);
  var_dump('Debug set to ' . $_GET['ic_debug']);
}

if ( ! empty($_GET['ic_debug_library'])) {
  update_option('ic_debug_library', $_GET['ic_debug_library']);
  var_dump('Debug set to ' . $_GET['ic_debug_library']);
}

error_reporting(E_ALL);
ini_set('display_errors', 1);

if (get_option('wps_ic_debug') == 'true') {
  error_reporting(E_ALL);
  ini_set('display_errors', 1);
} else {
  error_reporting(0);
  ini_set('display_errors', 0);
}


include 'defines.php';
include 'addons/nightowl/night-owl.php';
include 'addons/autooptimize/ic_autooptimize.php';
include 'addons/adaptive/adaptive.php';
include 'addons/cdn/cdn.php';

spl_autoload_register(function($class_name) {
  $class_name = str_replace('wps_ic_', '', $class_name);
  $class_name = $class_name . '.class.php';
  if (file_exists(WPS_IC_DIR . 'classes/' . $class_name)) {
    include 'classes/' . $class_name;
  }
});


class wps_ic {

  public static $slug;
  public static $version;
  public static $api_key;
  public static $ic_api_url;
  public static $total_referrals;
  public static $response_key;

  public $options;
  public $notices;
  public $enqueues;
  public $templates;
  public $menu;
  public $ajax;
  public $backup;
  public $media_library;
  public $compress;
  public $controller;
  public $log;
  public $nextgen;
  public $requests;
  public $bulk;
  public $queue;
  public $stats;
  public $tester;
  public $cdn;
  public $mu;
  public $mainwp;
  public $tests;


  /**
   * Our main class constructor
   */
  public function __construct() {
    global $wps_ic;

    $this->options = new wps_ic_options();

    add_action('ic_debug', array('wps_ic', 'action_log'), 10, 2);
    add_action('admin_footer', array('wps_ic', 'deactivate_script'));
    add_filter('cron_schedules', array('wps_ic', 'wp_ic_cron_timming'));
    #add_filter('http_request_host_is_external', array('wps_ic', 'valid_api_url'), 10, 3);

    // Basic plugin info
    $response_key       = $this->options->get_option('response_key');
    self::$slug         = 'wpcompress';
    self::$version      = '4.1.71';
    self::$api_key      = $this->options->get_option('api_key');
    self::$ic_api_url   = WPS_IC_APIURL;
    self::$response_key = $response_key;

    if ( ! defined('ALLOW_UNFILTERED_UPLOADS')) {
      define('ALLOW_UNFILTERED_UPLOADS', true);
    }

    $this->menu      = new wps_ic_menu();
    $this->ai        = new wps_addon_adaptive();
    $this->cdn       = new wps_addon_cdn();
    $this->cron      = new wps_ic_cron();
    $this->log       = new wps_ic_log();
    $this->enqueues  = new wps_ic_enqueues();
    $this->templates = new wps_ic_templates();
    $this->notices   = new wps_ic_notices();

    // Ajax
    $this->media_library = new wps_ic_media_library();

    $this->ajax = new wps_ic_ajax();

    $this->queue      = new wps_ic_queue();
    $this->compress   = new wps_ic_compress();
    $this->controller = new wps_ic_controller();
    $this->requests   = new wps_ic_requests();

    $this->remote_restore = new wps_ic_remote_restore();
    $this->upgrade        = new wps_ic_upgrade();
    $this->popups         = new wps_ic_popups();
    $this->comms          = new wps_ic_comms();
    $this->mainwp         = new wps_ic_mainwp();
    $this->tests          = new wps_ic_tests();


    $this->mu = new wps_ic_mu();

    // What is unlocked?
    $settings = get_option(WPS_IC_SETTINGS);

    #$nightowl = new wps_no();

    if ( ! empty($settings['autopilot']) && $settings['otto'] == 'on-upload') {
      $ao = new wps_autooptimize();
    }


    // Change PHP Limits
    $this->change_limits();
    $this->setup_Cron_schedules();

    $wps_ic = $this;

    // Autopilot - Advanced Mode
    if ( ! empty($settings) && ! empty($settings['otto']) && $settings['otto'] == 'automated') {
      $this->trigger_night_owl();

      if (!empty($_GET['ic_automatted']) && !empty($_GET['apikey'])) {
        if (self::$api_key == sanitize_text_field($_GET['apikey'])) {
          $this->run_night_owl();
        }
      }
    }




    if ( ! empty($_GET['get_crons'])) {
      foreach (_get_cron_array() as $timestamp => $events) {

        foreach ($events as $event_hook => $event_args) {
          foreach ($event_args as $event) {

            if ($event_hook == 'wps_ic_compress_single') {
              foreach ($event_args as $hash => $event_Args) {

                if ( ! empty($event_Args['args'])) {
                  delete_transient('doing_cron');
                  #set_transient('ic_cron_working', 'true', 6);
                  var_dump($event_hook);
                  #do_action_ref_array($event_hook, $event_Args['args']);
                }

                wp_send_json_success('compress');
              }
            }

            if ($event_hook == 'wps_ic_restore_single') {
              foreach ($event_args as $hash => $event_Args) {

                if ( ! empty($event_Args['args'])) {
                  delete_transient('doing_cron');
                  #set_transient('ic_cron_working', 'true', 6);
                  var_dump($event_hook);
                  #do_action_ref_array($event_hook, $event_Args['args']);
                }

                wp_send_json_success('restore');
              }
            }

          }

        }

      }
      die();
    }

    if ( ! empty($_GET['ic_queue_ping'])) {
      $wps_ic->ajax->wps_ic_compress_next_in_queue();
      wp_send_json_success('Nothing to ping.');
    }




    if ( ! empty($_GET['get_thumbs'])) {
      var_dump($wps_ic->compress->get_all_thumbnails($_GET['attID']));
      die();
    }

    if ( ! empty($_GET['rebuild_cdn'])) {
      $attachment_ID = '';
      if ( ! empty($_GET['attID'])) {
        $attachment_ID = $_GET['attID'];
      }
      $wps_ic->cron->rebuild_cdn($attachment_ID);
      die();
    }

    if ( ! empty($_GET['regen_thumbs'])) {
      #$img = wp_get_attachment_image_src($_GET['attID'], 'img-360');
      $m = $wps_ic->ai->thumbnail_rebuild($_GET['attID']);
      var_dump($m);
      die();
    }


    if ( ! empty($_GET['resync_cdn'])) {
      $wps_ic->ajax->resync_cdn_marker();
      wp_send_json_success();
    }

    if ( ! empty($_GET['ic_compress'])) {
      $wps_ic->ajax->wps_ic_call_compress();
      die('exit');
    }

    if ( ! empty($_GET['image_status'])) {
      $wps_ic->ajax->get_image_status();
      die('exit');
    }

    if ( ! empty($_GET['ic_restore'])) {
      $wps_ic->ajax->wps_ic_call_restore();
      die('exit');
    }

		if (is_admin()) {

			if ( ! empty($_GET['deauth'])) {
				$this->ajax->wps_ic_deauthorize_api();
				wp_safe_redirect(admin_url('admin.php?page=wpcompress'));
				die();
			}

			if ( ! empty($_GET['restore'])) {
				$wps_ic->trigger_restore();
			}

			// Test otto
			if ( ! empty($_GET['get_logs'])) {
				$log_file = WPS_IC_DIR . 'log-' . date('d-m-Y') . '.txt';
				var_dump($log_file);
				die();
			}

			if ( ! empty($_GET['test']) && $_GET['test'] != 'remote_setting') {
				$wps_ic->trigger_night_owl();
				die('no running');
			}

			if ( ! empty($_GET['no_trigger'])) {
				$this->cron->trigger_owl();
			}

			if (!empty($_GET['show_compress_queue'])) {
				global $wpdb;
				ini_set('display_errors', '1');
				error_reporting(E_ALL);
				ini_set('error_reporting', E_ALL);

				$fetch = $wpdb->get_results("SELECT attachment_ID, type, status, message FROM " . $wpdb->prefix . "ic_queue");

				if ($fetch) {
					echo '<table style="border: 1px solid black;">';
					foreach ($fetch as $item) {
						echo '<tr>';
						echo '<td style="border: 1px solid black;">' . $item->attachment_ID . '</td>';
						echo '<td style="border: 1px solid black;">' . $item->type . '</td>';
						echo '<td style="border: 1px solid black;">' . $item->status . '</td>';
						echo '<td style="border: 1px solid black;">' . $item->message . '</td>';
						echo '</tr>';
					}
					echo '</table>';
				}

				die('Nothing to show');
			}
		}

  }


  public function setup_Cron_schedules() {
    /*
    // Setup thumbnail regen cron
    if ( ! wp_next_scheduled('wps_ic_regen_thumbs')) {
      wp_schedule_event(time(), 'two_minutes', 'wps_ic_regen_thumbs');
    }

    // Setup upload cron
    if ( ! wp_next_scheduled('wps_ic_upload_image')) {
      wp_schedule_event(time(), 'daily', 'wps_ic_upload_image');
    }

    // Setup CDN cron
    if ( ! wp_next_scheduled('wps_ic_rebuild_cdn')) {
      wp_schedule_event(time(), 'five_minutes', 'wps_ic_rebuild_cdn');
    }

    // Setup thumbnail cron
    if ( ! wp_next_scheduled('wps_ic_compress')) {
      wp_schedule_event(time(), 'daily', 'wps_ic_compress');
    }

    // Cron Wake by Visiting Site
    if ( ! wp_next_scheduled('wps_ic_wake_up_cron_visit')) {
      wp_schedule_event(time(), 'every_minute', 'wps_ic_wake_up_cron_visit');
    }


    // Cron CDN Status
    if ( ! wp_next_scheduled('wps_ic_check_cdn_status')) {
      wp_schedule_event(time(), 'every_minute', 'wps_ic_check_cdn_status');
    }

    // Setup thumbnail cron
    if ( ! wp_next_scheduled('wps_ic_fetch_thumbnails')) {
      wp_schedule_event(time(), 'two_minutes', 'wps_ic_fetch_thumbnails');
    }

    if ( ! wp_next_scheduled('wps_ic_single_thumbnail')) {
      wp_schedule_event(time(), 'daily', 'wps_ic_single_thumbnail');
    }

    if ( ! wp_next_scheduled('wps_ic_owl')) {
      wp_schedule_event(time(), 'one_hour', 'wps_ic_owl');
    }

    if ( ! wp_next_scheduled('wps_ic_compress_queue')) {
      wp_schedule_event(time(), 'every_minute', 'wps_ic_compress_queue');
    }

    if ( ! wp_next_scheduled('wps_ic_restore_queue')) {
      wp_schedule_event(time(), 'every_minute', 'wps_ic_restore_queue');
    }
    */
  }


  public function action_log($attachmentID, $message) {
    if (get_option('ic_debug') == 'true') {
      $log_file = WPS_IC_DIR . 'action-log-' . date('d-m-Y') . '.txt';
      $time     = current_time('mysql');

      if ( ! file_exists($log_file)) {
        fopen($log_file, 'a');
      }

      $log = file_get_contents($log_file);
      $log .= '[' . $time . '] - Attachment ID: ' . $attachmentID . ' - ' . $message . "\r\n";
      file_put_contents($log_file, $log);
      fclose($log_file);
    }
  }


  public function trigger_restore() {
    global $wpdb, $wps_ic;
    $queue = $wps_ic->queue->get_hidden_restore_bulk_queue();

    if ($queue) {
      foreach ($queue as $attachment_ID => $values) {

        // Updating log
        $wps_ic->log->write_log('0', 'Hidden Bulk - Restore - Single Bulk - ' . $attachment_ID, '1', 'hidden_bulk_restore');
        $wps_ic->compress->hidden_restore($attachment_ID);

      }
    } else {
      delete_option('wps_ic_hidden_bulk_running');
    }
    die();
  }


  public function run_night_owl() {
    global $wpdb, $wps_ic;

    $start_no = microtime(true);
    ini_set('memory_limit', '2024M');
    ini_set('max_execution_time', WPS_IC_QUEUE_EXECUTION_TIME);

    $options = get_option(WPS_IC_OPTIONS);
    $settings        = get_option(WPS_IC_SETTINGS);

    if (!empty($_GET['reset'])) {
      delete_option('wps_ic_no_attachment');
    }

    $last_attachment = get_option('wps_ic_no_attachment');

    if (!empty($_GET['force_from'])) {
      update_option('wps_ic_no_attachment', $_GET['force_from']);
      $last_attachment = sanitize_text_field($_GET['force_from']);
    }

    if (empty($settings['autopilot']) || $settings['autopilot'] == '0' || empty($settings['otto']) || $settings['otto'] == 'off' || $settings['otto'] == 'on-upload') {
      return;
    }

    $check_credits = wp_remote_get(WPS_IC_APIURL . '?apikey=' . $options['api_key'] . '&check_credits=true', array('timeout' => '25', 'sslverify' => false));
    $credits       = 0;

    if (wp_remote_retrieve_response_code($check_credits) == 200) {
      $body = wp_remote_retrieve_body($check_credits);
      $body = json_decode($body, true);
      $credits = $body['data']['credits'];
    }

    if ($credits == 0 || empty($credits)) {
      /***
       * No Credits Warning mail
       */
      return;
    }

    /*
     * If not empty last_attachment, it has already been runned
     */
    $current_run = 0;
    $limit_per_run = 5;

    if ( ! empty($last_attachment)) {
      $filter = ' AND ID>' . $last_attachment . ' ';
      $attachments = $wpdb->get_results("SELECT ID FROM " . $wpdb->posts . " WHERE post_type='attachment' " . $filter . " ORDER BY ID ASC LIMIT 50");

    } else {
      /**
       * If empty lost_attachment, it is the first run, start from beggining
       */
      $attachments = $wpdb->get_results("SELECT ID FROM " . $wpdb->posts . " WHERE post_type='attachment' ORDER BY ID ASC LIMIT 50");
    }

    $last_attachment_ID = 0;
    $compress_queue = array();

    if ($attachments) {

      foreach ($attachments as $attachment) {

        $file_data = get_attached_file($attachment->ID);
        $type      = wp_check_filetype($file_data);

        $allowed_types         = array();
        $allowed_types['jpg']  = 'jpg';
        $allowed_types['jpeg'] = 'jpeg';
        $allowed_types['gif']  = 'gif';
        $allowed_types['png']  = 'png';

        if (!in_array(strtolower($type['ext']), $allowed_types)) {
          var_dump('Not allowed');
          $last_attachment_ID = $attachment->ID;
          #update_option('wps_ic_last_no_attachment', $attachment->ID);
          continue;
        }

        // Fetching various meta data
        $compress_data = get_post_meta($attachment->ID, 'wps_ic_data', true);
        if ( ! empty($compress_data) && ! empty($compress_data['new']) && $compress_data['new']['size'] != '') {
          var_dump('Compress not empty ' . $attachment->ID);
          $last_attachment_ID = $attachment->ID;
          #update_option('wps_ic_last_no_attachment', $attachment->ID);
          continue;
        }

        $current_run++;

        // Set compressing
        update_post_meta($attachment->ID, 'wps_ic_in_bulk', 'true');
        update_post_meta($attachment->ID, 'wps_ic_compressing', 'true');
        #update_option('wps_ic_last_no_attachment', $attachment->ID);
        $last_attachment_ID = $attachment->ID;
        var_dump('Tried: ' . $last_attachment_ID);

        // Add queue
        $wps_ic->queue->remove_queue($attachment->ID);
        $queue = $wps_ic->queue->add_queue($attachment->ID, 'hidden_compress_bulk');

        $compress_queue[] = $attachment->ID;
        if ($current_run>=$limit_per_run) {
          var_dump('Running queue: ' . $compress_queue);
          break;
        }
      }

      if (!empty($compress_queue) && $credits >= count($compress_queue)) {
        update_option('wps_ic_no_attachment', $last_attachment_ID);
        $wps_ic->compress->bulk(array('attachments' => $compress_queue));

        $end_no = microtime(true);
        wp_send_json_success('Last Att: ' . $last_attachment_ID . ' Took us ' . ($end_no-$start_no) . ' seconds');
      } else {
        update_option('wps_ic_no_attachment', $last_attachment_ID);
        wp_send_json_success('No credits for automatted Credits:' . $credits  . ' for ' . count($compress_queue));
      }


      die();
    } else {
      wp_send_json_success('Nothing to run');
    }
  }


  public function trigger_night_owl() {
    global $wpdb, $wps_ic;

    ini_set('memory_limit', '2024M');
    ini_set('max_execution_time', WPS_IC_QUEUE_EXECUTION_TIME);

    $options        = get_option(WPS_IC_OPTIONS);
    $settings        = get_option(WPS_IC_SETTINGS);
    $last_attachment = get_option('wps_ic_last_no_cron_attachment');

    if (empty($settings['autopilot']) || $settings['autopilot'] == '0' || empty($settings['otto']) || $settings['otto'] == 'off' || $settings['otto'] == 'on-upload') {
      return;
    }

    $check_credits = wp_remote_get(WPS_IC_APIURL . '?apikey=' . $options['api_key'] . '&check_credits=true', array('timeout' => '25', 'sslverify' => false));
    $credits       = 0;

    if (wp_remote_retrieve_response_code($check_credits) == 200) {
      $body = wp_remote_retrieve_body($check_credits);
      $body = json_decode($body, true);
      $credits = $body['data']['credits'];
    }

    if ($credits == 0 || empty($credits)) {
      /***
       * No Credits Warning mail
       */
      return;
    }


    if ( ! empty($last_attachment)) {
      $filter = ' AND ID>' . $last_attachment . ' ';
      $attachments = $wpdb->get_results("SELECT ID FROM " . $wpdb->posts . " WHERE post_type='attachment' " . $filter . " ORDER BY ID ASC LIMIT 50");

    } else {
      /**
       * If empty lost_attachment, it is the first run, start from beggining
       */
      $attachments = $wpdb->get_results("SELECT ID FROM " . $wpdb->posts . " WHERE post_type='attachment' ORDER BY ID ASC LIMIT 50");
    }

    $last_attachment_ID = 0;
    $compress_queue = array();
    $queued = 0;


    if ($attachments) {

      foreach ($attachments as $attachment) {

        $file_data = get_attached_file($attachment->ID);
        $type      = wp_check_filetype($file_data);

        $allowed_types         = array();
        $allowed_types['jpg']  = 'jpg';
        $allowed_types['jpeg'] = 'jpeg';
        $allowed_types['gif']  = 'gif';
        $allowed_types['png']  = 'png';

        /**
         * Not Allowed
         */
        if (!in_array(strtolower($type['ext']), $allowed_types)) {
          #update_option('wps_ic_last_no_attachment', $attachment->ID);
          $last_attachment_ID = $attachment->ID;
          continue;
        }

        /**
         * Already Compressed
         */
        $compress_data = get_post_meta($attachment->ID, 'wps_ic_data', true);
        if ( ! empty($compress_data) && ! empty($compress_data['new']) && $compress_data['new']['size'] != '') {
          #update_option('wps_ic_last_no_attachment', $attachment->ID);
          $last_attachment_ID = $attachment->ID;
          continue;
        }

        $queued ++;
        $attachment_array[] = $attachment->ID;

        #update_option('wps_ic_last_no_attachment', $attachment->ID);
        $last_attachment_ID = $attachment->ID;

        $wps_ic->queue->remove_queue($attachment->ID);
        $queue = $wps_ic->queue->add_queue($attachment->ID, 'hidden_compress_bulk');

        if ($queued >= WPS_IC_IMAGES_PER_QUEUE) {
          if ( ! empty($attachment_array) && count($attachment_array) > 0) {
            wp_schedule_single_event(time(), 'wps_ic_compress_single', array('attachments' => $attachment_array));
          }
        }

      }

      if ($queued <= WPS_IC_IMAGES_PER_QUEUE) {
        if ( ! empty($attachment_array) && count($attachment_array) > 0) {
          wp_schedule_single_event(time(), 'wps_ic_compress_single', array('attachments' => $attachment_array));
        }
      }

      update_option('wps_ic_last_no_cron_attachment', $last_attachment_ID);
    }
  }


  public static function valid_api_url($allow, $host, $url) {
    if ($host == WPS_IC_APIURL) {
      $allow = true;
    }

    return $allow;
  }


  public static function wp_ic_cron_timming($schedules) {
    $schedules['two_minutes'] = array(
      'interval' => 60 * 2,
      'display'  => __('Every two minutes')
    );

    $schedules['five_minutes'] = array(
      'interval' => 60 * 5,
      'display'  => __('Every five minutes')
    );

    $schedules['two_hours'] = array(
      'interval' => 60 * 60 * 2,
      'display'  => __('Every two hours')
    );

    $schedules['one_hour'] = array(
      'interval' => 60 * 60 * 1,
      'display'  => __('Every hour')
    );

    return $schedules;
  }


  /**
   * WP Init helper
   */
  public static function init() {
    $class = __CLASS__;
    new $class;
  }


  /**
   * Activation of the plugin
   */
  public static function activation() {

    $unlocks = get_option(WPS_IC_SETTINGS);
    $unlocks = $unlocks['unlocks'];

    // Setup Default Options
    $options = new wps_ic_options();
    $options->set_defaults();

    // Setup Database
    $database = new wps_ic_database();
    $database->create_tables();

    // Schedules
    add_filter('cron_schedules', array('wps_ic', 'wp_ic_cron_timming'));

    // Set pagination limit
    $users = get_users();
    if ($users) {
      foreach ($users as $user) {
        if (get_user_meta($user->ID, 'upload_per_page') >= 50) {
          update_user_meta($user->ID, 'upload_per_page', 30);
        }
      }
    }

    // Setup Stats
    update_option(WPS_IC_STATS_BYTES, 0);
    update_option(WPS_IC_STATS_FILES, 0);
    update_option(WPS_IC_STATS_AVG_REDUCTION, array());

    global $wpdb;
    $attachments = $wpdb->get_results("SELECT ID FROM " . $wpdb->posts . " WHERE post_type='attachment' AND post_status='inherit'", ARRAY_A);
    update_option('wps_ic_regen_thumbs', $attachments);

    if ( ! wp_next_scheduled('wps_ic_regen_thumbs')) {
      wp_schedule_single_event(time(), 'wps_ic_regen_thumbs');
    }

    // Run Compress From Queue
    #if ( ! wp_next_scheduled('wps_ic_compress_single')) {
    # wp_schedule_event(time(), 'two_minutes', 'wps_ic_compress_single');
    #}

  }


  /**
   * Deactivation of the plugin
   */
  public static function deactivation() {
    global $wpdb;

    delete_option('ic_set_recommended');
    delete_option('wps_ic_test_image');
    delete_option('wps_ic_affiliate_code');

    // Remove from active on API
    $options = get_option(WPS_IC_OPTIONS);
    $site    = site_url();
    $apikey  = $options['api_key'];

    // Setup URI
    $uri = WPS_IC_KEYSURL;
    $uri .= '?action=disconnect';
    $uri .= '&apikey=' . $apikey;
    $uri .= '&site=' . urlencode($site);

    // Verify API Key is our database and user has is confirmed getresponse
    $get = wp_remote_get($uri, array('timeout' => 20, 'sslverify' => false));

    $options['api_key']      = '';
    $options['response_key'] = '';
    $options['orp']          = '';
    update_option(WPS_IC_OPTIONS, $options);

    // Setup Database
    delete_option(WPS_IC_CRON_QUEUE);
    delete_option(WPS_IC_BULK_PROCESS);
    delete_option(WPS_IC_BULK);
    delete_option(WPS_IC_SAVINGS);
    delete_option(WPS_IC_AVG_SAVINGS);
    delete_option('wps_ic_last_no_attachment');

    wp_unschedule_event(time(), 'wps_ic_regen_thumbs_by_array');
    wp_unschedule_event(time(), 'wps_ic_regen_thumbs');
    wp_unschedule_event(time(), 'wps_ic_rebuild_cdn');
    wp_unschedule_event(time(), 'wps_ic_regenerate_thumbnails');
    wp_unschedule_event(time(), 'wps_ic_wake_up_cron');
    wp_unschedule_event(time(), 'wps_ic_compress_single');
    wp_unschedule_event(time(), 'wps_ic_upload_image');
    wp_unschedule_event(time(), 'wps_ic_compress');
    wp_unschedule_event(time(), 'wps_ic_daily_thumbnail');
    wp_unschedule_event(time(), 'wps_ic_single_thumbnail');
    wp_unschedule_event(time(), 'wps_ic_fetch_thumbnails');
    wp_unschedule_event(time(), 'wps_ic_restore_thumbnails');
    wp_unschedule_event(time(), 'wps_ic_owl');
  }


  public function change_limits() {
    if ( ! defined('WP_MEMORY_LIMIT')) {
      define('WP_MEMORY_LIMIT', '512M');
    }
    ini_set('memory_limit', '1024M');
    ini_set('max_execution_time', '3600');
  }


  public static function deactivate_script() {
    wp_enqueue_style('wp-pointer');
    wp_enqueue_script('wp-pointer');
    wp_enqueue_script('utils'); // for user settings
    ?>
    <script type="text/javascript">

        jQuery(document).ready(function ($) {
            var row = $('tr[data-slug="wp-compress-image-optimizer"]');
            var span_deactivate = $('span.deactivate', row);
            var link = $('a', span_deactivate);
            var pointer = '';

            $('#wps-ic-leave-active').live('click', function (e) {
                e.preventDefault();
                $(pointer).pointer('close');
                return false;
            });

            $(link).on('click', function (e) {
                e.preventDefault();

                pointer = $(this).pointer({
                    content: '<h3>Deactivating plugin may cause...</h3><p><ul style="padding:0px 15px;margin:0px 10px;list-style:disc;">' +
                        '<li>New images loading slow for visitors</li>' +
                        '<li>Backups removed from our cloud</li>' +
                        '<li>Competitors charging you for thumbnails</li>' +
                        '<li>Our team crying that you left... :(</li>' +
                        '</ul><p>Your images will stay in the current state upon deactivation.</p><div style="padding:15px;"><a id="wps-ic-leave-active" class="button button-primary" href="#">Leave active</a> <a id="everything" class="button ' +
                        'button-secondary" ' +
                        'href="' + $(link).attr
                        ('href') +
                        '">Deactivate Anyway</a></div></p>',
                    position: {
                        my: 'left top',
                        at: 'left top',
                        offset: '0 0'
                    },
                    close: function () {
                        //
                    }
                }).pointer('open');

                $('.wp-pointer-buttons').hide();

                return false;
            });
        });
    </script><?php
  }

}


add_action('init', array('wps_ic', 'init'));
register_activation_hook(__FILE__, array('wps_ic', 'activation'));
register_deactivation_hook(__FILE__, array('wps_ic', 'deactivation'));